// src/ui/utils.ts
function isPartialFrameStackItem(stackItem) {
  return stackItem.status === "done" && stackItem.frameResult.status === "failure" && (isValidPartialFrameV1(stackItem.frameResult) || isValidPartialFrameV2(stackItem.frameResult));
}
function getErrorMessageFromFramesStackItem(item) {
  if (item.status === "message") {
    return item.message;
  }
  if (item.requestError instanceof Error) {
    return item.requestError.message;
  }
  return "An error occurred";
}
function isValidPartialFrameV1(value) {
  if (value.specification !== "farcaster" && value.specification !== "openframes") {
    return false;
  }
  return !!value.frame.image && !!value.frame.buttons && value.frame.buttons.length > 0;
}
function isValidPartialFrameV2(value) {
  if (value.specification !== "farcaster_v2") {
    return false;
  }
  return !!value.frame.imageUrl && !!value.frame.button && !!value.frame.button.title && !!value.frame.button.action && !!value.frame.button.action.url;
}
function isValidPartialFrame(frameResult) {
  return isValidPartialFrameV1(frameResult) || isValidPartialFrameV2(frameResult);
}
function isValidFrameV1(value) {
  return value.status === "success" && (value.specification === "farcaster" || value.specification === "openframes");
}
function isValidFrameV2(value) {
  return value.status === "success" && value.specification === "farcaster_v2";
}
function isValidFrame(value) {
  return isValidFrameV1(value) || isValidFrameV2(value);
}

export {
  isPartialFrameStackItem,
  getErrorMessageFromFramesStackItem,
  isValidPartialFrameV1,
  isValidPartialFrameV2,
  isValidPartialFrame,
  isValidFrameV1,
  isValidFrameV2,
  isValidFrame
};
