import {
  fetchProxied
} from "./chunk-HWT6S3D6.js";
import {
  useFreshRef
} from "./chunk-KTZM67PO.js";
import {
  isParseFramesWithReportsResult
} from "./chunk-RVPSX5WL.js";

// src/frame-app/use-fetch-frame-app.ts
import { useCallback, useEffect, useRef, useState } from "react";
var defaultFetchFunction = (...args) => fetch(...args);
function useFetchFrameApp({
  fetchFn,
  source,
  proxyUrl
}) {
  const abortControllerRef = useRef(null);
  const fetchRef = useFreshRef(fetchFn ?? defaultFetchFunction);
  const [state, setState] = useState(() => {
    if (typeof source === "string" || source instanceof URL) {
      return {
        status: "pending"
      };
    }
    return {
      status: "success",
      frame: source,
      source
    };
  });
  const fetchFrame = useCallback(
    (sourceUrl) => {
      abortControllerRef.current?.abort();
      const abortController = new AbortController();
      abortControllerRef.current = abortController;
      Promise.resolve(sourceUrl).then(async (url) => {
        setState({
          status: "pending"
        });
        const responseOrError = await fetchProxied({
          fetchFn: fetchRef.current,
          url: url.toString(),
          parseFarcasterManifest: true,
          proxyUrl,
          signal: abortController.signal
        });
        if (responseOrError instanceof Error) {
          throw responseOrError;
        }
        if (!responseOrError.ok) {
          throw new Error(
            `Failed to fetch frame, server returned status ${responseOrError.status}`
          );
        }
        const data = await responseOrError.json();
        if (!isParseFramesWithReportsResult(data)) {
          throw new Error(
            "Invalid response, expected parse result, make sure you are using @frames.js/render proxy"
          );
        }
        if (abortController.signal.aborted) {
          return;
        }
        setState({
          status: "success",
          frame: data.farcaster_v2,
          source: sourceUrl
        });
      }).catch((e) => {
        if (abortController.signal.aborted) {
          return;
        }
        setState({
          status: "error",
          error: e instanceof Error ? e : new Error(String(e))
        });
      });
      return () => {
        abortController.abort();
      };
    },
    [fetchRef, proxyUrl]
  );
  useEffect(() => {
    if (typeof source === "string" || source instanceof URL) {
      return fetchFrame(source);
    }
    setState((val) => {
      if (val.status === "success" && val.source === source) {
        return val;
      }
      return {
        status: "success",
        frame: source,
        source
      };
    });
  }, [source, fetchFrame]);
  return state;
}

export {
  useFetchFrameApp
};
