// src/frame-app/use-resolve-client.ts
import { useCallback, useEffect, useRef, useState } from "react";
function useResolveClient({
  client
}) {
  const abortControllerRef = useRef(null);
  const [state, setState] = useState(() => {
    if (typeof client !== "function") {
      return {
        status: "success",
        client
      };
    }
    return {
      status: "pending"
    };
  });
  const resolveClient = useCallback((resolve) => {
    abortControllerRef.current?.abort();
    const abortController = new AbortController();
    abortControllerRef.current = abortController;
    Promise.resolve().then(async () => {
      setState({
        status: "pending"
      });
      const resolvedClient = await resolve({
        signal: abortController.signal
      });
      if (abortController.signal.aborted) {
        return;
      }
      setState({
        status: "success",
        client: resolvedClient
      });
    }).catch((e) => {
      if (abortController.signal.aborted) {
        return;
      }
      setState({
        status: "error",
        error: e instanceof Error ? e : new Error(String(e))
      });
    });
    return () => {
      abortController.abort();
    };
  }, []);
  useEffect(() => {
    if (typeof client !== "function") {
      setState((prevState) => {
        if (prevState.status === "success" && prevState.client !== client) {
          return {
            status: "success",
            client
          };
        }
        return prevState;
      });
    } else {
      resolveClient(client);
    }
  }, [client, resolveClient]);
  return state;
}

export {
  useResolveClient
};
