import {
  isSpecificationValid
} from "./chunk-F5WB6H3H.js";

// src/next/GET.tsx
import { parseFramesWithReports } from "frames.js/parseFramesWithReports";
import { getFrame } from "frames.js";
async function GET(request) {
  try {
    const searchParams = "nextUrl" in request ? request.nextUrl.searchParams : new URL(request.url).searchParams;
    const url = searchParams.get("url");
    const specification = searchParams.get("specification") ?? "farcaster";
    const parseFarcasterManifest = searchParams.get("parseFarcasterManifest") === "true";
    const strictlyParseFarcasterV2 = searchParams.get("looseFarcasterV2Parsing") !== "true";
    const multiSpecificationEnabled = searchParams.get("multispecification") === "true";
    if (!url) {
      return Response.json({ message: "Invalid URL" }, {
        status: 400
      });
    }
    const urlRes = await fetch(url);
    const html = await urlRes.text();
    if (multiSpecificationEnabled) {
      const result2 = await parseFramesWithReports(
        {
          html,
          fallbackPostUrl: url,
          frameUrl: url,
          parseSettings: {
            farcaster_v2: {
              parseManifest: parseFarcasterManifest,
              strict: strictlyParseFarcasterV2
            }
          }
        }
      );
      return Response.json(result2);
    }
    if (!isSpecificationValid(specification)) {
      return Response.json(
        { message: "Invalid specification" },
        {
          status: 400
        }
      );
    }
    const result = await getFrame({
      htmlString: html,
      frameUrl: url,
      url,
      specification
    });
    return Response.json(result);
  } catch (err) {
    console.error(err);
    return Response.json({ message: String(err) }, {
      status: 500
    });
  }
}

export {
  GET
};
