// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}
function tryCall(fn) {
  try {
    return fn();
  } catch (e) {
    if (e instanceof Error) {
      return e;
    }
    return new TypeError("Unexpected error, check the console for details");
  }
}
function isParseFramesWithReportsResult(value) {
  return typeof value === "object" && value !== null && "openframes" in value && "farcaster" in value;
}
function isParseResult(value) {
  return typeof value === "object" && value !== null && "status" in value && !("openframes" in value) && !("farcaster" in value);
}
function isPartialFrame(value) {
  return value.status === "failure" && !!value.frame.image && !!value.frame.buttons && value.frame.buttons.length > 0;
}
function isComposerFormActionResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "form";
}
function isCastActionFrameResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "frame" && "frameUrl" in response && typeof response.frameUrl === "string";
}
function isCastActionMessageResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "message" && "message" in response && typeof response.message === "string";
}
function mergeSearchParamsToUrl(url, ...searchParams) {
  const temporaryDomain = "temporary-for-parsing-purposes.tld";
  const parsedProxyUrl = new URL(url, `http://${temporaryDomain}`);
  searchParams.forEach((params) => {
    params.forEach((value, key) => {
      parsedProxyUrl.searchParams.set(key, value);
    });
  });
  return parsedProxyUrl.hostname === temporaryDomain ? `${parsedProxyUrl.pathname}${parsedProxyUrl.search}` : parsedProxyUrl.toString();
}

export {
  tryCallAsync,
  tryCall,
  isParseFramesWithReportsResult,
  isParseResult,
  isPartialFrame,
  isComposerFormActionResponse,
  isCastActionFrameResponse,
  isCastActionMessageResponse,
  mergeSearchParamsToUrl
};
