import { FrameActionMessage, CastId } from '@farcaster/core';
import { c as SignFrameActionFunc, F as FrameActionBodyPayload, a as FrameContext, S as SignerStateActionContext, b as SignedFrameAction } from '../types-CRf9XqD9.js';
import { SignComposerActionFunction, SignCastActionFunction, SignerStateComposerActionContext, SignerStateCastActionContext } from '../unstable-types.js';
import { FarcasterSigner } from './signers.js';
import { FarcasterFrameContext } from './types.js';
import 'frames.js';
import 'react';
import 'frames.js/frame-parsers';
import 'frames.js/types';

/**
 * Creates a singer request payload to fetch composer action url.
 */
declare const signComposerAction: SignComposerActionFunction;
declare const signCastAction: SignCastActionFunction;
/** Creates a frame action for use with `useFrame` and a proxy */
declare const signFrameAction: SignFrameActionFunc<FarcasterSigner, FrameActionBodyPayload, FarcasterFrameContext>;
declare function createComposerActionMessageWithSignerKey(signerKey: string, { fid, state, url }: SignerStateComposerActionContext): Promise<{
    message: FrameActionMessage;
    trustedBytes: string;
}>;
declare function createCastActionMessageWithSignerKey(signerKey: string, { fid, castId, postUrl }: SignerStateCastActionContext): Promise<{
    message: FrameActionMessage;
    trustedBytes: string;
}>;
declare function createFrameActionMessageWithSignerKey(signerKey: string, { fid, url, buttonIndex, castId, inputText, state, address, transactionId, }: {
    fid: number;
    url: Uint8Array;
    buttonIndex: number;
    inputText: Uint8Array | undefined;
    castId: CastId;
    state: Uint8Array | undefined;
    address: Uint8Array | undefined;
    transactionId: Uint8Array | undefined;
}): Promise<{
    message: null;
    trustedBytes: null;
} | {
    message: FrameActionMessage | null;
    trustedBytes: string;
}>;
/**
 * Used to create an unsigned frame action when signer is not defined
 */
declare function unsignedFrameAction<TSignerStorageType = Record<string, unknown>, TFrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload, TFrameContextType extends FrameContext = FarcasterFrameContext>(actionContext: SignerStateActionContext<TSignerStorageType, TFrameContextType>): Promise<SignedFrameAction<TFrameActionBodyType>>;

export { createCastActionMessageWithSignerKey, createComposerActionMessageWithSignerKey, createFrameActionMessageWithSignerKey, signCastAction, signComposerAction, signFrameAction, unsignedFrameAction };
