"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/farcaster/index.ts
var farcaster_exports = {};
__export(farcaster_exports, {
  attribution: () => attribution,
  createCastActionMessageWithSignerKey: () => createCastActionMessageWithSignerKey,
  createComposerActionMessageWithSignerKey: () => createComposerActionMessageWithSignerKey,
  createFrameActionMessageWithSignerKey: () => createFrameActionMessageWithSignerKey,
  mockFarcasterSigner: () => mockFarcasterSigner,
  signCastAction: () => signCastAction,
  signComposerAction: () => signComposerAction,
  signFrameAction: () => signFrameAction,
  unsignedFrameAction: () => unsignedFrameAction
});
module.exports = __toCommonJS(farcaster_exports);

// src/farcaster/frames.tsx
var import_core = require("@farcaster/core");
var import_viem = require("viem");

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}

// src/farcaster/frames.tsx
var signComposerAction = async function signComposerAction2(signerPrivateKey, actionContext) {
  const messageOrError = await tryCallAsync(
    () => createComposerActionMessageWithSignerKey(signerPrivateKey, actionContext)
  );
  if (messageOrError instanceof Error) {
    throw messageOrError;
  }
  const { message, trustedBytes } = messageOrError;
  return {
    untrustedData: {
      buttonIndex: message.data.frameActionBody.buttonIndex,
      fid: message.data.fid,
      messageHash: (0, import_viem.bytesToHex)(message.hash),
      network: import_core.FarcasterNetwork.MAINNET,
      state: Buffer.from(message.data.frameActionBody.state).toString(),
      timestamp: message.data.timestamp,
      url: Buffer.from(message.data.frameActionBody.url).toString()
    },
    trustedData: {
      messageBytes: trustedBytes
    }
  };
};
var signCastAction = async function signCastAction2(signerPrivateKey, actionContext) {
  const messageOrError = await tryCallAsync(
    () => createCastActionMessageWithSignerKey(signerPrivateKey, actionContext)
  );
  if (messageOrError instanceof Error) {
    throw messageOrError;
  }
  const { message, trustedBytes } = messageOrError;
  return {
    untrustedData: {
      buttonIndex: message.data.frameActionBody.buttonIndex,
      fid: message.data.fid,
      messageHash: (0, import_viem.bytesToHex)(message.hash),
      network: import_core.FarcasterNetwork.MAINNET,
      timestamp: message.data.timestamp,
      url: Buffer.from(message.data.frameActionBody.url).toString()
    },
    trustedData: {
      messageBytes: trustedBytes
    }
  };
};
var signFrameAction = async (actionContext) => {
  const {
    frameButton,
    signer,
    buttonIndex,
    inputText,
    frameContext,
    state,
    url,
    target
  } = actionContext;
  if (!signer || signer.status === "pending_approval") {
    throw new Error("Missing signer fid to sign message");
  }
  const { message, trustedBytes } = await createFrameActionMessageWithSignerKey(
    signer.privateKey,
    {
      fid: signer.fid,
      buttonIndex,
      castId: {
        fid: frameContext.castId.fid,
        hash: (0, import_viem.hexToBytes)(frameContext.castId.hash)
      },
      state: state !== void 0 ? Buffer.from(state) : void 0,
      url: Buffer.from(url),
      // it seems the message in hubs actually requires a value here.
      inputText: inputText !== void 0 ? Buffer.from(inputText) : void 0,
      address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? (0, import_viem.hexToBytes)(actionContext.address) : void 0,
      transactionId: actionContext.type === "tx-post" ? (0, import_viem.hexToBytes)(actionContext.transactionId) : void 0
    }
  );
  if (!message) {
    throw new Error("hub error");
  }
  const searchParams = new URLSearchParams({
    postType: actionContext.type === "tx-post" ? "post" : frameButton.action,
    postUrl: target ?? ""
  });
  return {
    searchParams,
    body: {
      untrustedData: {
        fid: signer.fid,
        url,
        messageHash: `0x${Buffer.from(message.hash).toString("hex")}`,
        timestamp: (/* @__PURE__ */ new Date()).getTime(),
        network: 1,
        buttonIndex: Number(message.data.frameActionBody.buttonIndex),
        castId: {
          fid: frameContext.castId.fid,
          hash: frameContext.castId.hash
        },
        inputText,
        state,
        address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? actionContext.address : void 0,
        transactionId: actionContext.type === "tx-post" ? actionContext.transactionId : void 0
      },
      trustedData: {
        messageBytes: trustedBytes
      }
    }
  };
};
async function createComposerActionMessageWithSignerKey(signerKey, { fid, state, url }) {
  const signer = new import_core.NobleEd25519Signer(Buffer.from(signerKey.slice(2), "hex"));
  const messageDataOptions = {
    fid,
    network: import_core.FarcasterNetwork.MAINNET
  };
  const message = await (0, import_core.makeFrameAction)(
    import_core.FrameActionBody.create({
      url: Buffer.from(url),
      buttonIndex: 1,
      state: Buffer.from(encodeURIComponent(JSON.stringify({ cast: state })))
    }),
    messageDataOptions,
    signer
  );
  if (message.isErr()) {
    throw message.error;
  }
  const messageData = message.value;
  const trustedBytes = Buffer.from(
    import_core.Message.encode(message._unsafeUnwrap()).finish()
  ).toString("hex");
  return { message: messageData, trustedBytes };
}
async function createCastActionMessageWithSignerKey(signerKey, { fid, castId, postUrl }) {
  const signer = new import_core.NobleEd25519Signer(Buffer.from(signerKey.slice(2), "hex"));
  const messageDataOptions = {
    fid,
    network: import_core.FarcasterNetwork.MAINNET
  };
  const message = await (0, import_core.makeFrameAction)(
    import_core.FrameActionBody.create({
      url: Buffer.from(postUrl),
      buttonIndex: 1,
      castId: {
        fid: castId.fid,
        hash: (0, import_viem.hexToBytes)(castId.hash)
      }
    }),
    messageDataOptions,
    signer
  );
  if (message.isErr()) {
    throw message.error;
  }
  const messageData = message.value;
  const trustedBytes = Buffer.from(
    import_core.Message.encode(message._unsafeUnwrap()).finish()
  ).toString("hex");
  return { message: messageData, trustedBytes };
}
async function createFrameActionMessageWithSignerKey(signerKey, {
  fid,
  url,
  buttonIndex,
  castId,
  inputText,
  state,
  address,
  transactionId
}) {
  const signer = new import_core.NobleEd25519Signer(Buffer.from(signerKey.slice(2), "hex"));
  const messageDataOptions = {
    fid,
    network: import_core.FarcasterNetwork.MAINNET
  };
  const message = await (0, import_core.makeFrameAction)(
    import_core.FrameActionBody.create({
      url,
      buttonIndex,
      castId,
      state,
      inputText: inputText !== void 0 ? Buffer.from(inputText) : void 0,
      address,
      transactionId
    }),
    messageDataOptions,
    signer
  );
  if (message.isErr()) {
    return { message: null, trustedBytes: null };
  }
  const trustedBytes = Buffer.from(
    import_core.Message.encode(message._unsafeUnwrap()).finish()
  ).toString("hex");
  return { message: message.unwrapOr(null), trustedBytes };
}
function isFarcasterFrameContext(frameContext) {
  return "castId" in frameContext;
}
async function unsignedFrameAction(actionContext) {
  const {
    frameButton,
    target,
    frameContext,
    url,
    buttonIndex,
    state,
    inputText
  } = actionContext;
  const searchParams = new URLSearchParams({
    postType: actionContext.type === "tx-post" ? "post" : frameButton.action,
    postUrl: target ?? ""
  });
  return Promise.resolve({
    searchParams,
    body: {
      untrustedData: {
        url,
        ...isFarcasterFrameContext(frameContext) ? {
          castId: {
            fid: frameContext.castId.fid,
            hash: frameContext.castId.hash
          },
          network: 1,
          timestamp: (0, import_core.getFarcasterTime)()._unsafeUnwrap()
        } : {
          timestamp: Date.now()
        },
        buttonIndex,
        state,
        inputText,
        address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? actionContext.address : void 0,
        transactionId: actionContext.type === "tx-post" ? actionContext.transactionId : void 0
      },
      trustedData: {
        messageBytes: Buffer.from("").toString("hex")
      }
    }
  });
}

// src/farcaster/signers.tsx
var mockFarcasterSigner = {
  fid: 1,
  status: "approved",
  publicKey: "0x00000000000000000000000000000000000000000000000000000000000000000",
  privateKey: "0x00000000000000000000000000000000000000000000000000000000000000000"
};

// src/farcaster/attribution.ts
var import_viem2 = require("viem");
function attribution(fid) {
  return (0, import_viem2.encodePacked)(["bytes1", "uint32"], ["0xfc", fid]);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  attribution,
  createCastActionMessageWithSignerKey,
  createComposerActionMessageWithSignerKey,
  createFrameActionMessageWithSignerKey,
  mockFarcasterSigner,
  signCastAction,
  signComposerAction,
  signFrameAction,
  unsignedFrameAction
});
