import {
  useFrameApp
} from "../chunk-UMNPS2C4.js";
import "../chunk-BXR2F7XL.js";
import "../chunk-DIJQCH5I.js";
import {
  useDebugLog
} from "../chunk-TLOKKM4F.js";
import "../chunk-HWT6S3D6.js";
import {
  useFreshRef
} from "../chunk-KTZM67PO.js";
import {
  assertNever
} from "../chunk-YAAIMQMX.js";
import "../chunk-LOKN5BYP.js";
import "../chunk-RVPSX5WL.js";
import "../chunk-6C3VEZWH.js";

// src/frame-app/iframe.ts
import {
  exposeToEndpoint,
  createIframeEndpoint
} from "@farcaster/frame-host";
import { useEffect, useMemo, useRef } from "react";
function useFrameAppInIframe(options) {
  const providerRef = useFreshRef(options.provider);
  const debugRef = useFreshRef(options.debug ?? false);
  const frameApp = useFrameApp(options);
  const iframeRef = useRef(null);
  const endpointRef = useRef(null);
  const emitterRef = useRef(null);
  const logDebug = useDebugLog(
    "@frames.js/render/frame-app/iframe",
    debugRef.current
  );
  const emitter = useMemo(() => {
    return {
      emit(...args) {
        if (emitterRef.current) {
          emitterRef.current.emit(...args);
        } else {
          logDebug(
            "endpoint not available, probably not initialized yet, skipping emit",
            args
          );
        }
      },
      emitEthProvider(...args) {
        if (emitterRef.current) {
          emitterRef.current.emitEthProvider(
            ...args
          );
        } else {
          logDebug(
            "endpoint not available, probably not initialized yet, skipping emitEthProvider",
            args
          );
        }
      }
    };
  }, [logDebug]);
  const iframeFrameApp = useMemo(() => {
    switch (frameApp.status) {
      case "error":
      case "pending":
        return frameApp;
      case "success": {
        const frameUrl = frameApp.frame.frame.button?.action?.url;
        if (!frameUrl) {
          return {
            status: "error",
            error: new Error(
              "Frame URL is not provided, please check button.action.url"
            )
          };
        }
        const frameOrigin = new URL(frameUrl).origin;
        return {
          ...frameApp,
          status: "success",
          frameOrigin,
          iframeProps: {
            src: frameUrl,
            ref: iframeRef
          },
          emitter
        };
      }
      default:
        assertNever(frameApp);
    }
  }, [frameApp, emitter]);
  useEffect(() => {
    if (iframeFrameApp.status !== "success") {
      return;
    }
    const iframe = iframeRef.current;
    if (!iframe) {
      logDebug("iframe ref not available");
      return;
    }
    const frameUrl = iframeFrameApp.iframeProps.src;
    let frameOrigin = "";
    if (!frameUrl) {
      logDebug("frame URL not available, using empty string");
    } else {
      frameOrigin = new URL(frameUrl).origin;
    }
    const endpoint = createIframeEndpoint({
      iframe,
      targetOrigin: frameOrigin,
      debug: debugRef.current
    });
    const cleanup = exposeToEndpoint({
      endpoint,
      frameOrigin,
      sdk: iframeFrameApp.sdk(endpoint),
      debug: debugRef.current,
      ethProvider: providerRef.current
    });
    endpointRef.current = endpoint;
    emitterRef.current = iframeFrameApp.getEmitter(endpoint);
    logDebug("iframe endpoint created");
    return () => {
      logDebug("iframe unmounted, cleaning up");
      endpointRef.current = null;
      iframeRef.current = null;
      emitterRef.current = null;
      cleanup();
    };
  }, [iframeFrameApp, logDebug, debugRef, providerRef]);
  return iframeFrameApp;
}
export {
  useFrameAppInIframe
};
