"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from2, except, desc) => {
  if (from2 && typeof from2 === "object" || typeof from2 === "function") {
    for (let key of __getOwnPropNames(from2))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from2[key], enumerable: !(desc = __getOwnPropDesc(from2, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/frame-app/provider/wagmi.ts
var wagmi_exports = {};
__export(wagmi_exports, {
  useWagmiProvider: () => useWagmiProvider
});
module.exports = __toCommonJS(wagmi_exports);
var import_Provider = require("ox/Provider");
var import_react3 = require("react");
var import_wagmi = require("wagmi");

// src/hooks/use-fresh-ref.ts
var import_react = require("react");
function useFreshRef(value) {
  const ref = (0, import_react.useRef)(value);
  ref.current = value;
  return ref;
}

// src/hooks/use-debug-log.ts
var import_react2 = require("react");
function useDebugLog(prefix, enabled) {
  const enabledRef = useFreshRef(enabled);
  const prefixRef = useFreshRef(prefix);
  return (0, import_react2.useCallback)(
    (...args) => {
      if (!enabledRef.current) {
        return;
      }
      console.debug(prefixRef.current, ...args);
    },
    [enabledRef, prefixRef]
  );
}

// src/frame-app/provider/helpers.ts
function isSendTransactionRpcRequest(request) {
  return request.method === "eth_sendTransaction";
}
function isSignMessageRpcRequest(request) {
  return request.method === "personal_sign";
}
function isSignTypedDataRpcRequest(request) {
  return request.method === "eth_signTypedData_v4";
}
function isEIP1193Provider(provider) {
  return typeof provider === "object" && provider !== null && "request" in provider && typeof provider.request === "function" && "on" in provider && typeof provider.on === "function" && "removeListener" in provider && typeof provider.removeListener === "function";
}

// src/frame-app/provider/wagmi.ts
function useWagmiProvider({
  debug = false
} = {}) {
  const logDebug = useDebugLog(
    "@frames.js/render/frame-app/provider/wagmi",
    debug
  );
  const account = (0, import_wagmi.useAccount)();
  const accountRef = useFreshRef(account);
  const emitterRef = (0, import_react3.useRef)(null);
  const providerRef = (0, import_react3.useRef)(null);
  const onSendTransactionRequestRef = (0, import_react3.useRef)(null);
  const onSignMessageRequestRef = (0, import_react3.useRef)(
    null
  );
  const onSignTypedDataRequestRef = (0, import_react3.useRef)(null);
  if (!providerRef.current) {
    emitterRef.current = (0, import_Provider.createEmitter)();
    providerRef.current = {
      ...(0, import_Provider.from)({
        ...emitterRef.current,
        async request(parameters) {
          const connector = accountRef.current.connector;
          if (!connector) {
            throw new Error(
              "No connector found, make sure you have wallet connected."
            );
          }
          const provider = await connector.getProvider();
          if (!isEIP1193Provider(provider)) {
            throw new Error(
              "Provider is not EIP-1193 compatible, make sure you have wallet connected."
            );
          }
          logDebug("sdk.ethProviderRequest() called", parameters);
          let isApproved = true;
          if (isSendTransactionRpcRequest(parameters)) {
            logDebug("sendTransaction request", parameters);
            isApproved = onSendTransactionRequestRef.current ? await onSendTransactionRequestRef.current(parameters) : true;
          } else if (isSignTypedDataRpcRequest(parameters)) {
            logDebug("signTypedData request", parameters);
            isApproved = onSignTypedDataRequestRef.current ? await onSignTypedDataRequestRef.current(parameters) : true;
          } else if (isSignMessageRpcRequest(parameters)) {
            logDebug("signMessage request", parameters);
            isApproved = onSignMessageRequestRef.current ? await onSignMessageRequestRef.current(parameters) : true;
          }
          if (!isApproved) {
            throw new import_Provider.UserRejectedRequestError(
              new Error("User rejected request")
            );
          }
          return provider.request(
            parameters
          );
        }
      }),
      setEventHandlers(handlers) {
        onSendTransactionRequestRef.current = handlers.onSendTransactionRequest;
        onSignMessageRequestRef.current = handlers.onSignMessageRequest;
        onSignTypedDataRequestRef.current = handlers.onSignTypedDataRequest;
      }
    };
  }
  (0, import_react3.useEffect)(() => {
    if (account.status !== "connected") {
      return;
    }
    const connector = account.connector;
    let cleanup = () => {
    };
    void connector.getProvider().then((provider) => {
      if (!isEIP1193Provider(provider)) {
        return;
      }
      if (!emitterRef.current) {
        return;
      }
      const accountsChanged = (...args) => {
        emitterRef.current?.emit("accountsChanged", ...args);
      };
      const chainChanged = (...args) => {
        emitterRef.current?.emit("chainChanged", ...args);
      };
      const connect = (...args) => {
        emitterRef.current?.emit("connect", ...args);
      };
      const disconnect = (...args) => {
        emitterRef.current?.emit("disconnect", ...args);
      };
      const message = (...args) => {
        emitterRef.current?.emit("message", ...args);
      };
      provider.on("accountsChanged", accountsChanged);
      provider.on("chainChanged", chainChanged);
      provider.on("connect", connect);
      provider.on("disconnect", disconnect);
      provider.on("message", message);
      cleanup = () => {
        provider.removeListener("accountsChanged", accountsChanged);
        provider.removeListener("chainChanged", chainChanged);
        provider.removeListener("connect", connect);
        provider.removeListener("disconnect", disconnect);
        provider.removeListener("message", message);
      };
    });
    return () => {
      cleanup();
    };
  }, [account]);
  return providerRef.current;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useWagmiProvider
});
