"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/frame-app/use-fetch-frame-app.ts
var use_fetch_frame_app_exports = {};
__export(use_fetch_frame_app_exports, {
  useFetchFrameApp: () => useFetchFrameApp
});
module.exports = __toCommonJS(use_fetch_frame_app_exports);
var import_react2 = require("react");

// src/hooks/use-fresh-ref.ts
var import_react = require("react");
function useFreshRef(value) {
  const ref = (0, import_react.useRef)(value);
  ref.current = value;
  return ref;
}

// src/unstable-use-fetch-frame.ts
var import_viem = require("viem");

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}
function isParseFramesWithReportsResult(value) {
  return typeof value === "object" && value !== null && "openframes" in value && "farcaster" in value;
}

// src/unstable-use-fetch-frame.ts
function proxyUrlAndSearchParamsToUrl(proxyUrl, ...searchParams) {
  const temporaryDomain = "temporary-for-parsing-purposes.tld";
  const parsedProxyUrl = new URL(proxyUrl, `http://${temporaryDomain}`);
  searchParams.forEach((params) => {
    params.forEach((value, key) => {
      parsedProxyUrl.searchParams.set(key, value);
    });
  });
  return parsedProxyUrl.hostname === temporaryDomain ? `${parsedProxyUrl.pathname}${parsedProxyUrl.search}` : parsedProxyUrl.toString();
}
async function fetchProxied(params) {
  const searchParams = new URLSearchParams({
    multispecification: "true"
  });
  if ("frameAction" in params) {
    const proxyUrl2 = proxyUrlAndSearchParamsToUrl(
      params.proxyUrl,
      searchParams,
      params.frameAction.searchParams
    );
    return tryCallAsync(
      () => params.fetchFn(proxyUrl2, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...params.extraRequestPayload,
          ...params.frameAction.body
        })
      })
    );
  }
  if (params.parseFarcasterManifest) {
    searchParams.set("parseFarcasterManifest", "true");
  }
  searchParams.set("url", params.url);
  const proxyUrl = proxyUrlAndSearchParamsToUrl(params.proxyUrl, searchParams);
  return tryCallAsync(
    () => params.fetchFn(proxyUrl, { method: "GET", signal: params.signal })
  );
}

// src/frame-app/use-fetch-frame-app.ts
var defaultFetchFunction = (...args) => fetch(...args);
function useFetchFrameApp({
  fetchFn,
  source,
  proxyUrl
}) {
  const abortControllerRef = (0, import_react2.useRef)(null);
  const fetchRef = useFreshRef(fetchFn ?? defaultFetchFunction);
  const [state, setState] = (0, import_react2.useState)(() => {
    if (typeof source === "string" || source instanceof URL) {
      return {
        status: "pending"
      };
    }
    return {
      status: "success",
      frame: source,
      source
    };
  });
  const fetchFrame = (0, import_react2.useCallback)(
    (sourceUrl) => {
      abortControllerRef.current?.abort();
      const abortController = new AbortController();
      abortControllerRef.current = abortController;
      Promise.resolve(sourceUrl).then(async (url) => {
        setState({
          status: "pending"
        });
        const responseOrError = await fetchProxied({
          fetchFn: fetchRef.current,
          url: url.toString(),
          parseFarcasterManifest: true,
          proxyUrl,
          signal: abortController.signal
        });
        if (responseOrError instanceof Error) {
          throw responseOrError;
        }
        if (!responseOrError.ok) {
          throw new Error(
            `Failed to fetch frame, server returned status ${responseOrError.status}`
          );
        }
        const data = await responseOrError.json();
        if (!isParseFramesWithReportsResult(data)) {
          throw new Error(
            "Invalid response, expected parse result, make sure you are using @frames.js/render proxy"
          );
        }
        if (abortController.signal.aborted) {
          return;
        }
        setState({
          status: "success",
          frame: data.farcaster_v2,
          source: sourceUrl
        });
      }).catch((e) => {
        if (abortController.signal.aborted) {
          return;
        }
        setState({
          status: "error",
          error: e instanceof Error ? e : new Error(String(e))
        });
      });
      return () => {
        abortController.abort();
      };
    },
    [fetchRef, proxyUrl]
  );
  (0, import_react2.useEffect)(() => {
    if (typeof source === "string" || source instanceof URL) {
      return fetchFrame(source);
    }
    setState((val) => {
      if (val.status === "success" && val.source === source) {
        return val;
      }
      return {
        status: "success",
        frame: source,
        source
      };
    });
  }, [source, fetchFrame]);
  return state;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useFetchFrameApp
});
