import {
  useFrameApp
} from "../chunk-UMNPS2C4.js";
import "../chunk-BXR2F7XL.js";
import "../chunk-DIJQCH5I.js";
import {
  useDebugLog
} from "../chunk-TLOKKM4F.js";
import "../chunk-HWT6S3D6.js";
import {
  useFreshRef
} from "../chunk-KTZM67PO.js";
import {
  assertNever
} from "../chunk-YAAIMQMX.js";
import "../chunk-LOKN5BYP.js";
import "../chunk-RVPSX5WL.js";
import "../chunk-6C3VEZWH.js";

// src/frame-app/web-view.ts
import { exposeToEndpoint } from "@farcaster/frame-host";
import {
  createWebViewRpcEndpoint
} from "@farcaster/frame-host-react-native";
import { useCallback, useEffect, useMemo, useRef } from "react";
function useFrameAppInWebView(options) {
  const providerRef = useFreshRef(options.provider);
  const webViewRef = useRef(null);
  const debugRef = useFreshRef(options.debug ?? false);
  const frameApp = useFrameApp(options);
  const endpointRef = useRef(null);
  const emitterRef = useRef(null);
  const logDebug = useDebugLog(
    "@frames.js/render/frame-app/web-view",
    debugRef.current
  );
  const emitter = useMemo(() => {
    return {
      emit(...args) {
        if (emitterRef.current) {
          emitterRef.current.emit(...args);
        } else {
          logDebug(
            "endpoint not available, probably not initialized yet, skipping emit",
            args
          );
        }
      },
      emitEthProvider(...args) {
        if (emitterRef.current) {
          emitterRef.current.emitEthProvider(
            ...args
          );
        } else {
          logDebug(
            "endpoint not available, probably not initialized yet, skipping emitEthProvider",
            args
          );
        }
      }
    };
  }, [logDebug]);
  const onMessage = useCallback(
    (event) => {
      logDebug("webView.onMessage() called", event);
      endpointRef.current?.onMessage(event);
    },
    [logDebug]
  );
  const webViewFrameApp = useMemo(() => {
    switch (frameApp.status) {
      case "error":
      case "pending":
        return frameApp;
      case "success": {
        const frame = frameApp.frame.frame;
        const frameUrl = frame.button?.action?.url;
        if (!frameUrl) {
          return {
            status: "error",
            error: new Error(
              "Frame URL is not provided, please check button.action.url"
            )
          };
        }
        return {
          ...frameApp,
          emitter,
          webViewProps: {
            source: { uri: frameUrl },
            onMessage,
            ref: webViewRef
          }
        };
      }
      default:
        assertNever(frameApp);
    }
  }, [frameApp, onMessage, emitter]);
  useEffect(() => {
    if (webViewFrameApp.status !== "success") {
      return;
    }
    const webView = webViewRef.current;
    if (!webView) {
      logDebug("WebView ref not available");
      return;
    }
    const endpoint = createWebViewRpcEndpoint(webViewRef);
    const cleanup = exposeToEndpoint({
      endpoint,
      frameOrigin: "ReactNativeWebView",
      sdk: webViewFrameApp.sdk(endpoint),
      debug: debugRef.current,
      ethProvider: providerRef.current
    });
    endpointRef.current = endpoint;
    emitterRef.current = webViewFrameApp.getEmitter(endpoint);
    logDebug("WebView endpoint created");
    return () => {
      logDebug("WebView unmounted, cleaning up");
      webViewRef.current = null;
      endpointRef.current = null;
      emitterRef.current = null;
      cleanup();
    };
  }, [webViewFrameApp, logDebug, debugRef, providerRef]);
  return webViewFrameApp;
}
export {
  useFrameAppInWebView
};
