"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/identity/lens/index.ts
var lens_exports = {};
__export(lens_exports, {
  useLensFrameContext: () => useLensFrameContext,
  useLensIdentity: () => useLensIdentity
});
module.exports = __toCommonJS(lens_exports);

// src/identity/create-frame-context-hook.ts
var import_react = require("react");

// src/identity/storage/web-storage.ts
var localStorageUnavailableMessage = "WebStorage: localStorage is not available, consider implementing your own Storage class";
var WebStorage = class {
  /**
   * Keeps latest values in memory so we don't need to read them every time
   * we set or read the value.
   */
  values;
  watchers = {};
  constructor() {
    this.values = /* @__PURE__ */ new Map();
    this.watchers = {};
  }
  get = async (key) => {
    if (this.values.has(key)) {
      return Promise.resolve(this.values.get(key));
    }
    if (typeof localStorage !== "undefined") {
      const value = localStorage.getItem(key);
      if (!value) {
        return Promise.resolve(void 0);
      }
      const parsedValue = JSON.parse(value);
      this.values.set(key, parsedValue);
      return Promise.resolve(parsedValue);
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve(void 0);
  };
  set = async (key, setter) => {
    const currentValue = await this.get(key);
    const newValue = setter(currentValue);
    this.values.set(key, newValue);
    this.notifyChange(key, newValue);
    if (typeof localStorage !== "undefined") {
      localStorage.setItem(key, JSON.stringify(newValue));
      return;
    }
    console.warn(localStorageUnavailableMessage);
  };
  delete = (key) => {
    this.values.delete(key);
    this.notifyChange(key, void 0);
    if (typeof localStorage !== "undefined") {
      localStorage.removeItem(key);
      return Promise.resolve();
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve();
  };
  watch = (key, listener) => {
    this.watchers[key] ??= [];
    this.watchers[key]?.push(listener);
    this.get(key).then((value) => {
      listener(value);
    }).catch((e) => {
      console.error(
        `@frames.js/render: Failed to get value from storage: ${e}`
      );
    });
    return () => {
      const listeners = this.watchers[key];
      if (listeners) {
        this.watchers[key] = listeners.filter(
          (watcher) => watcher !== listener
        );
      }
    };
  };
  notifyChange = (key, value) => {
    this.watchers[key]?.forEach((listener) => {
      listener(value);
    });
  };
};

// src/identity/create-frame-context-hook.ts
var defaultStorage = new WebStorage();
function createFrameContextHook(options) {
  return function useFrameContext({
    fallbackContext,
    storage = defaultStorage,
    storageKey = options.storageKey
  }) {
    const storageRef = (0, import_react.useRef)(storage);
    const [frameContext, setFrameContext] = (0, import_react.useState)(
      null
    );
    (0, import_react.useEffect)(() => {
      storageRef.current.get(storageKey).then((data) => {
        if (data) {
          setFrameContext(data);
        }
      }).catch((e) => {
        console.error(
          "@frames.js/render: Could not load frame context from storage",
          e
        );
      });
    }, [storageKey]);
    const handleSetFrameContext = (0, import_react.useCallback)(
      async (newFrameContext) => {
        await storageRef.current.set(
          storageKey,
          () => newFrameContext
        );
        setFrameContext(newFrameContext);
      },
      [storageKey]
    );
    const resetFrameContext = (0, import_react.useCallback)(async () => {
      await storageRef.current.delete(storageKey);
      setFrameContext(null);
    }, [storageKey]);
    return (0, import_react.useMemo)(
      () => ({
        frameContext: frameContext || fallbackContext,
        setFrameContext: handleSetFrameContext,
        resetFrameContext
      }),
      [fallbackContext, frameContext, handleSetFrameContext, resetFrameContext]
    );
  };
}

// src/identity/lens/use-lens-context.tsx
var useLensFrameContext = createFrameContextHook({
  storageKey: "lensFrameContext"
});

// src/identity/lens/use-lens-identity.tsx
var import_rainbowkit = require("@rainbow-me/rainbowkit");
var import_react3 = require("react");
var import_wagmi = require("wagmi");
var import_client = require("@lens-protocol/client");

// src/hooks/use-fresh-ref.ts
var import_react2 = require("react");
function useFreshRef(value) {
  const ref = (0, import_react2.useRef)(value);
  ref.current = value;
  return ref;
}

// src/identity/lens/use-lens-identity.tsx
var defaultStorage2 = new WebStorage();
function useLensIdentity({
  storage = defaultStorage2,
  storageKey = "lensProfile",
  environment = "production"
} = {}) {
  const storageRef = (0, import_react3.useRef)(storage);
  const [isLoading, setIsLoading] = (0, import_react3.useState)(false);
  const [lensSigner, setLensSigner] = (0, import_react3.useState)(null);
  const [showProfileSelector, setShowProfileSelector] = (0, import_react3.useState)(false);
  const [availableProfiles, setAvailableProfiles] = (0, import_react3.useState)([]);
  const connect = (0, import_rainbowkit.useConnectModal)();
  const { address } = (0, import_wagmi.useAccount)();
  const activeConnection = (0, import_wagmi.useConnections)();
  const lensSignerRef = useFreshRef(lensSigner);
  const { switchChainAsync } = (0, import_wagmi.useSwitchChain)();
  const { signMessageAsync } = (0, import_wagmi.useSignMessage)();
  const { signTypedDataAsync } = (0, import_wagmi.useSignTypedData)();
  const storageKeyRef = useFreshRef(storageKey);
  const addressRef = useFreshRef(address);
  const lensClient = (0, import_react3.useRef)(
    new import_client.LensClient({
      environment: environment === "development" ? import_client.development : import_client.production
    })
  ).current;
  const connectRef = useFreshRef(connect);
  (0, import_react3.useEffect)(() => {
    storageRef.current.get(storageKey).then((storedData) => {
      if (storedData) {
        setLensSigner(storedData);
      }
    }).catch((e) => {
      console.error("@frames.js/render: Could not get the Lens profile", e);
    });
  }, [storageKey]);
  const logout = (0, import_react3.useCallback)(async () => {
    await storageRef.current.delete(storageKeyRef.current);
    setLensSigner(null);
  }, [storageKeyRef]);
  const handleSelectProfile = (0, import_react3.useCallback)(
    async (profile) => {
      try {
        const walletAddress = addressRef.current;
        if (!walletAddress) {
          throw new Error("No wallet connected");
        }
        setShowProfileSelector(false);
        const { id, text } = await lensClient.authentication.generateChallenge({
          signedBy: walletAddress,
          for: profile.id
        });
        const signature = await signMessageAsync({
          message: {
            raw: typeof text === "string" ? Buffer.from(text) : Buffer.from(text)
          }
        });
        await lensClient.authentication.authenticate({ id, signature });
        const accessTokenResult = await lensClient.authentication.getAccessToken();
        const identityTokenResult = await lensClient.authentication.getIdentityToken();
        const accessToken = accessTokenResult.unwrap();
        const identityToken = identityTokenResult.unwrap();
        const profileId = await lensClient.authentication.getProfileId();
        const profileInfo = await lensClient.profile.fetch({
          forProfileId: profileId
        });
        const handle = profileInfo?.handle?.localName ? `${profileInfo.handle.localName}.lens` : "";
        if (profileId) {
          const signer = {
            accessToken,
            profileId,
            address: walletAddress,
            identityToken,
            handle
          };
          await storageRef.current.set(
            storageKeyRef.current,
            () => signer
          );
          setLensSigner(signer);
        }
      } catch (error) {
        console.error("@frames.js/render: Create Lens signer failed", error);
      }
    },
    [
      addressRef,
      lensClient.authentication,
      lensClient.profile,
      signMessageAsync,
      storageKeyRef
    ]
  );
  const onSignerlessFramePress = (0, import_react3.useCallback)(async () => {
    try {
      setIsLoading(true);
      const signer = lensSignerRef.current;
      const currentAddress = addressRef.current;
      if (signer) {
        return;
      }
      if (!currentAddress) {
        connectRef.current.openConnectModal?.();
        return;
      }
      const managedProfiles = await lensClient.wallet.profilesManaged({
        for: currentAddress
      });
      const profiles = managedProfiles.items.map((p) => ({
        id: p.id,
        handle: p.handle ? `${p.handle.localName}.lens` : void 0
      }));
      if (!profiles[0]) {
        throw new Error("No Lens profiles managed by connected address");
      }
      if (managedProfiles.items.length > 1) {
        setAvailableProfiles(profiles);
        setShowProfileSelector(true);
      } else {
        await handleSelectProfile(profiles[0]);
      }
    } catch (error) {
      console.error("@frames.js/render: Create Lens signer failed", error);
    } finally {
      setIsLoading(false);
    }
  }, [
    addressRef,
    connectRef,
    handleSelectProfile,
    lensClient.wallet,
    lensSignerRef
  ]);
  const activeConnectionRef = useFreshRef(activeConnection);
  const signFrameAction = (0, import_react3.useCallback)(
    async (actionContext) => {
      const signer = lensSignerRef.current;
      if (!signer) {
        throw new Error("No lens signer active");
      }
      const profileManagers = await lensClient.profile.managers({
        for: signer.profileId
      });
      const lensManagerEnabled = profileManagers.items.some(
        (manager) => manager.isLensManager
      );
      if (lensManagerEnabled) {
        const result = await lensClient.frames.signFrameAction({
          url: actionContext.url,
          inputText: actionContext.inputText || "",
          state: actionContext.state || "",
          buttonIndex: actionContext.buttonIndex,
          actionResponse: actionContext.type === "tx-post" ? actionContext.transactionId : "",
          profileId: signer.profileId,
          pubId: actionContext.frameContext.pubId || "",
          specVersion: "1.0.0"
        });
        if (result.isFailure()) {
          throw new Error("credential expired or not authenticated");
        }
        const searchParams2 = new URLSearchParams({
          postType: actionContext.type === "tx-post" ? "post" : actionContext.frameButton.action,
          postUrl: actionContext.frameButton.target ?? actionContext.target ?? ""
        });
        return {
          body: {
            clientProtocol: "lens@1.0.0",
            untrustedData: {
              ...result.value.signedTypedData.value,
              identityToken: signer.identityToken,
              unixTimestamp: Date.now()
            },
            trustedData: {
              messageBytes: result.value.signature
            }
          },
          searchParams: searchParams2
        };
      }
      const typedData = await lensClient.frames.createFrameTypedData({
        url: actionContext.url,
        inputText: actionContext.inputText || "",
        state: actionContext.state || "",
        buttonIndex: actionContext.buttonIndex,
        actionResponse: actionContext.type === "tx-post" ? actionContext.transactionId : "",
        profileId: signer.profileId,
        pubId: actionContext.frameContext.pubId || "",
        specVersion: "1.0.0",
        deadline: Math.floor(Date.now() / 1e3) + 86400
        // 1 day
      });
      if (activeConnectionRef.current[0]?.chainId !== typedData.domain.chainId) {
        await switchChainAsync({ chainId: typedData.domain.chainId });
      }
      const signature = await signTypedDataAsync({
        domain: {
          ...typedData.domain,
          verifyingContract: typedData.domain.verifyingContract
        },
        types: typedData.types,
        message: typedData.value,
        primaryType: "FrameData"
      });
      const searchParams = new URLSearchParams({
        postType: actionContext.type === "tx-post" ? "post" : actionContext.frameButton.action,
        postUrl: actionContext.target ?? ""
      });
      return {
        body: {
          clientProtocol: "lens@1.0.0",
          untrustedData: {
            ...typedData.value,
            identityToken: signer.identityToken,
            unixTimestamp: Date.now()
          },
          trustedData: {
            messageBytes: signature
          }
        },
        searchParams
      };
    },
    [
      activeConnectionRef,
      lensClient.frames,
      lensClient.profile,
      lensSignerRef,
      signTypedDataAsync,
      switchChainAsync
    ]
  );
  const closeProfileSelector = (0, import_react3.useCallback)(() => {
    setShowProfileSelector(false);
  }, []);
  const isLoadingRef = useFreshRef(isLoading);
  const availableProfilesRef = useFreshRef(availableProfiles);
  return (0, import_react3.useMemo)(() => {
    return {
      specification: "openframes",
      get signer() {
        return lensSignerRef.current;
      },
      get hasSigner() {
        return !!lensSignerRef.current?.accessToken;
      },
      signFrameAction,
      get isLoadingSigner() {
        return isLoadingRef.current;
      },
      onSignerlessFramePress,
      logout,
      showProfileSelector,
      closeProfileSelector,
      get availableProfiles() {
        return availableProfilesRef.current;
      },
      handleSelectProfile,
      withContext(frameContext, overrides) {
        return {
          signerState: {
            ...this,
            ...overrides
          },
          frameContext
        };
      }
    };
  }, [
    availableProfiles,
    availableProfilesRef,
    closeProfileSelector,
    handleSelectProfile,
    isLoading,
    isLoadingRef,
    lensSigner,
    lensSignerRef,
    logout,
    onSignerlessFramePress,
    showProfileSelector,
    signFrameAction
  ]);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useLensFrameContext,
  useLensIdentity
});
