import { B as FramesStackItem, A as FrameStackMessage, z as FrameStackRequestError, G as GetFrameResult, x as FrameStackDone } from '../types-CRf9XqD9.js';
import { FramesStackItem as FramesStackItem$1, FrameStackMessage as FrameStackMessage$1, FrameStackRequestError as FrameStackRequestError$1, PartialFrameV2 } from '../unstable-types.js';
import { PartialFrame } from './types.js';
import 'frames.js';
import 'react';
import 'frames.js/frame-parsers';
import 'frames.js/types';

type FrameResultFailure = Exclude<GetFrameResult, {
    status: "success";
}>;
type FrameResultSuccess = Extract<GetFrameResult, {
    status: "success";
}>;
type FrameResultSuccessFrameV1 = Extract<FrameResultSuccess, {
    specification: "farcaster" | "openframes";
}>;
type FrameResultFailureFrameV1 = Extract<FrameResultFailure, {
    specification: "farcaster" | "openframes";
}>;
type FrameV1FailureResult = Omit<FrameResultFailureFrameV1, "frame"> & {
    frame: PartialFrame;
};
type FrameResultFailureFrameV2 = Extract<FrameResultFailure, {
    specification: "farcaster_v2";
}>;
type FrameResultSuccessFrameV2 = Extract<FrameResultSuccess, {
    specification: "farcaster_v2";
}>;
type FrameV2FailureResult = Omit<FrameResultFailureFrameV2, "frame"> & {
    frame: PartialFrameV2;
};
type FrameStackItemWithPartialFrame = Omit<FrameStackDone, "frameResult"> & {
    frameResult: FrameV1FailureResult | FrameV2FailureResult;
};
declare function isPartialFrameStackItem(stackItem: FramesStackItem | FramesStackItem$1): stackItem is FrameStackItemWithPartialFrame;
declare function getErrorMessageFromFramesStackItem(item: FrameStackMessage | FrameStackRequestError | FrameStackMessage$1 | FrameStackRequestError$1): string;
declare function isValidPartialFrameV1(value: GetFrameResult): value is FrameV1FailureResult;
declare function isValidPartialFrameV2(value: GetFrameResult): value is FrameV2FailureResult;
/**
 * All partial frames need at least an image and button to be considered valid.
 */
declare function isValidPartialFrame(frameResult: GetFrameResult): boolean;
declare function isValidFrameV1(value: GetFrameResult): value is FrameResultSuccessFrameV1;
declare function isValidFrameV2(value: GetFrameResult): value is FrameResultSuccessFrameV2;
declare function isValidFrame(value: GetFrameResult): boolean;

export { getErrorMessageFromFramesStackItem, isPartialFrameStackItem, isValidFrame, isValidFrameV1, isValidFrameV2, isValidPartialFrame, isValidPartialFrameV1, isValidPartialFrameV2 };
