"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/unstable-use-frame-state.ts
var unstable_use_frame_state_exports = {};
__export(unstable_use_frame_state_exports, {
  useFrameState: () => useFrameState
});
module.exports = __toCommonJS(unstable_use_frame_state_exports);
var import_react2 = require("react");

// src/hooks/use-fresh-ref.ts
var import_react = require("react");
function useFreshRef(value) {
  const ref = (0, import_react.useRef)(value);
  ref.current = value;
  return ref;
}

// src/unstable-use-frame-state.ts
function resolveParseResultForSpecification(parseResult, specification) {
  const specifications = Array.isArray(specification) ? specification : [specification];
  if (specifications.length === 0) {
    throw new Error("Signer does not have any specification defined");
  }
  let frameResult;
  for (const currentSpecification of specifications) {
    if (parseResult[currentSpecification].status === "success") {
      frameResult = parseResult[currentSpecification];
      break;
    }
    if (!frameResult) {
      frameResult = parseResult[currentSpecification];
    }
  }
  if (!frameResult) {
    throw new Error("No frame for the given specification");
  }
  return frameResult;
}
function createFramesStackReducer(resolveSignerRef) {
  return function framesStackReducer(state, action) {
    switch (action.action) {
      case "LOAD":
        return {
          ...state,
          stack: [action.item, ...state.stack]
        };
      case "DONE_REDIRECT": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = {
          ...action.pendingItem,
          ...action.item,
          status: "doneRedirect"
        };
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "DONE_WITH_ERROR_MESSAGE": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = {
          ...action.pendingItem,
          ...action.item
        };
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "DONE": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        let signerState;
        let specification;
        let frameContext;
        let homeframeUrl;
        let parseResult = action.parseResult;
        if (state.type === "not-initialized") {
          const resolvedSigner = resolveSignerRef.current({
            parseResult: action.parseResult
          });
          ({ signerState, frameContext = {} } = resolvedSigner);
          homeframeUrl = action.pendingItem.url;
          specification = signerState.specification;
        } else {
          ({
            signerState,
            specification,
            frameContext,
            homeframeUrl,
            parseResult
          } = state);
        }
        const frameResult = resolveParseResultForSpecification(
          action.parseResult,
          specification
        );
        state.stack[index] = {
          ...action.pendingItem,
          status: "done",
          frameResult,
          extra: action.extra
        };
        return {
          ...state,
          parseResult,
          signerState,
          frameContext,
          homeframeUrl,
          specification: frameResult.specification,
          type: "initialized",
          stack: state.stack.slice()
        };
      }
      case "REQUEST_ERROR": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = action.item;
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "RESET": {
        if (state.type === "not-initialized") {
          return state;
        }
        const { frameContext = {}, signerState } = resolveSignerRef.current({
          parseResult: state.parseResult
        });
        const frameResult = resolveParseResultForSpecification(
          state.parseResult,
          signerState.specification
        );
        const stackItem = state.stack[0];
        if (stackItem?.status === "done") {
          stackItem.frameResult = frameResult;
        }
        return {
          ...state,
          stack: stackItem ? [{ ...stackItem }] : [],
          type: "initialized",
          frameContext,
          signerState,
          specification: frameResult.specification
        };
      }
      case "RESET_INITIAL_FRAME": {
        const { frameContext = {}, signerState } = resolveSignerRef.current({
          parseResult: action.parseResult
        });
        const frameResult = resolveParseResultForSpecification(
          action.parseResult,
          signerState.specification
        );
        return {
          type: "initialized",
          signerState,
          frameContext,
          specification: frameResult.specification,
          homeframeUrl: action.homeframeUrl,
          parseResult: action.parseResult,
          stack: [
            {
              request: {
                method: "GET",
                url: action.homeframeUrl
              },
              url: action.homeframeUrl,
              id: Date.now(),
              frameResult,
              status: "done",
              extra: action.extra
            }
          ]
        };
      }
      case "CLEAR":
        return {
          type: "not-initialized",
          stack: []
        };
      default:
        return state;
    }
  };
}
function useFrameState({
  initialParseResult,
  initialFrameUrl,
  initialPendingExtra,
  resolveSpecification,
  resolveGETPendingExtra,
  resolvePOSTPendingExtra,
  resolveDoneExtra,
  resolveDoneRedirectExtra,
  resolveDoneWithErrorMessageExtra,
  resolveFailedExtra,
  resolveFailedWithRequestErrorExtra
}) {
  const idCounterRef = (0, import_react2.useRef)(0);
  const resolveSpecificationRef = useFreshRef(resolveSpecification);
  const resolveGETPendingExtraRef = useFreshRef(resolveGETPendingExtra);
  const resolvePOSTPendingExtraRef = useFreshRef(resolvePOSTPendingExtra);
  const resolveDoneExtraRef = useFreshRef(resolveDoneExtra);
  const resolveDoneRedirectExtraRef = useFreshRef(resolveDoneRedirectExtra);
  const resolveDoneWithErrorMessageExtraRef = useFreshRef(
    resolveDoneWithErrorMessageExtra
  );
  const resolveFailedExtraRef = useFreshRef(resolveFailedExtra);
  const resolveFailedWithRequestErrorExtraRef = useFreshRef(
    resolveFailedWithRequestErrorExtra
  );
  const reducerRef = (0, import_react2.useRef)(
    createFramesStackReducer(resolveSpecificationRef)
  );
  const initialPendingExtraRef = useFreshRef(initialPendingExtra);
  const [state, dispatch] = (0, import_react2.useReducer)(
    reducerRef.current,
    [initialParseResult, initialFrameUrl, initialPendingExtra],
    ([parseResult, frameUrl, extra]) => {
      if (parseResult && frameUrl) {
        const { frameContext = {}, signerState } = resolveSpecification({
          parseResult
        });
        const frameResult = resolveParseResultForSpecification(
          parseResult,
          signerState.specification
        );
        return {
          type: "initialized",
          frameContext,
          signerState,
          specification: frameResult.specification,
          homeframeUrl: frameUrl,
          parseResult,
          stack: [
            {
              id: idCounterRef.current++,
              request: {
                method: "GET",
                url: frameUrl
              },
              frameResult,
              status: "done",
              url: frameUrl,
              extra: extra ?? {}
            }
          ]
        };
      }
      return {
        type: "not-initialized",
        stack: frameUrl ? [
          // prevent flash of empty content by adding pending item because initial frame is being loaded
          {
            id: idCounterRef.current++,
            method: "GET",
            request: {
              method: "GET",
              url: frameUrl
            },
            url: frameUrl,
            status: "pending",
            extra: extra ?? {}
          }
        ] : []
      };
    }
  );
  const api = (0, import_react2.useMemo)(() => {
    return {
      dispatch,
      clear() {
        dispatch({
          action: "CLEAR"
        });
      },
      createGetPendingItem(arg) {
        const item = {
          id: idCounterRef.current++,
          method: "GET",
          request: arg.request,
          url: arg.request.url,
          status: "pending",
          extra: resolveGETPendingExtraRef.current?.(arg) ?? {}
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createPostPendingItem(arg) {
        const item = {
          id: idCounterRef.current++,
          method: "POST",
          request: arg.request,
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl",
          status: "pending",
          extra: resolvePOSTPendingExtraRef.current?.(arg) ?? {}
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      markAsDone(arg) {
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          parseResult: arg.parseResult,
          extra: resolveDoneExtraRef.current?.(arg) ?? {}
        });
      },
      markAsDoneWithErrorMessage(arg) {
        dispatch({
          action: "DONE_WITH_ERROR_MESSAGE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "message",
            type: "error",
            message: arg.responseData.message,
            extra: resolveDoneWithErrorMessageExtraRef.current?.(arg) ?? {}
          }
        });
      },
      markAsDoneWithRedirect(arg) {
        dispatch({
          action: "DONE_REDIRECT",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            location: arg.location,
            status: "doneRedirect",
            extra: resolveDoneRedirectExtraRef.current?.(arg) ?? {}
          }
        });
      },
      markAsFailed(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            request: arg.pendingItem.request,
            id: arg.pendingItem.id,
            url: arg.pendingItem.url,
            requestError: arg.requestError,
            status: "requestError",
            extra: resolveFailedExtraRef.current?.(arg) ?? {}
          }
        });
      },
      markAsFailedWithRequestError(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "requestError",
            requestError: arg.error,
            extra: resolveFailedWithRequestErrorExtraRef.current?.(arg) ?? {}
          }
        });
      },
      reset(arg) {
        if (!arg) {
          dispatch({ action: "RESET" });
        } else {
          dispatch({
            action: "RESET_INITIAL_FRAME",
            homeframeUrl: arg.homeframeUrl,
            parseResult: arg.parseResult,
            extra: initialPendingExtraRef.current ?? {}
          });
        }
      }
    };
  }, [
    initialPendingExtraRef,
    resolveDoneExtraRef,
    resolveDoneRedirectExtraRef,
    resolveDoneWithErrorMessageExtraRef,
    resolveFailedExtraRef,
    resolveFailedWithRequestErrorExtraRef,
    resolveGETPendingExtraRef,
    resolvePOSTPendingExtraRef
  ]);
  return [state, api];
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useFrameState
});
