"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/unstable-use-frame.tsx
var unstable_use_frame_exports = {};
__export(unstable_use_frame_exports, {
  useFrame_unstable: () => useFrame_unstable
});
module.exports = __toCommonJS(unstable_use_frame_exports);
var import_react3 = require("react");

// src/unstable-use-frame-state.ts
var import_react2 = require("react");

// src/hooks/use-fresh-ref.ts
var import_react = require("react");
function useFreshRef(value) {
  const ref = (0, import_react.useRef)(value);
  ref.current = value;
  return ref;
}

// src/unstable-use-frame-state.ts
function resolveParseResultForSpecification(parseResult, specification) {
  const specifications = Array.isArray(specification) ? specification : [specification];
  if (specifications.length === 0) {
    throw new Error("Signer does not have any specification defined");
  }
  let frameResult;
  for (const currentSpecification of specifications) {
    if (parseResult[currentSpecification].status === "success") {
      frameResult = parseResult[currentSpecification];
      break;
    }
    if (!frameResult) {
      frameResult = parseResult[currentSpecification];
    }
  }
  if (!frameResult) {
    throw new Error("No frame for the given specification");
  }
  return frameResult;
}
function createFramesStackReducer(resolveSignerRef) {
  return function framesStackReducer(state, action) {
    switch (action.action) {
      case "LOAD":
        return {
          ...state,
          stack: [action.item, ...state.stack]
        };
      case "DONE_REDIRECT": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = {
          ...action.pendingItem,
          ...action.item,
          status: "doneRedirect"
        };
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "DONE_WITH_ERROR_MESSAGE": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = {
          ...action.pendingItem,
          ...action.item
        };
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "DONE": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        let signerState;
        let specification;
        let frameContext;
        let homeframeUrl;
        let parseResult = action.parseResult;
        if (state.type === "not-initialized") {
          const resolvedSigner = resolveSignerRef.current({
            parseResult: action.parseResult
          });
          ({ signerState, frameContext = {} } = resolvedSigner);
          homeframeUrl = action.pendingItem.url;
          specification = signerState.specification;
        } else {
          ({
            signerState,
            specification,
            frameContext,
            homeframeUrl,
            parseResult
          } = state);
        }
        const frameResult = resolveParseResultForSpecification(
          action.parseResult,
          specification
        );
        state.stack[index] = {
          ...action.pendingItem,
          status: "done",
          frameResult,
          extra: action.extra
        };
        return {
          ...state,
          parseResult,
          signerState,
          frameContext,
          homeframeUrl,
          specification: frameResult.specification,
          type: "initialized",
          stack: state.stack.slice()
        };
      }
      case "REQUEST_ERROR": {
        const index = state.stack.findIndex(
          (item) => item.id === action.pendingItem.id
        );
        if (index === -1) {
          return state;
        }
        state.stack[index] = action.item;
        return {
          ...state,
          stack: state.stack.slice()
        };
      }
      case "RESET": {
        if (state.type === "not-initialized") {
          return state;
        }
        const { frameContext = {}, signerState } = resolveSignerRef.current({
          parseResult: state.parseResult
        });
        const frameResult = resolveParseResultForSpecification(
          state.parseResult,
          signerState.specification
        );
        const stackItem = state.stack[0];
        if (stackItem?.status === "done") {
          stackItem.frameResult = frameResult;
        }
        return {
          ...state,
          stack: stackItem ? [{ ...stackItem }] : [],
          type: "initialized",
          frameContext,
          signerState,
          specification: frameResult.specification
        };
      }
      case "RESET_INITIAL_FRAME": {
        const { frameContext = {}, signerState } = resolveSignerRef.current({
          parseResult: action.parseResult
        });
        const frameResult = resolveParseResultForSpecification(
          action.parseResult,
          signerState.specification
        );
        return {
          type: "initialized",
          signerState,
          frameContext,
          specification: frameResult.specification,
          homeframeUrl: action.homeframeUrl,
          parseResult: action.parseResult,
          stack: [
            {
              request: {
                method: "GET",
                url: action.homeframeUrl
              },
              url: action.homeframeUrl,
              id: Date.now(),
              frameResult,
              status: "done",
              extra: action.extra
            }
          ]
        };
      }
      case "CLEAR":
        return {
          type: "not-initialized",
          stack: []
        };
      default:
        return state;
    }
  };
}
function useFrameState({
  initialParseResult,
  initialFrameUrl,
  initialPendingExtra,
  resolveSpecification,
  resolveGETPendingExtra,
  resolvePOSTPendingExtra,
  resolveDoneExtra,
  resolveDoneRedirectExtra,
  resolveDoneWithErrorMessageExtra,
  resolveFailedExtra,
  resolveFailedWithRequestErrorExtra
}) {
  const idCounterRef = (0, import_react2.useRef)(0);
  const resolveSpecificationRef = useFreshRef(resolveSpecification);
  const resolveGETPendingExtraRef = useFreshRef(resolveGETPendingExtra);
  const resolvePOSTPendingExtraRef = useFreshRef(resolvePOSTPendingExtra);
  const resolveDoneExtraRef = useFreshRef(resolveDoneExtra);
  const resolveDoneRedirectExtraRef = useFreshRef(resolveDoneRedirectExtra);
  const resolveDoneWithErrorMessageExtraRef = useFreshRef(
    resolveDoneWithErrorMessageExtra
  );
  const resolveFailedExtraRef = useFreshRef(resolveFailedExtra);
  const resolveFailedWithRequestErrorExtraRef = useFreshRef(
    resolveFailedWithRequestErrorExtra
  );
  const reducerRef = (0, import_react2.useRef)(
    createFramesStackReducer(resolveSpecificationRef)
  );
  const initialPendingExtraRef = useFreshRef(initialPendingExtra);
  const [state, dispatch] = (0, import_react2.useReducer)(
    reducerRef.current,
    [initialParseResult, initialFrameUrl, initialPendingExtra],
    ([parseResult, frameUrl, extra]) => {
      if (parseResult && frameUrl) {
        const { frameContext = {}, signerState } = resolveSpecification({
          parseResult
        });
        const frameResult = resolveParseResultForSpecification(
          parseResult,
          signerState.specification
        );
        return {
          type: "initialized",
          frameContext,
          signerState,
          specification: frameResult.specification,
          homeframeUrl: frameUrl,
          parseResult,
          stack: [
            {
              id: idCounterRef.current++,
              request: {
                method: "GET",
                url: frameUrl
              },
              frameResult,
              status: "done",
              url: frameUrl,
              extra: extra ?? {}
            }
          ]
        };
      }
      return {
        type: "not-initialized",
        stack: frameUrl ? [
          // prevent flash of empty content by adding pending item because initial frame is being loaded
          {
            id: idCounterRef.current++,
            method: "GET",
            request: {
              method: "GET",
              url: frameUrl
            },
            url: frameUrl,
            status: "pending",
            extra: extra ?? {}
          }
        ] : []
      };
    }
  );
  const api = (0, import_react2.useMemo)(() => {
    return {
      dispatch,
      clear() {
        dispatch({
          action: "CLEAR"
        });
      },
      createGetPendingItem(arg) {
        const item = {
          id: idCounterRef.current++,
          method: "GET",
          request: arg.request,
          url: arg.request.url,
          status: "pending",
          extra: resolveGETPendingExtraRef.current?.(arg) ?? {}
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createPostPendingItem(arg) {
        const item = {
          id: idCounterRef.current++,
          method: "POST",
          request: arg.request,
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl",
          status: "pending",
          extra: resolvePOSTPendingExtraRef.current?.(arg) ?? {}
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      markAsDone(arg) {
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          parseResult: arg.parseResult,
          extra: resolveDoneExtraRef.current?.(arg) ?? {}
        });
      },
      markAsDoneWithErrorMessage(arg) {
        dispatch({
          action: "DONE_WITH_ERROR_MESSAGE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "message",
            type: "error",
            message: arg.responseData.message,
            extra: resolveDoneWithErrorMessageExtraRef.current?.(arg) ?? {}
          }
        });
      },
      markAsDoneWithRedirect(arg) {
        dispatch({
          action: "DONE_REDIRECT",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            location: arg.location,
            status: "doneRedirect",
            extra: resolveDoneRedirectExtraRef.current?.(arg) ?? {}
          }
        });
      },
      markAsFailed(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            request: arg.pendingItem.request,
            id: arg.pendingItem.id,
            url: arg.pendingItem.url,
            requestError: arg.requestError,
            status: "requestError",
            extra: resolveFailedExtraRef.current?.(arg) ?? {}
          }
        });
      },
      markAsFailedWithRequestError(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "requestError",
            requestError: arg.error,
            extra: resolveFailedWithRequestErrorExtraRef.current?.(arg) ?? {}
          }
        });
      },
      reset(arg) {
        if (!arg) {
          dispatch({ action: "RESET" });
        } else {
          dispatch({
            action: "RESET_INITIAL_FRAME",
            homeframeUrl: arg.homeframeUrl,
            parseResult: arg.parseResult,
            extra: initialPendingExtraRef.current ?? {}
          });
        }
      }
    };
  }, [
    initialPendingExtraRef,
    resolveDoneExtraRef,
    resolveDoneRedirectExtraRef,
    resolveDoneWithErrorMessageExtraRef,
    resolveFailedExtraRef,
    resolveFailedWithRequestErrorExtraRef,
    resolveGETPendingExtraRef,
    resolvePOSTPendingExtraRef
  ]);
  return [state, api];
}

// src/unstable-use-fetch-frame.ts
var import_viem = require("viem");

// src/errors.ts
var TransactionDataTargetMalformedError = class extends Error {
  constructor(response) {
    super("Malformed transaction target data from the server");
    this.response = response;
  }
};
var TransactionDataErrorResponseError = class extends Error {
  constructor(response) {
    super("Non OK transaction data response from the server");
    this.response = response;
  }
};
var TransactionHandlerDidNotReturnTransactionIdError = class extends Error {
  constructor() {
    super("onTransaction() handler did not return a transaction id");
  }
};
var SignatureHandlerDidNotReturnTransactionIdError = class extends Error {
  constructor() {
    super("onSignature() handler did not return a transaction id");
  }
};

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}
function tryCall(fn) {
  try {
    return fn();
  } catch (e) {
    if (e instanceof Error) {
      return e;
    }
    return new TypeError("Unexpected error, check the console for details");
  }
}
function isParseFramesWithReportsResult(value) {
  return typeof value === "object" && value !== null && "openframes" in value && "farcaster" in value;
}

// src/unstable-use-fetch-frame.ts
function isErrorMessageResponse(response) {
  return typeof response === "object" && response !== null && "message" in response && typeof response.message === "string";
}
function defaultErrorHandler(error) {
  console.error(error);
}
function useFetchFrame({
  frameStateAPI,
  frameState,
  frameActionProxy,
  frameGetProxy,
  extraButtonRequestPayload,
  parseFarcasterManifest = false,
  onTransaction,
  transactionDataSuffix,
  onSignature,
  onError = defaultErrorHandler,
  fetchFn,
  onRedirect,
  onTransactionDataError,
  onTransactionDataStart,
  onTransactionDataSuccess,
  onTransactionError,
  onTransactionStart,
  onTransactionSuccess,
  onSignatureError,
  onSignatureStart,
  onSignatureSuccess,
  onTransactionProcessingError,
  onTransactionProcessingStart,
  onTransactionProcessingSuccess
}) {
  async function handleFailedResponse({
    response,
    endTime,
    frameStackPendingItem,
    onError: onErrorInternal
  }) {
    if (response.ok) {
      throw new TypeError(
        "handleFailedResponse called with a successful response"
      );
    }
    const responseBody = await getResponseBody(response);
    if (response.status >= 400 && response.status < 500) {
      if (frameStackPendingItem.method === "POST" && isErrorMessageResponse(responseBody)) {
        frameStateAPI.markAsDoneWithErrorMessage({
          pendingItem: frameStackPendingItem,
          endTime,
          response,
          responseData: responseBody
        });
        const error = new Error(responseBody.message);
        tryCall(() => {
          onError(error);
        });
        tryCall(() => onErrorInternal?.(error));
        return;
      }
    }
    const requestError = new Error(
      `The server returned an error but it does not contain message property. Status code: ${response.status}`
    );
    frameStateAPI.markAsFailed({
      endTime,
      pendingItem: frameStackPendingItem,
      requestError,
      responseStatus: response.status,
      response,
      responseBody
    });
    tryCall(() => {
      onError(requestError);
    });
    tryCall(() => onErrorInternal?.(requestError));
  }
  async function fetchGETRequest(request, shouldClear) {
    if (shouldClear) {
      frameStateAPI.clear();
    }
    const frameStackPendingItem = frameStateAPI.createGetPendingItem({
      request
    });
    const response = await fetchProxied({
      proxyUrl: frameGetProxy,
      fetchFn,
      url: request.url,
      parseFarcasterManifest
    });
    const endTime = /* @__PURE__ */ new Date();
    if (response instanceof Response) {
      if (!response.ok) {
        await handleFailedResponse({
          response,
          endTime,
          frameStackPendingItem
        });
        return;
      }
      const parseResult = await tryCallAsync(
        () => response.clone().json()
      );
      if (parseResult instanceof Error) {
        frameStateAPI.markAsFailed({
          pendingItem: frameStackPendingItem,
          endTime,
          requestError: parseResult,
          response,
          responseBody: "none",
          responseStatus: 500
        });
        tryCall(() => {
          onError(parseResult);
        });
        return;
      }
      if (!isParseFramesWithReportsResult(parseResult)) {
        const error = new Error("The server returned an unexpected response.");
        frameStateAPI.markAsFailed({
          pendingItem: frameStackPendingItem,
          endTime,
          requestError: error,
          response,
          responseBody: "none",
          responseStatus: 500
        });
        tryCall(() => {
          onError(error);
        });
        return;
      }
      frameStateAPI.markAsDone({
        pendingItem: frameStackPendingItem,
        endTime,
        parseResult,
        responseBody: await response.clone().text(),
        response: response.clone()
      });
      return;
    }
    frameStateAPI.markAsFailed({
      pendingItem: frameStackPendingItem,
      endTime,
      requestError: response,
      response: null,
      responseBody: "none",
      responseStatus: 500
    });
    tryCall(() => {
      onError(response);
    });
  }
  async function fetchPOSTRequest(request, options) {
    let pendingItem;
    if (frameState.type === "not-initialized") {
      throw new Error(
        "POST Request cannot be made before the frame is initialized"
      );
    }
    if (options?.shouldClear) {
      frameStateAPI.clear();
    }
    const signedDataOrError = await signAndGetFrameActionBodyPayload({
      request,
      signerState: frameState.signerState
    });
    if (signedDataOrError instanceof Error) {
      if (options?.preflightRequest) {
        frameStateAPI.markAsFailed({
          pendingItem: options.preflightRequest.pendingFrameStackItem,
          endTime: /* @__PURE__ */ new Date(),
          requestError: signedDataOrError,
          response: null,
          // there is no response because didn't even got to request
          responseBody: "none",
          responseStatus: 500
        });
      }
      tryCall(() => {
        onError(signedDataOrError);
      });
      tryCall(() => options?.onError?.(signedDataOrError));
      return;
    }
    if (!options?.preflightRequest) {
      pendingItem = frameStateAPI.createPostPendingItem({
        action: signedDataOrError,
        request
      });
    } else {
      pendingItem = options.preflightRequest.pendingFrameStackItem;
    }
    const response = await fetchProxied({
      proxyUrl: frameActionProxy,
      fetchFn,
      frameAction: signedDataOrError,
      extraRequestPayload: extraButtonRequestPayload
    });
    const endTime = /* @__PURE__ */ new Date();
    async function handleRedirect({
      response: res,
      currentPendingItem,
      onError: onErrorInternal,
      onSuccess: onSuccessInternal
    }) {
      try {
        let location = res.headers.get("location");
        if (!location) {
          const responseData = await res.clone().json();
          if (responseData && typeof responseData === "object" && "location" in responseData && typeof responseData.location === "string") {
            location = responseData.location;
          }
        }
        if (!location) {
          throw new Error(
            `Response data does not contain 'location' key and no 'location' header is found.`
          );
        }
        const locationUrl = new URL(location);
        if (locationUrl.protocol !== "http:" && locationUrl.protocol !== "https:") {
          throw new Error(
            `Redirect location ${location} is not a valid HTTP or HTTPS URL.`
          );
        }
        tryCall(() => {
          onRedirect(locationUrl);
        });
        tryCall(() => onSuccessInternal?.());
        frameStateAPI.markAsDoneWithRedirect({
          pendingItem: currentPendingItem,
          endTime,
          location,
          response: res.clone(),
          responseBody: await res.clone().text()
        });
      } catch (e) {
        const error = e instanceof Error ? e : new Error(
          "Response body must be a json with 'location' property or response 'Location' header must contain fully qualified URL."
        );
        frameStateAPI.markAsFailedWithRequestError({
          pendingItem: currentPendingItem,
          error,
          response: res,
          endTime,
          responseBody: await res.clone().text()
        });
        tryCall(() => onErrorInternal?.(error));
        tryCall(() => {
          onError(error);
        });
      }
    }
    if (response instanceof Response) {
      if (response.status === 302) {
        await handleRedirect({
          response,
          currentPendingItem: pendingItem,
          onError: options?.onError,
          onSuccess: options?.onSuccess
        });
        return;
      }
      if (!response.ok) {
        await handleFailedResponse({
          response,
          endTime,
          frameStackPendingItem: pendingItem,
          onError: options?.onError
        });
        return;
      }
      const responseData = await tryCall(
        () => response.clone().json()
      );
      if (responseData instanceof Error) {
        frameStateAPI.markAsFailed({
          endTime,
          pendingItem,
          requestError: responseData,
          response,
          responseBody: "none",
          responseStatus: 500
        });
        tryCall(() => {
          onError(responseData);
        });
        tryCall(() => options?.onError?.(responseData));
        return;
      }
      if (!isParseFramesWithReportsResult(responseData)) {
        const error = new Error("The server returned an unexpected response.");
        frameStateAPI.markAsFailed({
          endTime,
          pendingItem,
          requestError: error,
          response,
          responseBody: responseData,
          responseStatus: 500
        });
        tryCall(() => {
          onError(error);
        });
        tryCall(() => options?.onError?.(error));
        return;
      }
      frameStateAPI.markAsDone({
        endTime,
        parseResult: responseData,
        pendingItem,
        response,
        responseBody: await response.clone().text()
      });
      tryCall(() => options?.onSuccess?.());
      return;
    }
    frameStateAPI.markAsFailed({
      endTime,
      pendingItem,
      requestError: response,
      response: new Response(response.message, {
        status: 500,
        headers: { "Content-Type": "text/plain" }
      }),
      responseBody: "none",
      responseStatus: 500
    });
    tryCall(() => {
      onError(response);
    });
    tryCall(() => options?.onError?.(response));
  }
  async function fetchTransactionRequest(request, shouldClear) {
    if ("source" in request) {
      throw new Error(
        "Invalid request, transaction should be invoked only from a Frame. It was probably invoked from cast or composer action."
      );
    }
    if (frameState.type === "not-initialized") {
      throw new Error(
        "Transaction Request cannot be made before the frame is initialized"
      );
    }
    const button = request.frameButton;
    const sourceFrame = request.sourceFrame;
    if (button.action !== "tx") {
      throw new Error("Invalid frame button action, tx expected");
    }
    if (request.signerStateActionContext.type !== "tx-data") {
      throw new Error(
        "Invalid signer state action context type, tx-data expected"
      );
    }
    if (shouldClear) {
      frameStateAPI.clear();
    }
    tryCall(() => onTransactionDataStart?.({ button }));
    const signedTransactionDataActionOrError = await tryCallAsync(
      () => frameState.signerState.signFrameAction(request.signerStateActionContext)
    );
    if (signedTransactionDataActionOrError instanceof Error) {
      tryCall(() => {
        onError(signedTransactionDataActionOrError);
      });
      tryCall(
        () => onTransactionDataError?.(signedTransactionDataActionOrError)
      );
      return;
    }
    const transactionDataStartTime = /* @__PURE__ */ new Date();
    const transactionDataResponse = await fetchProxied({
      proxyUrl: frameActionProxy,
      frameAction: signedTransactionDataActionOrError,
      fetchFn,
      extraRequestPayload: extraButtonRequestPayload
    });
    const transactionDataEndTime = /* @__PURE__ */ new Date();
    if (transactionDataResponse instanceof Error) {
      const pendingItem2 = frameStateAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request,
        startTime: transactionDataStartTime
      });
      frameStateAPI.markAsFailed({
        endTime: transactionDataEndTime,
        pendingItem: pendingItem2,
        requestError: transactionDataResponse,
        response: null,
        responseBody: "none",
        responseStatus: 500
      });
      tryCall(() => onTransactionDataError?.(transactionDataResponse));
      tryCall(() => {
        onError(transactionDataResponse);
      });
      return;
    }
    if (!transactionDataResponse.ok) {
      const pendingItem2 = frameStateAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request,
        startTime: transactionDataStartTime
      });
      await handleFailedResponse({
        response: transactionDataResponse,
        endTime: transactionDataEndTime,
        frameStackPendingItem: pendingItem2
      });
      tryCall(
        () => onTransactionDataError?.(
          new TransactionDataErrorResponseError(transactionDataResponse.clone())
        )
      );
      return;
    }
    function isTransactionTargetResponse(response) {
      return typeof response === "object" && response !== null && "method" in response;
    }
    const transactionData = await tryCallAsync(
      () => transactionDataResponse.clone().json()
    );
    if (!isTransactionTargetResponse(transactionData)) {
      const pendingItem2 = frameStateAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request,
        startTime: transactionDataStartTime
      });
      const error = new TransactionDataTargetMalformedError(
        transactionDataResponse.clone()
      );
      frameStateAPI.markAsFailed({
        endTime: transactionDataEndTime,
        pendingItem: pendingItem2,
        requestError: error,
        response: transactionDataResponse,
        responseBody: transactionData,
        responseStatus: 500
      });
      tryCall(() => onTransactionDataError?.(error));
      tryCall(() => {
        onError(error);
      });
      return;
    }
    tryCall(
      () => onTransactionDataSuccess?.({ button, data: transactionData })
    );
    let transactionIdOrError;
    if (transactionData.method === "eth_sendTransaction") {
      if (transactionData.params.data && transactionData.attribution !== false && transactionDataSuffix && // Has a function signature
      (0, import_viem.hexToBytes)(transactionData.params.data).length > 4) {
        transactionData.params.data = transactionData.params.data + transactionDataSuffix.slice(2);
      }
      tryCall(() => onTransactionStart?.({ button, data: transactionData }));
      transactionIdOrError = await tryCallAsync(
        () => onTransaction({
          frame: sourceFrame,
          frameButton: request.frameButton,
          transactionData
        }).then((transactionId) => {
          if (!transactionId) {
            return new TransactionHandlerDidNotReturnTransactionIdError();
          }
          return transactionId;
        })
      );
      if (!(transactionIdOrError instanceof Error)) {
        tryCall(() => onTransactionSuccess?.({ button }));
      } else {
        tryCall(() => onTransactionError?.(transactionIdOrError));
      }
    } else {
      tryCall(() => onSignatureStart?.({ button, data: transactionData }));
      transactionIdOrError = await tryCallAsync(
        () => onSignature({
          frame: sourceFrame,
          frameButton: request.frameButton,
          signatureData: transactionData
        }).then((signatureHash) => {
          if (!signatureHash) {
            return new SignatureHandlerDidNotReturnTransactionIdError();
          }
          return signatureHash;
        })
      );
      if (!(transactionIdOrError instanceof Error)) {
        tryCall(() => onSignatureSuccess?.({ button }));
      } else {
        tryCall(() => onSignatureError?.(transactionIdOrError));
      }
    }
    if (transactionIdOrError instanceof Error) {
      const pendingItem2 = frameStateAPI.createPostPendingItem({
        action: signedTransactionDataActionOrError,
        request
      });
      frameStateAPI.markAsFailed({
        pendingItem: pendingItem2,
        endTime: /* @__PURE__ */ new Date(),
        requestError: transactionIdOrError,
        response: null,
        responseBody: "none",
        responseStatus: 500
      });
      tryCall(() => {
        onError(transactionIdOrError);
      });
      return;
    }
    tryCall(
      () => onTransactionProcessingStart?.({
        button,
        transactionId: transactionIdOrError
      })
    );
    const startTime = /* @__PURE__ */ new Date();
    const pendingItem = frameStateAPI.createPostPendingItem({
      action: signedTransactionDataActionOrError,
      request
    });
    await fetchPOSTRequest(
      {
        ...request,
        signerStateActionContext: {
          ...request.signerStateActionContext,
          type: "tx-post",
          // include transactionId in payload
          transactionId: transactionIdOrError,
          // override target so the the request is sent to proper endpoint
          target: button.post_url || sourceFrame.postUrl || button.target
        }
      },
      {
        // we are continuing with the same pending item
        preflightRequest: {
          pendingFrameStackItem: pendingItem,
          startTime
        },
        onError(error) {
          tryCall(() => onTransactionProcessingError?.(error));
        },
        onSuccess() {
          tryCall(
            () => onTransactionProcessingSuccess?.({
              button,
              transactionId: transactionIdOrError
            })
          );
        }
      }
    );
  }
  return (request, shouldClear = false) => {
    if (request.method === "GET") {
      return fetchGETRequest(request, shouldClear);
    }
    if (request.frameButton.action === "tx") {
      return fetchTransactionRequest(request, shouldClear);
    }
    return fetchPOSTRequest(request, {
      shouldClear
    });
  };
}
function proxyUrlAndSearchParamsToUrl(proxyUrl, ...searchParams) {
  const temporaryDomain = "temporary-for-parsing-purposes.tld";
  const parsedProxyUrl = new URL(proxyUrl, `http://${temporaryDomain}`);
  searchParams.forEach((params) => {
    params.forEach((value, key) => {
      parsedProxyUrl.searchParams.set(key, value);
    });
  });
  return parsedProxyUrl.hostname === temporaryDomain ? `${parsedProxyUrl.pathname}${parsedProxyUrl.search}` : parsedProxyUrl.toString();
}
async function fetchProxied(params) {
  const searchParams = new URLSearchParams({
    multispecification: "true"
  });
  if ("frameAction" in params) {
    const proxyUrl2 = proxyUrlAndSearchParamsToUrl(
      params.proxyUrl,
      searchParams,
      params.frameAction.searchParams
    );
    return tryCallAsync(
      () => params.fetchFn(proxyUrl2, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...params.extraRequestPayload,
          ...params.frameAction.body
        })
      })
    );
  }
  if (params.parseFarcasterManifest) {
    searchParams.set("parseFarcasterManifest", "true");
  }
  searchParams.set("url", params.url);
  const proxyUrl = proxyUrlAndSearchParamsToUrl(params.proxyUrl, searchParams);
  return tryCallAsync(
    () => params.fetchFn(proxyUrl, { method: "GET", signal: params.signal })
  );
}
function getResponseBody(response) {
  if (response.headers.get("content-type")?.includes("/json")) {
    return response.clone().json();
  }
  return response.clone().text();
}
async function signAndGetFrameActionBodyPayload({
  request,
  signerState
}) {
  const { address: _, ...requiredFrameContext } = request.signerStateActionContext.frameContext;
  return tryCallAsync(
    () => signerState.signFrameAction({
      ...request.signerStateActionContext,
      frameContext: requiredFrameContext
    })
  );
}

// src/unstable-use-frame.tsx
function onErrorFallback(e) {
  console.error("@frames.js/render", e);
}
function onMintFallback({ target }) {
  console.log("Please provide your own onMint function to useFrame() hook.");
  const message = `Mint requested: ${target}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
}
function resolveAddressFallback() {
  throw new Error(
    "Please implement this function in order to use transactions"
  );
}
async function onTransactionFallback({
  transactionData
}) {
  console.log(
    "Please provide your own onTransaction function to useFrame() hook."
  );
  const message = `Requesting a transaction on chain with ID ${transactionData.chainId} with the following params: ${JSON.stringify(
    transactionData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
async function onSignatureFallback({
  signatureData
}) {
  console.log(
    "Please provide your own onSignature function to useFrame() hook."
  );
  const message = `Requesting a signature on chain with ID ${signatureData.chainId} with the following params: ${JSON.stringify(
    signatureData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
function handleRedirectFallback(location) {
  console.log(
    "Please provide your own onRedirect function to useFetchFrame() hook."
  );
  const message = `You are about to be redirected to ${location.toString()}`;
  if (typeof window !== "undefined") {
    if (window.confirm(message)) {
      window.open(location, "_blank")?.focus();
    }
  } else {
    console.log(message);
  }
}
function handleLinkButtonClickFallback(button) {
  console.log(
    "Please provide your own onLinkButtonClick function to useFrame() hook."
  );
  if (typeof window !== "undefined") {
    if (window.confirm(`You are about to be redirected to ${button.target}`)) {
      parent.window.open(button.target, "_blank");
    }
  } else {
    console.log(`Link button with target ${button.target} clicked.`);
  }
}
function validateLinkButtonTarget(target) {
  const locationUrl = new URL(target);
  if (locationUrl.protocol !== "http:" && locationUrl.protocol !== "https:") {
    throw new Error(
      `Redirect location ${locationUrl.toString()} is not a valid HTTP or HTTPS URL.`
    );
  }
  return true;
}
function defaultFetchFunction(input, init) {
  return fetch(input, init);
}
function useFrame_unstable({
  frameStateHook: useFrameStateHook = useFrameState,
  homeframeUrl,
  onMint = onMintFallback,
  onTransaction = onTransactionFallback,
  transactionDataSuffix,
  onSignature = onSignatureFallback,
  resolveAddress = resolveAddressFallback,
  frame,
  /** Ex: /frames */
  frameActionProxy,
  /** Ex: /frames */
  frameGetProxy,
  parseFarcasterManifest = false,
  extraButtonRequestPayload,
  resolveSigner: resolveSpecification,
  onError = onErrorFallback,
  onLinkButtonClick = handleLinkButtonClickFallback,
  onRedirect = handleRedirectFallback,
  fetchFn = defaultFetchFunction,
  onLaunchFrameButtonPressed,
  onTransactionDataError,
  onTransactionDataStart,
  onTransactionDataSuccess,
  onTransactionError,
  onTransactionProcessingError,
  onTransactionProcessingStart,
  onTransactionProcessingSuccess,
  onTransactionStart,
  onTransactionSuccess
}) {
  const [inputText, setInputText] = (0, import_react3.useState)("");
  const inputTextRef = useFreshRef(inputText);
  const [frameState, frameStateAPI] = useFrameStateHook({
    resolveSpecification,
    initialFrameUrl: homeframeUrl,
    initialParseResult: frame
  });
  const frameStateRef = useFreshRef(frameState);
  const {
    clear: clearFrameState,
    dispatch: dispatchFrameState,
    reset: resetFrameState
  } = frameStateAPI;
  const fetchFrame = useFetchFrame({
    frameState,
    frameStateAPI,
    frameActionProxy,
    frameGetProxy,
    onTransaction,
    transactionDataSuffix,
    onSignature,
    extraButtonRequestPayload,
    onError,
    fetchFn,
    onRedirect,
    onTransactionDataError,
    onTransactionDataStart,
    onTransactionDataSuccess,
    onTransactionError,
    onTransactionProcessingError,
    onTransactionProcessingStart,
    onTransactionProcessingSuccess,
    onTransactionStart,
    onTransactionSuccess,
    parseFarcasterManifest
  });
  const fetchFrameRef = useFreshRef(fetchFrame);
  const onErrorRef = useFreshRef(onError);
  (0, import_react3.useEffect)(() => {
    if (!homeframeUrl) {
      clearFrameState();
      return;
    }
    if (!frame) {
      fetchFrameRef.current(
        {
          url: homeframeUrl,
          method: "GET"
        },
        // tell the fetchFrame function to clear the stack because this is called only on initial render
        // and there could potentially be a pending object returned from SSR
        true
      ).catch((e) => {
        onErrorRef.current(e instanceof Error ? e : new Error(String(e)));
      });
    } else {
      resetFrameState({
        homeframeUrl,
        parseResult: frame
      });
    }
  }, [
    frame,
    homeframeUrl,
    clearFrameState,
    fetchFrameRef,
    resetFrameState,
    onErrorRef
  ]);
  const onPostButton = (0, import_react3.useCallback)(
    async function onPostButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton,
      target,
      state,
      fetchFrameOverride
    }) {
      const currentState = frameStateRef.current;
      if (currentState.type === "not-initialized") {
        const error = new Error(
          "Cannot perform post/post_redirect without a frame"
        );
        onErrorRef.current(error);
        return;
      }
      if (!currentState.signerState.hasSigner) {
        await currentState.signerState.onSignerlessFramePress();
        return;
      }
      const _fetchFrame = fetchFrameOverride ?? fetchFrameRef.current;
      await _fetchFrame({
        frameButton,
        isDangerousSkipSigning: false,
        method: "POST",
        signerStateActionContext: {
          inputText: postInputText,
          signer: currentState.signerState.signer,
          frameContext: currentState.frameContext,
          url: currentState.homeframeUrl,
          target,
          frameButton,
          buttonIndex,
          state
        },
        sourceFrame: currentFrame
      });
    },
    [fetchFrameRef, frameStateRef, onErrorRef]
  );
  const resolveAddressRef = useFreshRef(resolveAddress);
  const onTransactionButton = (0, import_react3.useCallback)(
    async function onTransactionButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton
    }) {
      const currentState = frameStateRef.current;
      if (currentState.type === "not-initialized") {
        const error = new Error("Cannot perform transaction without a frame");
        onErrorRef.current(error);
        return;
      }
      if (!currentState.signerState.hasSigner) {
        await currentState.signerState.onSignerlessFramePress();
        return;
      }
      const addressOrError = await tryCallAsync(
        () => resolveAddressRef.current()
      );
      if (addressOrError instanceof Error) {
        onErrorRef.current(addressOrError);
        return;
      }
      if (!addressOrError) {
        onErrorRef.current(
          new Error(
            "Wallet address missing, please check resolveAddress() function passed to useFrame_unstable() hook."
          )
        );
        return;
      }
      const frameContext = {
        ...currentState.frameContext,
        address: addressOrError
      };
      await fetchFrameRef.current({
        frameButton,
        isDangerousSkipSigning: false,
        method: "POST",
        signerStateActionContext: {
          type: "tx-data",
          inputText: postInputText,
          signer: currentState.signerState.signer,
          frameContext,
          address: addressOrError,
          url: currentState.homeframeUrl,
          target: frameButton.target,
          frameButton,
          buttonIndex,
          state: currentFrame.state
        },
        sourceFrame: currentFrame
      });
    },
    [frameStateRef, fetchFrameRef, onErrorRef, resolveAddressRef]
  );
  const onLaunchFrameButtonPressRef = useFreshRef(onLaunchFrameButtonPressed);
  const onLaunchFrameButtonPress = (0, import_react3.useCallback)(
    (event) => {
      onLaunchFrameButtonPressRef.current?.(event);
    },
    [onLaunchFrameButtonPressRef]
  );
  const onButtonPress = (0, import_react3.useCallback)(
    async function onButtonPress2(currentFrame, frameButton, index, fetchFrameOverride) {
      switch (frameButton.action) {
        case "link": {
          try {
            validateLinkButtonTarget(frameButton.target);
          } catch (error) {
            if (error instanceof Error) {
              onErrorRef.current(error);
            }
            return;
          }
          onLinkButtonClick(frameButton);
          break;
        }
        case "mint": {
          onMint({
            frameButton,
            target: frameButton.target,
            frame: currentFrame
          });
          break;
        }
        case "tx": {
          await onTransactionButton({
            frameButton,
            buttonIndex: index + 1,
            postInputText: currentFrame.inputText !== void 0 ? inputTextRef.current : void 0,
            currentFrame
          });
          break;
        }
        case "post":
        case "post_redirect": {
          try {
            const target = frameButton.target || frameButton.post_url || currentFrame.postUrl || homeframeUrl;
            if (!target) {
              onErrorRef.current(new Error(`Missing target`));
              return;
            }
            try {
              validateLinkButtonTarget(target);
            } catch (error) {
              if (error instanceof Error) {
                onErrorRef.current(error);
              }
              return;
            }
            await onPostButton({
              currentFrame,
              frameButton,
              /** https://docs.farcaster.xyz/reference/frames/spec#handling-clicks
              
                      POST the packet to fc:frame:button:$idx:action:target if present
                      POST the packet to fc:frame:post_url if target was not present.
                      POST the packet to or the frame's embed URL if neither target nor action were present.
                      */
              target,
              buttonIndex: index + 1,
              postInputText: currentFrame.inputText !== void 0 ? inputTextRef.current : void 0,
              state: currentFrame.state,
              fetchFrameOverride
            });
            setInputText("");
          } catch (err) {
            onErrorRef.current(
              err instanceof Error ? err : new Error(String(err))
            );
          }
          break;
        }
        default:
          throw new Error("Unrecognized frame button action");
      }
    },
    [
      homeframeUrl,
      inputTextRef,
      onErrorRef,
      onLinkButtonClick,
      onMint,
      onPostButton,
      onTransactionButton
    ]
  );
  const { stack } = frameState;
  const { signerState, specification } = frameState.type === "initialized" ? frameState : { signerState: void 0, specification: void 0 };
  return (0, import_react3.useMemo)(() => {
    return {
      signerState,
      specification,
      inputText,
      setInputText,
      clearFrameStack: clearFrameState,
      dispatchFrameStack: dispatchFrameState,
      reset: resetFrameState,
      onButtonPress,
      fetchFrame,
      homeframeUrl,
      framesStack: stack,
      currentFrameStackItem: stack[0],
      onLaunchFrameButtonPress
    };
  }, [
    signerState,
    specification,
    inputText,
    clearFrameState,
    dispatchFrameState,
    onButtonPress,
    resetFrameState,
    fetchFrame,
    homeframeUrl,
    stack,
    onLaunchFrameButtonPress
  ]);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useFrame_unstable
});
