"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-cast-action.ts
var use_cast_action_exports = {};
__export(use_cast_action_exports, {
  useCastAction: () => useCastAction
});
module.exports = __toCommonJS(use_cast_action_exports);
var import_react2 = require("react");

// src/hooks/use-fresh-ref.ts
var import_react = require("react");
function useFreshRef(value) {
  const ref = (0, import_react.useRef)(value);
  ref.current = value;
  return ref;
}

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}
function tryCall(fn) {
  try {
    return fn();
  } catch (e) {
    if (e instanceof Error) {
      return e;
    }
    return new TypeError("Unexpected error, check the console for details");
  }
}
function isParseFramesWithReportsResult(value) {
  return typeof value === "object" && value !== null && "openframes" in value && "farcaster" in value;
}
function isCastActionFrameResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "frame" && "frameUrl" in response && typeof response.frameUrl === "string";
}
function isCastActionMessageResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "message" && "message" in response && typeof response.message === "string";
}
function mergeSearchParamsToUrl(url, ...searchParams) {
  const temporaryDomain = "temporary-for-parsing-purposes.tld";
  const parsedProxyUrl = new URL(url, `http://${temporaryDomain}`);
  searchParams.forEach((params) => {
    params.forEach((value, key) => {
      parsedProxyUrl.searchParams.set(key, value);
    });
  });
  return parsedProxyUrl.hostname === temporaryDomain ? `${parsedProxyUrl.pathname}${parsedProxyUrl.search}` : parsedProxyUrl.toString();
}

// src/errors.ts
var CastActionUnexpectedResponseError = class extends Error {
  constructor() {
    super("Unexpected cast action response from the server");
  }
};

// src/use-cast-action.ts
function useCastAction({
  castId,
  enabled = true,
  fetch: fetchFunction = defaultFetchFn,
  onError,
  proxyUrl,
  signer,
  postUrl,
  onFrameResponse,
  onMessageResponse,
  onInvalidResponse
}) {
  const onErrorRef = useFreshRef(onError);
  const fetchRef = useFreshRef(fetchFunction);
  const onFrameResponseRef = useFreshRef(onFrameResponse);
  const onMessageResponseRef = useFreshRef(onMessageResponse);
  const onInvalidResponseRef = useFreshRef(onInvalidResponse);
  const signerRef = useFreshRef(signer);
  const lastFetchActionArgRef = (0, import_react2.useRef)(null);
  const [state, dispatch] = (0, import_react2.useReducer)(castActionReducer, {
    status: "idle",
    enabled
  });
  const castIdRef = useFreshRef(castId);
  const fetchAction = (0, import_react2.useCallback)(
    async (arg) => {
      const currentSigner = arg.signer;
      if (currentSigner?.status !== "approved" && currentSigner?.status !== "impersonating") {
        await signerRef.current.onSignerlessFramePress();
        return;
      }
      dispatch({ type: "loading-url" });
      const signedDataOrError = await tryCallAsync(
        () => signerRef.current.signCastAction(currentSigner.privateKey, {
          postUrl: arg.postUrl,
          fid: currentSigner.fid,
          castId: arg.castId
        })
      );
      if (signedDataOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(signedDataOrError));
        dispatch({ type: "error", error: signedDataOrError });
        return;
      }
      const proxiedUrl = mergeSearchParamsToUrl(
        arg.proxyUrl,
        new URLSearchParams({ postUrl: arg.postUrl })
      );
      const actionResponseOrError = await tryCallAsync(
        () => fetchRef.current(proxiedUrl, {
          method: "POST",
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify(signedDataOrError),
          cache: "no-cache"
        })
      );
      if (actionResponseOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(actionResponseOrError));
        dispatch({ type: "error", error: actionResponseOrError });
        return;
      }
      if (!actionResponseOrError.ok) {
        const error2 = new Error(
          `Unexpected response status ${actionResponseOrError.status}`
        );
        tryCall(() => onErrorRef.current?.(error2));
        dispatch({ type: "error", error: error2 });
        return;
      }
      const actionResponseDataOrError = await tryCallAsync(
        () => actionResponseOrError.clone().json()
      );
      if (actionResponseDataOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(actionResponseDataOrError));
        dispatch({ type: "error", error: actionResponseDataOrError });
        return;
      }
      if (isCastActionFrameResponse(actionResponseDataOrError)) {
        tryCall(() => onFrameResponseRef.current?.(actionResponseDataOrError));
        const signedFrameDataOrError = await tryCallAsync(() => {
          return signerRef.current.signFrameAction({
            buttonIndex: 1,
            frameButton: {
              action: "post",
              label: "cast action",
              post_url: actionResponseDataOrError.frameUrl,
              target: actionResponseDataOrError.frameUrl
            },
            frameContext: {
              castId: castIdRef.current
            },
            signer: currentSigner,
            url: actionResponseDataOrError.frameUrl,
            type: "default"
          });
        });
        if (signedFrameDataOrError instanceof Error) {
          tryCall(() => onErrorRef.current?.(signedFrameDataOrError));
          dispatch({ type: "error", error: signedFrameDataOrError });
          return;
        }
        const proxiedFrameUrl = mergeSearchParamsToUrl(
          arg.proxyUrl,
          signedFrameDataOrError.searchParams,
          new URLSearchParams({
            multispecification: "true",
            postUrl: actionResponseDataOrError.frameUrl
          })
        );
        const frameResponseOrError = await tryCallAsync(() => {
          return fetchRef.current(proxiedFrameUrl, {
            method: "POST",
            headers: {
              "Content-Type": "application/json"
            },
            body: JSON.stringify(signedFrameDataOrError.body),
            cache: "no-cache"
          });
        });
        if (frameResponseOrError instanceof Error) {
          tryCall(() => onErrorRef.current?.(frameResponseOrError));
          dispatch({ type: "error", error: frameResponseOrError });
          return;
        }
        if (!frameResponseOrError.ok) {
          const error2 = new Error(
            `Unexpected response status ${frameResponseOrError.status}`
          );
          tryCall(() => onErrorRef.current?.(error2));
          dispatch({ type: "error", error: error2 });
          return;
        }
        const frameResponseDataOrError = await tryCallAsync(
          () => frameResponseOrError.clone().json()
        );
        if (frameResponseDataOrError instanceof Error) {
          tryCall(() => onErrorRef.current?.(frameResponseDataOrError));
          dispatch({ type: "error", error: frameResponseDataOrError });
          return;
        }
        if (!isParseFramesWithReportsResult(frameResponseDataOrError)) {
          const error2 = new Error("Invalid frame response");
          tryCall(() => onErrorRef.current?.(error2));
          dispatch({ type: "error", error: error2 });
          return;
        }
        dispatch({
          type: "frame",
          response: actionResponseDataOrError,
          frame: frameResponseDataOrError
        });
        return;
      }
      if (isCastActionMessageResponse(actionResponseDataOrError)) {
        tryCall(
          () => onMessageResponseRef.current?.(actionResponseDataOrError)
        );
        dispatch({
          type: "message",
          response: actionResponseDataOrError
        });
        return;
      }
      tryCall(() => onInvalidResponseRef.current?.(actionResponseDataOrError));
      const error = new CastActionUnexpectedResponseError();
      tryCall(() => onErrorRef.current?.(error));
      dispatch({ type: "error", error });
    },
    [
      castIdRef,
      fetchRef,
      onErrorRef,
      onFrameResponseRef,
      onInvalidResponseRef,
      onMessageResponseRef,
      signerRef
    ]
  );
  const stateRef = useFreshRef(state);
  const refetch = (0, import_react2.useCallback)(() => {
    if (!stateRef.current.enabled || !lastFetchActionArgRef.current) {
      return Promise.resolve();
    }
    return fetchAction(lastFetchActionArgRef.current);
  }, [fetchAction, stateRef]);
  (0, import_react2.useEffect)(() => {
    dispatch({ type: "enabled-change", enabled });
  }, [enabled]);
  (0, import_react2.useEffect)(() => {
    if (!enabled || !postUrl) {
      return;
    }
    lastFetchActionArgRef.current = {
      signer: signer.signer,
      postUrl,
      proxyUrl,
      castId
    };
    fetchAction(lastFetchActionArgRef.current).catch((e) => {
      onErrorRef.current?.(e instanceof Error ? e : new Error(String(e)));
    });
  }, [
    castId,
    postUrl,
    proxyUrl,
    signer.signer,
    enabled,
    fetchAction,
    onErrorRef
  ]);
  return (0, import_react2.useMemo)(() => {
    switch (state.status) {
      case "idle":
        return {
          status: "idle",
          data: void 0,
          error: void 0,
          refetch
        };
      case "loading":
        return {
          status: "loading",
          data: void 0,
          error: void 0,
          refetch
        };
      case "error":
        return {
          status: "error",
          data: void 0,
          error: state.error,
          refetch
        };
      default: {
        if (state.type === "frame") {
          return {
            status: "success",
            type: "frame",
            data: state.response,
            frame: state.frame,
            error: void 0,
            refetch
          };
        }
        return {
          status: "success",
          type: "message",
          data: state.response,
          error: void 0,
          refetch
        };
      }
    }
  }, [state, refetch]);
}
function defaultFetchFn(url, init) {
  return fetch(url, init);
}
function castActionReducer(state, action) {
  if (action.type === "enabled-change") {
    if (action.enabled) {
      return {
        ...state,
        enabled: true
      };
    }
    return {
      status: "idle",
      enabled: false
    };
  }
  if (!state.enabled) {
    return state;
  }
  switch (action.type) {
    case "frame":
      return {
        ...state,
        status: "success",
        type: "frame",
        response: action.response,
        frame: action.frame
      };
    case "message":
      return {
        ...state,
        status: "success",
        type: "message",
        response: action.response
      };
    case "loading-url":
      return {
        ...state,
        status: "loading"
      };
    case "error":
      return {
        ...state,
        status: "error",
        error: action.error
      };
    default:
      return state;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useCastAction
});
