"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-composer-action.ts
var use_composer_action_exports = {};
__export(use_composer_action_exports, {
  defaultRegisterMessageListener: () => defaultRegisterMessageListener,
  miniAppMessageSchema: () => miniAppMessageSchema,
  useComposerAction: () => useComposerAction
});
module.exports = __toCommonJS(use_composer_action_exports);
var import_react2 = require("react");

// src/hooks/use-fresh-ref.ts
var import_react = require("react");
function useFreshRef(value) {
  const ref = (0, import_react.useRef)(value);
  ref.current = value;
  return ref;
}

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}
function tryCall(fn) {
  try {
    return fn();
  } catch (e) {
    if (e instanceof Error) {
      return e;
    }
    return new TypeError("Unexpected error, check the console for details");
  }
}
function isComposerFormActionResponse(response) {
  return typeof response === "object" && response !== null && "type" in response && response.type === "form";
}
function mergeSearchParamsToUrl(url, ...searchParams) {
  const temporaryDomain = "temporary-for-parsing-purposes.tld";
  const parsedProxyUrl = new URL(url, `http://${temporaryDomain}`);
  searchParams.forEach((params) => {
    params.forEach((value, key) => {
      parsedProxyUrl.searchParams.set(key, value);
    });
  });
  return parsedProxyUrl.hostname === temporaryDomain ? `${parsedProxyUrl.pathname}${parsedProxyUrl.search}` : parsedProxyUrl.toString();
}

// src/errors.ts
var ComposerActionUnexpectedResponseError = class extends Error {
  constructor() {
    super("Unexpected composer action response from the server");
  }
};
var ComposerActionUserRejectedRequestError = class extends Error {
  constructor() {
    super("User rejected the request");
  }
};

// src/mini-app-messages.ts
var import_zod = require("zod");
var createCastRequestSchemaLegacy = import_zod.z.object({
  type: import_zod.z.literal("createCast"),
  data: import_zod.z.object({
    cast: import_zod.z.object({
      parent: import_zod.z.string().optional(),
      text: import_zod.z.string(),
      embeds: import_zod.z.array(import_zod.z.string().min(1).url()).min(1)
    })
  })
});
var createCastRequestSchema = import_zod.z.object({
  jsonrpc: import_zod.z.literal("2.0"),
  id: import_zod.z.union([import_zod.z.string(), import_zod.z.number(), import_zod.z.null()]),
  method: import_zod.z.literal("fc_createCast"),
  params: import_zod.z.object({
    cast: import_zod.z.object({
      parent: import_zod.z.string().optional(),
      text: import_zod.z.string(),
      embeds: import_zod.z.array(import_zod.z.string().min(1).url()).min(1)
    })
  })
});
var ethSendTransactionActionSchema = import_zod.z.object({
  chainId: import_zod.z.string(),
  method: import_zod.z.literal("eth_sendTransaction"),
  attribution: import_zod.z.boolean().optional(),
  params: import_zod.z.object({
    abi: import_zod.z.custom(),
    to: import_zod.z.custom(
      (val) => typeof val === "string" && val.startsWith("0x")
    ),
    value: import_zod.z.string().optional(),
    data: import_zod.z.custom((val) => typeof val === "string" && val.startsWith("0x")).optional()
  })
});
var ethSignTypedDataV4ActionSchema = import_zod.z.object({
  chainId: import_zod.z.string(),
  method: import_zod.z.literal("eth_signTypedData_v4"),
  params: import_zod.z.object({
    domain: import_zod.z.custom(),
    types: import_zod.z.custom((value) => {
      const result = import_zod.z.record(import_zod.z.unknown()).safeParse(value);
      return result.success;
    }),
    primaryType: import_zod.z.string(),
    message: import_zod.z.record(import_zod.z.unknown())
  })
});
var walletActionRequestSchema = import_zod.z.object({
  jsonrpc: import_zod.z.literal("2.0"),
  id: import_zod.z.string(),
  method: import_zod.z.literal("fc_requestWalletAction"),
  params: import_zod.z.object({
    action: import_zod.z.union([
      ethSendTransactionActionSchema,
      ethSignTypedDataV4ActionSchema
    ])
  })
});
var miniAppMessageSchema = import_zod.z.union([
  createCastRequestSchemaLegacy,
  walletActionRequestSchema,
  createCastRequestSchema
]);

// src/use-composer-action.ts
function useComposerAction({
  actionState,
  enabled = true,
  proxyUrl,
  signer,
  url,
  fetch: fetchFunction,
  onError,
  onCreateCast,
  onSignature,
  onTransaction,
  resolveAddress,
  registerMessageListener = defaultRegisterMessageListener,
  onMessageRespond
}) {
  const onErrorRef = useFreshRef(onError);
  const [state, dispatch] = (0, import_react2.useReducer)(composerActionReducer, {
    status: "idle",
    enabled
  });
  const registerMessageListenerRef = useFreshRef(registerMessageListener);
  const actionStateRef = useFreshRef(actionState);
  const onCreateCastRef = useFreshRef(onCreateCast);
  const onMessageRespondRef = useFreshRef(onMessageRespond);
  const onTransactionRef = useFreshRef(onTransaction);
  const onSignatureRef = useFreshRef(onSignature);
  const resolveAddressRef = useFreshRef(resolveAddress);
  const lastFetchActionArgRef = (0, import_react2.useRef)(
    null
  );
  const signerRef = useFreshRef(signer);
  const fetchRef = useFreshRef(fetchFunction);
  const messageListener = (0, import_react2.useCallback)(
    async (successState, message) => {
      if ("type" in message || message.method === "fc_createCast") {
        const cast = "type" in message ? message.data.cast : message.params.cast;
        const resultOrError = await tryCallAsync(
          () => onCreateCastRef.current({
            cast
          })
        );
        if (resultOrError instanceof Error) {
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: "method" in message ? message.id : null,
              error: {
                code: -32e3,
                message: resultOrError.message
              }
            },
            successState.response
          );
        }
        onMessageRespondRef.current(
          {
            jsonrpc: "2.0",
            id: "method" in message ? message.id : null,
            result: {
              success: true
            }
          },
          successState.response
        );
      } else if (message.method === "fc_requestWalletAction") {
        const addressOrError = await tryCallAsync(
          () => resolveAddressRef.current()
        );
        if (addressOrError instanceof Error) {
          tryCall(() => onErrorRef.current?.(addressOrError));
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: message.id,
              error: {
                code: -32e3,
                message: addressOrError.message
              }
            },
            successState.response
          );
          return;
        }
        if (!addressOrError) {
          return;
        }
        if (message.params.action.method === "eth_sendTransaction") {
          const action = message.params.action;
          const resultOrError = await tryCallAsync(
            () => onTransactionRef.current({
              action,
              address: addressOrError
            })
          );
          if (resultOrError instanceof Error) {
            tryCall(() => onErrorRef.current?.(resultOrError));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: resultOrError.message
                }
              },
              successState.response
            );
            return;
          }
          if (!resultOrError) {
            const error = new ComposerActionUserRejectedRequestError();
            tryCall(() => onErrorRef.current?.(error));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: error.message
                }
              },
              successState.response
            );
            return;
          }
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: message.id,
              result: {
                address: resultOrError.address,
                transactionHash: resultOrError.hash
              }
            },
            successState.response
          );
        } else if (message.params.action.method === "eth_signTypedData_v4") {
          const action = message.params.action;
          const resultOrError = await tryCallAsync(
            () => onSignatureRef.current({
              action,
              address: addressOrError
            })
          );
          if (resultOrError instanceof Error) {
            tryCall(() => onErrorRef.current?.(resultOrError));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: resultOrError.message
                }
              },
              successState.response
            );
            return;
          }
          if (!resultOrError) {
            const error = new ComposerActionUserRejectedRequestError();
            tryCall(() => onErrorRef.current?.(error));
            onMessageRespondRef.current(
              {
                jsonrpc: "2.0",
                id: message.id,
                error: {
                  code: -32e3,
                  message: error.message
                }
              },
              successState.response
            );
            return;
          }
          onMessageRespondRef.current(
            {
              jsonrpc: "2.0",
              id: message.id,
              result: {
                address: resultOrError.address,
                signature: resultOrError.hash
              }
            },
            successState.response
          );
        } else {
          tryCall(
            () => onErrorRef.current?.(
              new Error(
                `Unknown fc_requestWalletAction action method: ${message.params.action.method}`
              )
            )
          );
        }
      } else {
        tryCall(() => onErrorRef.current?.(new Error("Unknown message")));
      }
    },
    [
      onCreateCastRef,
      onErrorRef,
      onMessageRespondRef,
      onSignatureRef,
      onTransactionRef,
      resolveAddressRef
    ]
  );
  const fetchAction = (0, import_react2.useCallback)(
    async (arg) => {
      const currentSigner = arg.signer;
      if (currentSigner?.status !== "approved" && currentSigner?.status !== "impersonating") {
        await signerRef.current.onSignerlessFramePress();
        return;
      }
      dispatch({ type: "loading-url" });
      const signedDataOrError = await tryCallAsync(
        () => signerRef.current.signComposerAction(currentSigner.privateKey, {
          url: arg.url,
          state: arg.actionState,
          fid: currentSigner.fid
        })
      );
      if (signedDataOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(signedDataOrError));
        dispatch({ type: "error", error: signedDataOrError });
        return;
      }
      const proxiedUrl = mergeSearchParamsToUrl(
        arg.proxyUrl,
        new URLSearchParams({ postUrl: arg.url })
      );
      const actionResponseOrError = await tryCallAsync(
        () => (fetchRef.current || fetch)(proxiedUrl, {
          method: "POST",
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify(signedDataOrError),
          cache: "no-cache"
        })
      );
      if (actionResponseOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(actionResponseOrError));
        dispatch({ type: "error", error: actionResponseOrError });
        return;
      }
      if (!actionResponseOrError.ok) {
        const error = new Error(
          `Unexpected response status ${actionResponseOrError.status}`
        );
        tryCall(() => onErrorRef.current?.(error));
        dispatch({ type: "error", error });
        return;
      }
      const actionResponseDataOrError = await tryCallAsync(
        () => actionResponseOrError.clone().json()
      );
      if (actionResponseDataOrError instanceof Error) {
        tryCall(() => onErrorRef.current?.(actionResponseDataOrError));
        dispatch({ type: "error", error: actionResponseDataOrError });
        return;
      }
      if (!isComposerFormActionResponse(actionResponseDataOrError)) {
        const error = new ComposerActionUnexpectedResponseError();
        tryCall(() => onErrorRef.current?.(error));
        dispatch({ type: "error", error });
        return;
      }
      dispatch({ type: "done", response: actionResponseDataOrError });
    },
    [fetchRef, onErrorRef, signerRef]
  );
  const stateRef = useFreshRef(state);
  const refetch = (0, import_react2.useCallback)(() => {
    if (!stateRef.current.enabled || !lastFetchActionArgRef.current) {
      return Promise.resolve();
    }
    return fetchAction(lastFetchActionArgRef.current);
  }, [fetchAction, stateRef]);
  (0, import_react2.useEffect)(() => {
    dispatch({ type: "enabled-change", enabled });
  }, [enabled]);
  (0, import_react2.useEffect)(() => {
    if (!enabled) {
      return;
    }
    lastFetchActionArgRef.current = {
      actionState: actionStateRef.current,
      signer: signer.signer,
      url,
      proxyUrl
    };
    fetchAction(lastFetchActionArgRef.current).catch((e) => {
      onErrorRef.current?.(e instanceof Error ? e : new Error(String(e)));
    });
  }, [
    url,
    proxyUrl,
    signer.signer,
    enabled,
    fetchAction,
    actionStateRef,
    onErrorRef
  ]);
  (0, import_react2.useEffect)(() => {
    if (state.status === "success") {
      return registerMessageListenerRef.current(
        state.response,
        messageListener.bind(null, state)
      );
    }
  }, [messageListener, registerMessageListenerRef, state]);
  return (0, import_react2.useMemo)(() => {
    switch (state.status) {
      case "idle":
        return {
          status: "idle",
          data: void 0,
          error: void 0,
          refetch
        };
      case "loading":
        return {
          status: "loading",
          data: void 0,
          error: void 0,
          refetch
        };
      case "error":
        return {
          status: "error",
          data: void 0,
          error: state.error,
          refetch
        };
      default:
        return {
          status: "success",
          data: state.response,
          error: void 0,
          refetch
        };
    }
  }, [state, refetch]);
}
var defaultRegisterMessageListener = function defaultRegisterMessageListener2(formResponse, messageListener) {
  if (typeof window === "undefined") {
    console.warn(
      "@frames.js/render: You are using default registerMessageListener in an environment without window object"
    );
    return () => {
    };
  }
  const miniAppOrigin = new URL(formResponse.url).origin;
  const messageParserListener = (event) => {
    if (event.origin !== miniAppOrigin) {
      return;
    }
    const result = miniAppMessageSchema.safeParse(event.data);
    if (!result.success) {
      console.warn(
        "@frames.js/render: Invalid message received",
        event.data,
        result.error
      );
      return;
    }
    const message = result.data;
    messageListener(message).catch((e) => {
      console.error(`@frames.js/render:`, e);
    });
  };
  window.addEventListener("message", messageParserListener);
  return () => {
    window.removeEventListener("message", messageParserListener);
  };
};
function composerActionReducer(state, action) {
  if (action.type === "enabled-change") {
    if (action.enabled) {
      return {
        ...state,
        enabled: true
      };
    }
    return {
      status: "idle",
      enabled: false
    };
  }
  if (!state.enabled) {
    return state;
  }
  switch (action.type) {
    case "done":
      return {
        ...state,
        status: "success",
        response: action.response
      };
    case "loading-url":
      return {
        ...state,
        status: "loading"
      };
    case "error":
      return {
        ...state,
        status: "error",
        error: action.error
      };
    default:
      return state;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  defaultRegisterMessageListener,
  miniAppMessageSchema,
  useComposerAction
});
