"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-frame-app.ts
var use_frame_app_exports = {};
__export(use_frame_app_exports, {
  useFrameApp: () => useFrameApp
});
module.exports = __toCommonJS(use_frame_app_exports);
var import_frame_host = require("@farcaster/frame-host");
var import_react5 = require("react");

// src/hooks/use-fresh-ref.ts
var import_react = require("react");
function useFreshRef(value) {
  const ref = (0, import_react.useRef)(value);
  ref.current = value;
  return ref;
}

// src/assert-never.ts
function assertNever(x) {
  throw new Error(`Unhandled value ${JSON.stringify(x)}`);
}

// src/frame-app/use-fetch-frame-app.ts
var import_react2 = require("react");

// src/unstable-use-fetch-frame.ts
var import_viem = require("viem");

// src/helpers.ts
async function tryCallAsync(promiseFn) {
  try {
    return promiseFn().catch((e) => {
      if (e instanceof Error) {
        return e;
      }
      return new TypeError("Unexpected error, check the console for details");
    });
  } catch (e) {
    return new TypeError("Unexpected error, check the console for details");
  }
}
function isParseFramesWithReportsResult(value) {
  return typeof value === "object" && value !== null && "openframes" in value && "farcaster" in value;
}

// src/unstable-use-fetch-frame.ts
function proxyUrlAndSearchParamsToUrl(proxyUrl, ...searchParams) {
  const temporaryDomain = "temporary-for-parsing-purposes.tld";
  const parsedProxyUrl = new URL(proxyUrl, `http://${temporaryDomain}`);
  searchParams.forEach((params) => {
    params.forEach((value, key) => {
      parsedProxyUrl.searchParams.set(key, value);
    });
  });
  return parsedProxyUrl.hostname === temporaryDomain ? `${parsedProxyUrl.pathname}${parsedProxyUrl.search}` : parsedProxyUrl.toString();
}
async function fetchProxied(params) {
  const searchParams = new URLSearchParams({
    multispecification: "true"
  });
  if ("frameAction" in params) {
    const proxyUrl2 = proxyUrlAndSearchParamsToUrl(
      params.proxyUrl,
      searchParams,
      params.frameAction.searchParams
    );
    return tryCallAsync(
      () => params.fetchFn(proxyUrl2, {
        method: "POST",
        headers: {
          "Content-Type": "application/json"
        },
        body: JSON.stringify({
          ...params.extraRequestPayload,
          ...params.frameAction.body
        })
      })
    );
  }
  if (params.parseFarcasterManifest) {
    searchParams.set("parseFarcasterManifest", "true");
  }
  searchParams.set("url", params.url);
  const proxyUrl = proxyUrlAndSearchParamsToUrl(params.proxyUrl, searchParams);
  return tryCallAsync(
    () => params.fetchFn(proxyUrl, { method: "GET", signal: params.signal })
  );
}

// src/frame-app/use-fetch-frame-app.ts
var defaultFetchFunction = (...args) => fetch(...args);
function useFetchFrameApp({
  fetchFn,
  source,
  proxyUrl
}) {
  const abortControllerRef = (0, import_react2.useRef)(null);
  const fetchRef = useFreshRef(fetchFn ?? defaultFetchFunction);
  const [state, setState] = (0, import_react2.useState)(() => {
    if (typeof source === "string" || source instanceof URL) {
      return {
        status: "pending"
      };
    }
    return {
      status: "success",
      frame: source,
      source
    };
  });
  const fetchFrame = (0, import_react2.useCallback)(
    (sourceUrl) => {
      abortControllerRef.current?.abort();
      const abortController = new AbortController();
      abortControllerRef.current = abortController;
      Promise.resolve(sourceUrl).then(async (url) => {
        setState({
          status: "pending"
        });
        const responseOrError = await fetchProxied({
          fetchFn: fetchRef.current,
          url: url.toString(),
          parseFarcasterManifest: true,
          proxyUrl,
          signal: abortController.signal
        });
        if (responseOrError instanceof Error) {
          throw responseOrError;
        }
        if (!responseOrError.ok) {
          throw new Error(
            `Failed to fetch frame, server returned status ${responseOrError.status}`
          );
        }
        const data = await responseOrError.json();
        if (!isParseFramesWithReportsResult(data)) {
          throw new Error(
            "Invalid response, expected parse result, make sure you are using @frames.js/render proxy"
          );
        }
        if (abortController.signal.aborted) {
          return;
        }
        setState({
          status: "success",
          frame: data.farcaster_v2,
          source: sourceUrl
        });
      }).catch((e) => {
        if (abortController.signal.aborted) {
          return;
        }
        setState({
          status: "error",
          error: e instanceof Error ? e : new Error(String(e))
        });
      });
      return () => {
        abortController.abort();
      };
    },
    [fetchRef, proxyUrl]
  );
  (0, import_react2.useEffect)(() => {
    if (typeof source === "string" || source instanceof URL) {
      return fetchFrame(source);
    }
    setState((val) => {
      if (val.status === "success" && val.source === source) {
        return val;
      }
      return {
        status: "success",
        frame: source,
        source
      };
    });
  }, [source, fetchFrame]);
  return state;
}

// src/frame-app/use-resolve-client.ts
var import_react3 = require("react");
function useResolveClient({
  client
}) {
  const abortControllerRef = (0, import_react3.useRef)(null);
  const [state, setState] = (0, import_react3.useState)(() => {
    if (typeof client !== "function") {
      return {
        status: "success",
        client
      };
    }
    return {
      status: "pending"
    };
  });
  const resolveClient = (0, import_react3.useCallback)((resolve) => {
    abortControllerRef.current?.abort();
    const abortController = new AbortController();
    abortControllerRef.current = abortController;
    Promise.resolve().then(async () => {
      setState({
        status: "pending"
      });
      const resolvedClient = await resolve({
        signal: abortController.signal
      });
      if (abortController.signal.aborted) {
        return;
      }
      setState({
        status: "success",
        client: resolvedClient
      });
    }).catch((e) => {
      if (abortController.signal.aborted) {
        return;
      }
      setState({
        status: "error",
        error: e instanceof Error ? e : new Error(String(e))
      });
    });
    return () => {
      abortController.abort();
    };
  }, []);
  (0, import_react3.useEffect)(() => {
    if (typeof client !== "function") {
      setState((prevState) => {
        if (prevState.status === "success" && prevState.client !== client) {
          return {
            status: "success",
            client
          };
        }
        return prevState;
      });
    } else {
      resolveClient(client);
    }
  }, [client, resolveClient]);
  return state;
}

// src/hooks/use-debug-log.ts
var import_react4 = require("react");
function useDebugLog(prefix, enabled) {
  const enabledRef = useFreshRef(enabled);
  const prefixRef = useFreshRef(prefix);
  return (0, import_react4.useCallback)(
    (...args) => {
      if (!enabledRef.current) {
        return;
      }
      console.debug(prefixRef.current, ...args);
    },
    [enabledRef, prefixRef]
  );
}

// src/use-frame-app.ts
var defaultFrameRequestCache = /* @__PURE__ */ new Set();
var defaultOnAddFrameRequested = function defaultOnAddFrameRequested2() {
  console.error(
    "@frames.js/render/use-frame-app",
    "onAddFrameRequested not implemented"
  );
  return Promise.reject(new Error("onAddFrameRequested not implemented"));
};
var defaultOnSendTransactionRequest = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onSendTransactionRequest not implemented"
  );
  return Promise.resolve(true);
};
var defaultOnSignMessageRequest = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onSignMessageRequest not implemented"
  );
  return Promise.resolve(true);
};
var defaultOnSignTypedDataRequest = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onSignTypedDataRequest not implemented"
  );
  return Promise.resolve(true);
};
var defaultViewProfile = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onViewProfile not implemented"
  );
  return Promise.reject(new Error("onViewProfile not implemented"));
};
var defaultEIP6963RequestProviderRequested = () => {
  console.warn(
    "@frames.js/render/use-frame-app",
    "onEIP6963RequestProviderRequested not implemented"
  );
};
var defaultSignIn = () => {
  console.warn("@frames.js/render/use-frame-app", "onSignIn not implemented");
  return Promise.reject(new Error("onSignIn not implemented"));
};
var defaultLocation = {
  type: "launcher"
};
function useFrameApp({
  provider,
  client,
  location = defaultLocation,
  user,
  source,
  fetchFn,
  proxyUrl,
  onClose,
  onOpenUrl,
  onPrimaryButtonSet,
  onReady,
  debug = false,
  addFrameRequestsCache = defaultFrameRequestCache,
  onAddFrameRequested = defaultOnAddFrameRequested,
  onSendTransactionRequest = defaultOnSendTransactionRequest,
  onSignMessageRequest = defaultOnSignMessageRequest,
  onSignTypedDataRequest = defaultOnSignTypedDataRequest,
  onViewProfile = defaultViewProfile,
  onEIP6963RequestProviderRequested = defaultEIP6963RequestProviderRequested,
  onSignIn = defaultSignIn
}) {
  const providerRef = useFreshRef(provider);
  const debugRef = useFreshRef(debug);
  const locationRef = useFreshRef(location);
  const readyRef = useFreshRef(onReady);
  const closeRef = useFreshRef(onClose);
  const onOpenUrlRef = useFreshRef(onOpenUrl);
  const onPrimaryButtonSetRef = useFreshRef(onPrimaryButtonSet);
  const onViewProfileRef = useFreshRef(onViewProfile);
  const onEIP6963RequestProviderRequestedRef = useFreshRef(
    onEIP6963RequestProviderRequested
  );
  const onSignInRef = useFreshRef(onSignIn);
  const onAddFrameRequestedRef = useFreshRef(onAddFrameRequested);
  const addFrameRequestsCacheRef = useFreshRef(addFrameRequestsCache);
  const clientResolutionState = useResolveClient({ client });
  const frameResolutionState = useFetchFrameApp({
    source,
    fetchFn,
    proxyUrl
  });
  const logDebug = useDebugLog(
    "@frames.js/render/use-frame-app",
    debugRef.current
  );
  providerRef.current.setEventHandlers({
    onSendTransactionRequest,
    onSignMessageRequest,
    onSignTypedDataRequest
  });
  return (0, import_react5.useMemo)(() => {
    if (clientResolutionState.status === "pending") {
      return {
        status: "pending"
      };
    }
    if (clientResolutionState.status === "error") {
      return {
        status: "error",
        error: clientResolutionState.error
      };
    }
    const resolvedClient = clientResolutionState.client;
    switch (frameResolutionState.status) {
      case "success": {
        const frame = frameResolutionState.frame;
        return {
          getEmitter: (endpoint) => ({
            emit: endpoint.emit,
            emitEthProvider: endpoint.emitEthProvider
          }),
          sdk: (endpoint) => ({
            async addFrame() {
              logDebug("sdk.addFrame() called");
              if (frame.status !== "success" || frame.manifest?.status !== "success") {
                logDebug("Invalid frame domain manifest");
                endpoint.emit({
                  event: "frame_add_rejected",
                  reason: "invalid_domain_manifest"
                });
                throw new import_frame_host.AddFrame.InvalidDomainManifest();
              }
              if (addFrameRequestsCacheRef.current.has(
                frame.frame.button.action.url
              )) {
                logDebug("Frame already requested to be added.");
                endpoint.emit({
                  event: "frame_add_rejected",
                  reason: "rejected_by_user"
                });
                throw new import_frame_host.AddFrame.RejectedByUser();
              }
              const result = await onAddFrameRequestedRef.current(frame);
              logDebug("onAddFrameRequested() called", result);
              addFrameRequestsCacheRef.current.add(
                frame.frame.button.action.url
              );
              if (!result) {
                logDebug("Frame add request rejected by user");
                endpoint.emit({
                  event: "frame_add_rejected",
                  reason: "rejected_by_user"
                });
                throw new import_frame_host.AddFrame.RejectedByUser();
              }
              endpoint.emit({
                event: "frame_added",
                notificationDetails: result.notificationDetails
              });
              return result;
            },
            close() {
              logDebug("sdk.close() called");
              closeRef.current?.();
            },
            context: {
              client: resolvedClient,
              location: locationRef.current,
              user
            },
            async ethProviderRequest(parameters) {
              return providerRef.current.request(parameters);
            },
            eip6963RequestProvider() {
              onEIP6963RequestProviderRequestedRef.current({ endpoint });
            },
            openUrl(url) {
              logDebug("sdk.openUrl() called", url);
              onOpenUrlRef.current?.(url);
            },
            ready(options) {
              logDebug("sdk.ready() called");
              readyRef.current?.(options);
            },
            setPrimaryButton(options) {
              logDebug("sdk.setPrimaryButton() called", options);
              onPrimaryButtonSetRef.current?.(options, () => {
                logDebug('"primaryButtonClicked" called');
                endpoint.emit({
                  event: "primary_button_clicked"
                });
              });
            },
            signIn(options) {
              return onSignInRef.current({
                ...options,
                frame
              });
            },
            viewProfile(options) {
              return onViewProfileRef.current(options);
            }
          }),
          status: "success",
          frame: frameResolutionState.frame,
          client: clientResolutionState.client
        };
      }
      case "error": {
        return {
          status: "error",
          error: frameResolutionState.error
        };
      }
      case "pending": {
        return {
          status: "pending"
        };
      }
      default:
        assertNever(frameResolutionState);
    }
  }, [
    clientResolutionState,
    frameResolutionState,
    locationRef,
    logDebug,
    addFrameRequestsCacheRef,
    onAddFrameRequestedRef,
    closeRef,
    providerRef,
    onOpenUrlRef,
    readyRef,
    onPrimaryButtonSetRef,
    onViewProfileRef,
    onEIP6963RequestProviderRequestedRef,
    onSignInRef,
    user
  ]);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useFrameApp
});
