import * as viem from 'viem';
import { Abi, TypedDataDomain } from 'viem';
import { z } from 'zod';

type TransactionResponse = TransactionResponseSuccess | TransactionResponseFailure;
type TransactionResponseSuccess = {
    jsonrpc: "2.0";
    id: string | number | null;
    result: TransactionSuccessBody;
};
type TransactionSuccessBody = EthSendTransactionSuccessBody | EthSignTypedDataV4SuccessBody;
type EthSendTransactionSuccessBody = {
    address: `0x${string}`;
    transactionHash: `0x${string}`;
};
type EthSignTypedDataV4SuccessBody = {
    address: `0x${string}`;
    signature: `0x${string}`;
};
type TransactionResponseFailure = {
    jsonrpc: "2.0";
    id: string | number | null;
    error: {
        code: number;
        message: string;
    };
};
type CreateCastResponse = {
    jsonrpc: "2.0";
    id: string | number | null;
    result: {
        success: true;
    };
};
type MiniAppResponse = TransactionResponse | CreateCastResponse;
declare const createCastRequestSchemaLegacy: z.ZodObject<{
    type: z.ZodLiteral<"createCast">;
    data: z.ZodObject<{
        cast: z.ZodObject<{
            parent: z.ZodOptional<z.ZodString>;
            text: z.ZodString;
            embeds: z.ZodArray<z.ZodString, "many">;
        }, "strip", z.ZodTypeAny, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }>;
    }, "strip", z.ZodTypeAny, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }>;
}, "strip", z.ZodTypeAny, {
    type: "createCast";
    data: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
}, {
    type: "createCast";
    data: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
}>;
type CreateCastLegacyMessage = z.infer<typeof createCastRequestSchemaLegacy>;
declare const createCastRequestSchema: z.ZodObject<{
    jsonrpc: z.ZodLiteral<"2.0">;
    id: z.ZodUnion<[z.ZodString, z.ZodNumber, z.ZodNull]>;
    method: z.ZodLiteral<"fc_createCast">;
    params: z.ZodObject<{
        cast: z.ZodObject<{
            parent: z.ZodOptional<z.ZodString>;
            text: z.ZodString;
            embeds: z.ZodArray<z.ZodString, "many">;
        }, "strip", z.ZodTypeAny, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }>;
    }, "strip", z.ZodTypeAny, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }>;
}, "strip", z.ZodTypeAny, {
    id: string | number | null;
    method: "fc_createCast";
    params: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
    jsonrpc: "2.0";
}, {
    id: string | number | null;
    method: "fc_createCast";
    params: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
    jsonrpc: "2.0";
}>;
type CreateCastMessage = z.infer<typeof createCastRequestSchema>;
declare const ethSendTransactionActionSchema: z.ZodObject<{
    chainId: z.ZodString;
    method: z.ZodLiteral<"eth_sendTransaction">;
    attribution: z.ZodOptional<z.ZodBoolean>;
    params: z.ZodObject<{
        abi: z.ZodType<Abi, z.ZodTypeDef, Abi>;
        to: z.ZodType<`0x${string}`, z.ZodTypeDef, `0x${string}`>;
        value: z.ZodOptional<z.ZodString>;
        data: z.ZodOptional<z.ZodType<`0x${string}`, z.ZodTypeDef, `0x${string}`>>;
    }, "strip", z.ZodTypeAny, {
        abi: Abi;
        to: `0x${string}`;
        data?: `0x${string}` | undefined;
        value?: string | undefined;
    }, {
        abi: Abi;
        to: `0x${string}`;
        data?: `0x${string}` | undefined;
        value?: string | undefined;
    }>;
}, "strip", z.ZodTypeAny, {
    method: "eth_sendTransaction";
    params: {
        abi: Abi;
        to: `0x${string}`;
        data?: `0x${string}` | undefined;
        value?: string | undefined;
    };
    chainId: string;
    attribution?: boolean | undefined;
}, {
    method: "eth_sendTransaction";
    params: {
        abi: Abi;
        to: `0x${string}`;
        data?: `0x${string}` | undefined;
        value?: string | undefined;
    };
    chainId: string;
    attribution?: boolean | undefined;
}>;
type EthSendTransactionAction = z.infer<typeof ethSendTransactionActionSchema>;
declare const ethSignTypedDataV4ActionSchema: z.ZodObject<{
    chainId: z.ZodString;
    method: z.ZodLiteral<"eth_signTypedData_v4">;
    params: z.ZodObject<{
        domain: z.ZodType<TypedDataDomain, z.ZodTypeDef, TypedDataDomain>;
        types: z.ZodType<{
            [x: string]: readonly viem.TypedDataParameter[];
            [x: `string[${string}]`]: undefined;
            [x: `function[${string}]`]: undefined;
            [x: `address[${string}]`]: undefined;
            [x: `bool[${string}]`]: undefined;
            [x: `bytes[${string}]`]: undefined;
            [x: `bytes2[${string}]`]: undefined;
            [x: `bytes1[${string}]`]: undefined;
            [x: `bytes3[${string}]`]: undefined;
            [x: `bytes6[${string}]`]: undefined;
            [x: `bytes12[${string}]`]: undefined;
            [x: `bytes4[${string}]`]: undefined;
            [x: `bytes5[${string}]`]: undefined;
            [x: `bytes7[${string}]`]: undefined;
            [x: `bytes8[${string}]`]: undefined;
            [x: `bytes9[${string}]`]: undefined;
            [x: `bytes10[${string}]`]: undefined;
            [x: `bytes11[${string}]`]: undefined;
            [x: `bytes13[${string}]`]: undefined;
            [x: `bytes14[${string}]`]: undefined;
            [x: `bytes15[${string}]`]: undefined;
            [x: `bytes16[${string}]`]: undefined;
            [x: `bytes17[${string}]`]: undefined;
            [x: `bytes18[${string}]`]: undefined;
            [x: `bytes19[${string}]`]: undefined;
            [x: `bytes20[${string}]`]: undefined;
            [x: `bytes21[${string}]`]: undefined;
            [x: `bytes22[${string}]`]: undefined;
            [x: `bytes23[${string}]`]: undefined;
            [x: `bytes24[${string}]`]: undefined;
            [x: `bytes25[${string}]`]: undefined;
            [x: `bytes26[${string}]`]: undefined;
            [x: `bytes27[${string}]`]: undefined;
            [x: `bytes28[${string}]`]: undefined;
            [x: `bytes29[${string}]`]: undefined;
            [x: `bytes30[${string}]`]: undefined;
            [x: `bytes31[${string}]`]: undefined;
            [x: `bytes32[${string}]`]: undefined;
            [x: `int[${string}]`]: undefined;
            [x: `int200[${string}]`]: undefined;
            [x: `int8[${string}]`]: undefined;
            [x: `int16[${string}]`]: undefined;
            [x: `int24[${string}]`]: undefined;
            [x: `int32[${string}]`]: undefined;
            [x: `int40[${string}]`]: undefined;
            [x: `int48[${string}]`]: undefined;
            [x: `int56[${string}]`]: undefined;
            [x: `int64[${string}]`]: undefined;
            [x: `int72[${string}]`]: undefined;
            [x: `int80[${string}]`]: undefined;
            [x: `int88[${string}]`]: undefined;
            [x: `int96[${string}]`]: undefined;
            [x: `int104[${string}]`]: undefined;
            [x: `int112[${string}]`]: undefined;
            [x: `int120[${string}]`]: undefined;
            [x: `int128[${string}]`]: undefined;
            [x: `int136[${string}]`]: undefined;
            [x: `int144[${string}]`]: undefined;
            [x: `int152[${string}]`]: undefined;
            [x: `int160[${string}]`]: undefined;
            [x: `int168[${string}]`]: undefined;
            [x: `int176[${string}]`]: undefined;
            [x: `int184[${string}]`]: undefined;
            [x: `int192[${string}]`]: undefined;
            [x: `int208[${string}]`]: undefined;
            [x: `int216[${string}]`]: undefined;
            [x: `int224[${string}]`]: undefined;
            [x: `int232[${string}]`]: undefined;
            [x: `int240[${string}]`]: undefined;
            [x: `int248[${string}]`]: undefined;
            [x: `int256[${string}]`]: undefined;
            [x: `uint[${string}]`]: undefined;
            [x: `uint200[${string}]`]: undefined;
            [x: `uint8[${string}]`]: undefined;
            [x: `uint16[${string}]`]: undefined;
            [x: `uint24[${string}]`]: undefined;
            [x: `uint32[${string}]`]: undefined;
            [x: `uint40[${string}]`]: undefined;
            [x: `uint48[${string}]`]: undefined;
            [x: `uint56[${string}]`]: undefined;
            [x: `uint64[${string}]`]: undefined;
            [x: `uint72[${string}]`]: undefined;
            [x: `uint80[${string}]`]: undefined;
            [x: `uint88[${string}]`]: undefined;
            [x: `uint96[${string}]`]: undefined;
            [x: `uint104[${string}]`]: undefined;
            [x: `uint112[${string}]`]: undefined;
            [x: `uint120[${string}]`]: undefined;
            [x: `uint128[${string}]`]: undefined;
            [x: `uint136[${string}]`]: undefined;
            [x: `uint144[${string}]`]: undefined;
            [x: `uint152[${string}]`]: undefined;
            [x: `uint160[${string}]`]: undefined;
            [x: `uint168[${string}]`]: undefined;
            [x: `uint176[${string}]`]: undefined;
            [x: `uint184[${string}]`]: undefined;
            [x: `uint192[${string}]`]: undefined;
            [x: `uint208[${string}]`]: undefined;
            [x: `uint216[${string}]`]: undefined;
            [x: `uint224[${string}]`]: undefined;
            [x: `uint232[${string}]`]: undefined;
            [x: `uint240[${string}]`]: undefined;
            [x: `uint248[${string}]`]: undefined;
            [x: `uint256[${string}]`]: undefined;
            string?: undefined;
            address?: undefined;
            bool?: undefined;
            bytes?: undefined;
            bytes2?: undefined;
            bytes1?: undefined;
            bytes3?: undefined;
            bytes6?: undefined;
            bytes12?: undefined;
            bytes4?: undefined;
            bytes5?: undefined;
            bytes7?: undefined;
            bytes8?: undefined;
            bytes9?: undefined;
            bytes10?: undefined;
            bytes11?: undefined;
            bytes13?: undefined;
            bytes14?: undefined;
            bytes15?: undefined;
            bytes16?: undefined;
            bytes17?: undefined;
            bytes18?: undefined;
            bytes19?: undefined;
            bytes20?: undefined;
            bytes21?: undefined;
            bytes22?: undefined;
            bytes23?: undefined;
            bytes24?: undefined;
            bytes25?: undefined;
            bytes26?: undefined;
            bytes27?: undefined;
            bytes28?: undefined;
            bytes29?: undefined;
            bytes30?: undefined;
            bytes31?: undefined;
            bytes32?: undefined;
            int200?: undefined;
            int8?: undefined;
            int16?: undefined;
            int24?: undefined;
            int32?: undefined;
            int40?: undefined;
            int48?: undefined;
            int56?: undefined;
            int64?: undefined;
            int72?: undefined;
            int80?: undefined;
            int88?: undefined;
            int96?: undefined;
            int104?: undefined;
            int112?: undefined;
            int120?: undefined;
            int128?: undefined;
            int136?: undefined;
            int144?: undefined;
            int152?: undefined;
            int160?: undefined;
            int168?: undefined;
            int176?: undefined;
            int184?: undefined;
            int192?: undefined;
            int208?: undefined;
            int216?: undefined;
            int224?: undefined;
            int232?: undefined;
            int240?: undefined;
            int248?: undefined;
            int256?: undefined;
            uint200?: undefined;
            uint8?: undefined;
            uint16?: undefined;
            uint24?: undefined;
            uint32?: undefined;
            uint40?: undefined;
            uint48?: undefined;
            uint56?: undefined;
            uint64?: undefined;
            uint72?: undefined;
            uint80?: undefined;
            uint88?: undefined;
            uint96?: undefined;
            uint104?: undefined;
            uint112?: undefined;
            uint120?: undefined;
            uint128?: undefined;
            uint136?: undefined;
            uint144?: undefined;
            uint152?: undefined;
            uint160?: undefined;
            uint168?: undefined;
            uint176?: undefined;
            uint184?: undefined;
            uint192?: undefined;
            uint208?: undefined;
            uint216?: undefined;
            uint224?: undefined;
            uint232?: undefined;
            uint240?: undefined;
            uint248?: undefined;
            uint256?: undefined;
        }, z.ZodTypeDef, {
            [x: string]: readonly viem.TypedDataParameter[];
            [x: `string[${string}]`]: undefined;
            [x: `function[${string}]`]: undefined;
            [x: `address[${string}]`]: undefined;
            [x: `bool[${string}]`]: undefined;
            [x: `bytes[${string}]`]: undefined;
            [x: `bytes2[${string}]`]: undefined;
            [x: `bytes1[${string}]`]: undefined;
            [x: `bytes3[${string}]`]: undefined;
            [x: `bytes6[${string}]`]: undefined;
            [x: `bytes12[${string}]`]: undefined;
            [x: `bytes4[${string}]`]: undefined;
            [x: `bytes5[${string}]`]: undefined;
            [x: `bytes7[${string}]`]: undefined;
            [x: `bytes8[${string}]`]: undefined;
            [x: `bytes9[${string}]`]: undefined;
            [x: `bytes10[${string}]`]: undefined;
            [x: `bytes11[${string}]`]: undefined;
            [x: `bytes13[${string}]`]: undefined;
            [x: `bytes14[${string}]`]: undefined;
            [x: `bytes15[${string}]`]: undefined;
            [x: `bytes16[${string}]`]: undefined;
            [x: `bytes17[${string}]`]: undefined;
            [x: `bytes18[${string}]`]: undefined;
            [x: `bytes19[${string}]`]: undefined;
            [x: `bytes20[${string}]`]: undefined;
            [x: `bytes21[${string}]`]: undefined;
            [x: `bytes22[${string}]`]: undefined;
            [x: `bytes23[${string}]`]: undefined;
            [x: `bytes24[${string}]`]: undefined;
            [x: `bytes25[${string}]`]: undefined;
            [x: `bytes26[${string}]`]: undefined;
            [x: `bytes27[${string}]`]: undefined;
            [x: `bytes28[${string}]`]: undefined;
            [x: `bytes29[${string}]`]: undefined;
            [x: `bytes30[${string}]`]: undefined;
            [x: `bytes31[${string}]`]: undefined;
            [x: `bytes32[${string}]`]: undefined;
            [x: `int[${string}]`]: undefined;
            [x: `int200[${string}]`]: undefined;
            [x: `int8[${string}]`]: undefined;
            [x: `int16[${string}]`]: undefined;
            [x: `int24[${string}]`]: undefined;
            [x: `int32[${string}]`]: undefined;
            [x: `int40[${string}]`]: undefined;
            [x: `int48[${string}]`]: undefined;
            [x: `int56[${string}]`]: undefined;
            [x: `int64[${string}]`]: undefined;
            [x: `int72[${string}]`]: undefined;
            [x: `int80[${string}]`]: undefined;
            [x: `int88[${string}]`]: undefined;
            [x: `int96[${string}]`]: undefined;
            [x: `int104[${string}]`]: undefined;
            [x: `int112[${string}]`]: undefined;
            [x: `int120[${string}]`]: undefined;
            [x: `int128[${string}]`]: undefined;
            [x: `int136[${string}]`]: undefined;
            [x: `int144[${string}]`]: undefined;
            [x: `int152[${string}]`]: undefined;
            [x: `int160[${string}]`]: undefined;
            [x: `int168[${string}]`]: undefined;
            [x: `int176[${string}]`]: undefined;
            [x: `int184[${string}]`]: undefined;
            [x: `int192[${string}]`]: undefined;
            [x: `int208[${string}]`]: undefined;
            [x: `int216[${string}]`]: undefined;
            [x: `int224[${string}]`]: undefined;
            [x: `int232[${string}]`]: undefined;
            [x: `int240[${string}]`]: undefined;
            [x: `int248[${string}]`]: undefined;
            [x: `int256[${string}]`]: undefined;
            [x: `uint[${string}]`]: undefined;
            [x: `uint200[${string}]`]: undefined;
            [x: `uint8[${string}]`]: undefined;
            [x: `uint16[${string}]`]: undefined;
            [x: `uint24[${string}]`]: undefined;
            [x: `uint32[${string}]`]: undefined;
            [x: `uint40[${string}]`]: undefined;
            [x: `uint48[${string}]`]: undefined;
            [x: `uint56[${string}]`]: undefined;
            [x: `uint64[${string}]`]: undefined;
            [x: `uint72[${string}]`]: undefined;
            [x: `uint80[${string}]`]: undefined;
            [x: `uint88[${string}]`]: undefined;
            [x: `uint96[${string}]`]: undefined;
            [x: `uint104[${string}]`]: undefined;
            [x: `uint112[${string}]`]: undefined;
            [x: `uint120[${string}]`]: undefined;
            [x: `uint128[${string}]`]: undefined;
            [x: `uint136[${string}]`]: undefined;
            [x: `uint144[${string}]`]: undefined;
            [x: `uint152[${string}]`]: undefined;
            [x: `uint160[${string}]`]: undefined;
            [x: `uint168[${string}]`]: undefined;
            [x: `uint176[${string}]`]: undefined;
            [x: `uint184[${string}]`]: undefined;
            [x: `uint192[${string}]`]: undefined;
            [x: `uint208[${string}]`]: undefined;
            [x: `uint216[${string}]`]: undefined;
            [x: `uint224[${string}]`]: undefined;
            [x: `uint232[${string}]`]: undefined;
            [x: `uint240[${string}]`]: undefined;
            [x: `uint248[${string}]`]: undefined;
            [x: `uint256[${string}]`]: undefined;
            string?: undefined;
            address?: undefined;
            bool?: undefined;
            bytes?: undefined;
            bytes2?: undefined;
            bytes1?: undefined;
            bytes3?: undefined;
            bytes6?: undefined;
            bytes12?: undefined;
            bytes4?: undefined;
            bytes5?: undefined;
            bytes7?: undefined;
            bytes8?: undefined;
            bytes9?: undefined;
            bytes10?: undefined;
            bytes11?: undefined;
            bytes13?: undefined;
            bytes14?: undefined;
            bytes15?: undefined;
            bytes16?: undefined;
            bytes17?: undefined;
            bytes18?: undefined;
            bytes19?: undefined;
            bytes20?: undefined;
            bytes21?: undefined;
            bytes22?: undefined;
            bytes23?: undefined;
            bytes24?: undefined;
            bytes25?: undefined;
            bytes26?: undefined;
            bytes27?: undefined;
            bytes28?: undefined;
            bytes29?: undefined;
            bytes30?: undefined;
            bytes31?: undefined;
            bytes32?: undefined;
            int200?: undefined;
            int8?: undefined;
            int16?: undefined;
            int24?: undefined;
            int32?: undefined;
            int40?: undefined;
            int48?: undefined;
            int56?: undefined;
            int64?: undefined;
            int72?: undefined;
            int80?: undefined;
            int88?: undefined;
            int96?: undefined;
            int104?: undefined;
            int112?: undefined;
            int120?: undefined;
            int128?: undefined;
            int136?: undefined;
            int144?: undefined;
            int152?: undefined;
            int160?: undefined;
            int168?: undefined;
            int176?: undefined;
            int184?: undefined;
            int192?: undefined;
            int208?: undefined;
            int216?: undefined;
            int224?: undefined;
            int232?: undefined;
            int240?: undefined;
            int248?: undefined;
            int256?: undefined;
            uint200?: undefined;
            uint8?: undefined;
            uint16?: undefined;
            uint24?: undefined;
            uint32?: undefined;
            uint40?: undefined;
            uint48?: undefined;
            uint56?: undefined;
            uint64?: undefined;
            uint72?: undefined;
            uint80?: undefined;
            uint88?: undefined;
            uint96?: undefined;
            uint104?: undefined;
            uint112?: undefined;
            uint120?: undefined;
            uint128?: undefined;
            uint136?: undefined;
            uint144?: undefined;
            uint152?: undefined;
            uint160?: undefined;
            uint168?: undefined;
            uint176?: undefined;
            uint184?: undefined;
            uint192?: undefined;
            uint208?: undefined;
            uint216?: undefined;
            uint224?: undefined;
            uint232?: undefined;
            uint240?: undefined;
            uint248?: undefined;
            uint256?: undefined;
        }>;
        primaryType: z.ZodString;
        message: z.ZodRecord<z.ZodString, z.ZodUnknown>;
    }, "strip", z.ZodTypeAny, {
        message: Record<string, unknown>;
        domain: TypedDataDomain;
        types: {
            [x: string]: readonly viem.TypedDataParameter[];
            [x: `string[${string}]`]: undefined;
            [x: `function[${string}]`]: undefined;
            [x: `address[${string}]`]: undefined;
            [x: `bool[${string}]`]: undefined;
            [x: `bytes[${string}]`]: undefined;
            [x: `bytes2[${string}]`]: undefined;
            [x: `bytes1[${string}]`]: undefined;
            [x: `bytes3[${string}]`]: undefined;
            [x: `bytes6[${string}]`]: undefined;
            [x: `bytes12[${string}]`]: undefined;
            [x: `bytes4[${string}]`]: undefined;
            [x: `bytes5[${string}]`]: undefined;
            [x: `bytes7[${string}]`]: undefined;
            [x: `bytes8[${string}]`]: undefined;
            [x: `bytes9[${string}]`]: undefined;
            [x: `bytes10[${string}]`]: undefined;
            [x: `bytes11[${string}]`]: undefined;
            [x: `bytes13[${string}]`]: undefined;
            [x: `bytes14[${string}]`]: undefined;
            [x: `bytes15[${string}]`]: undefined;
            [x: `bytes16[${string}]`]: undefined;
            [x: `bytes17[${string}]`]: undefined;
            [x: `bytes18[${string}]`]: undefined;
            [x: `bytes19[${string}]`]: undefined;
            [x: `bytes20[${string}]`]: undefined;
            [x: `bytes21[${string}]`]: undefined;
            [x: `bytes22[${string}]`]: undefined;
            [x: `bytes23[${string}]`]: undefined;
            [x: `bytes24[${string}]`]: undefined;
            [x: `bytes25[${string}]`]: undefined;
            [x: `bytes26[${string}]`]: undefined;
            [x: `bytes27[${string}]`]: undefined;
            [x: `bytes28[${string}]`]: undefined;
            [x: `bytes29[${string}]`]: undefined;
            [x: `bytes30[${string}]`]: undefined;
            [x: `bytes31[${string}]`]: undefined;
            [x: `bytes32[${string}]`]: undefined;
            [x: `int[${string}]`]: undefined;
            [x: `int200[${string}]`]: undefined;
            [x: `int8[${string}]`]: undefined;
            [x: `int16[${string}]`]: undefined;
            [x: `int24[${string}]`]: undefined;
            [x: `int32[${string}]`]: undefined;
            [x: `int40[${string}]`]: undefined;
            [x: `int48[${string}]`]: undefined;
            [x: `int56[${string}]`]: undefined;
            [x: `int64[${string}]`]: undefined;
            [x: `int72[${string}]`]: undefined;
            [x: `int80[${string}]`]: undefined;
            [x: `int88[${string}]`]: undefined;
            [x: `int96[${string}]`]: undefined;
            [x: `int104[${string}]`]: undefined;
            [x: `int112[${string}]`]: undefined;
            [x: `int120[${string}]`]: undefined;
            [x: `int128[${string}]`]: undefined;
            [x: `int136[${string}]`]: undefined;
            [x: `int144[${string}]`]: undefined;
            [x: `int152[${string}]`]: undefined;
            [x: `int160[${string}]`]: undefined;
            [x: `int168[${string}]`]: undefined;
            [x: `int176[${string}]`]: undefined;
            [x: `int184[${string}]`]: undefined;
            [x: `int192[${string}]`]: undefined;
            [x: `int208[${string}]`]: undefined;
            [x: `int216[${string}]`]: undefined;
            [x: `int224[${string}]`]: undefined;
            [x: `int232[${string}]`]: undefined;
            [x: `int240[${string}]`]: undefined;
            [x: `int248[${string}]`]: undefined;
            [x: `int256[${string}]`]: undefined;
            [x: `uint[${string}]`]: undefined;
            [x: `uint200[${string}]`]: undefined;
            [x: `uint8[${string}]`]: undefined;
            [x: `uint16[${string}]`]: undefined;
            [x: `uint24[${string}]`]: undefined;
            [x: `uint32[${string}]`]: undefined;
            [x: `uint40[${string}]`]: undefined;
            [x: `uint48[${string}]`]: undefined;
            [x: `uint56[${string}]`]: undefined;
            [x: `uint64[${string}]`]: undefined;
            [x: `uint72[${string}]`]: undefined;
            [x: `uint80[${string}]`]: undefined;
            [x: `uint88[${string}]`]: undefined;
            [x: `uint96[${string}]`]: undefined;
            [x: `uint104[${string}]`]: undefined;
            [x: `uint112[${string}]`]: undefined;
            [x: `uint120[${string}]`]: undefined;
            [x: `uint128[${string}]`]: undefined;
            [x: `uint136[${string}]`]: undefined;
            [x: `uint144[${string}]`]: undefined;
            [x: `uint152[${string}]`]: undefined;
            [x: `uint160[${string}]`]: undefined;
            [x: `uint168[${string}]`]: undefined;
            [x: `uint176[${string}]`]: undefined;
            [x: `uint184[${string}]`]: undefined;
            [x: `uint192[${string}]`]: undefined;
            [x: `uint208[${string}]`]: undefined;
            [x: `uint216[${string}]`]: undefined;
            [x: `uint224[${string}]`]: undefined;
            [x: `uint232[${string}]`]: undefined;
            [x: `uint240[${string}]`]: undefined;
            [x: `uint248[${string}]`]: undefined;
            [x: `uint256[${string}]`]: undefined;
            string?: undefined;
            address?: undefined;
            bool?: undefined;
            bytes?: undefined;
            bytes2?: undefined;
            bytes1?: undefined;
            bytes3?: undefined;
            bytes6?: undefined;
            bytes12?: undefined;
            bytes4?: undefined;
            bytes5?: undefined;
            bytes7?: undefined;
            bytes8?: undefined;
            bytes9?: undefined;
            bytes10?: undefined;
            bytes11?: undefined;
            bytes13?: undefined;
            bytes14?: undefined;
            bytes15?: undefined;
            bytes16?: undefined;
            bytes17?: undefined;
            bytes18?: undefined;
            bytes19?: undefined;
            bytes20?: undefined;
            bytes21?: undefined;
            bytes22?: undefined;
            bytes23?: undefined;
            bytes24?: undefined;
            bytes25?: undefined;
            bytes26?: undefined;
            bytes27?: undefined;
            bytes28?: undefined;
            bytes29?: undefined;
            bytes30?: undefined;
            bytes31?: undefined;
            bytes32?: undefined;
            int200?: undefined;
            int8?: undefined;
            int16?: undefined;
            int24?: undefined;
            int32?: undefined;
            int40?: undefined;
            int48?: undefined;
            int56?: undefined;
            int64?: undefined;
            int72?: undefined;
            int80?: undefined;
            int88?: undefined;
            int96?: undefined;
            int104?: undefined;
            int112?: undefined;
            int120?: undefined;
            int128?: undefined;
            int136?: undefined;
            int144?: undefined;
            int152?: undefined;
            int160?: undefined;
            int168?: undefined;
            int176?: undefined;
            int184?: undefined;
            int192?: undefined;
            int208?: undefined;
            int216?: undefined;
            int224?: undefined;
            int232?: undefined;
            int240?: undefined;
            int248?: undefined;
            int256?: undefined;
            uint200?: undefined;
            uint8?: undefined;
            uint16?: undefined;
            uint24?: undefined;
            uint32?: undefined;
            uint40?: undefined;
            uint48?: undefined;
            uint56?: undefined;
            uint64?: undefined;
            uint72?: undefined;
            uint80?: undefined;
            uint88?: undefined;
            uint96?: undefined;
            uint104?: undefined;
            uint112?: undefined;
            uint120?: undefined;
            uint128?: undefined;
            uint136?: undefined;
            uint144?: undefined;
            uint152?: undefined;
            uint160?: undefined;
            uint168?: undefined;
            uint176?: undefined;
            uint184?: undefined;
            uint192?: undefined;
            uint208?: undefined;
            uint216?: undefined;
            uint224?: undefined;
            uint232?: undefined;
            uint240?: undefined;
            uint248?: undefined;
            uint256?: undefined;
        };
        primaryType: string;
    }, {
        message: Record<string, unknown>;
        domain: TypedDataDomain;
        types: {
            [x: string]: readonly viem.TypedDataParameter[];
            [x: `string[${string}]`]: undefined;
            [x: `function[${string}]`]: undefined;
            [x: `address[${string}]`]: undefined;
            [x: `bool[${string}]`]: undefined;
            [x: `bytes[${string}]`]: undefined;
            [x: `bytes2[${string}]`]: undefined;
            [x: `bytes1[${string}]`]: undefined;
            [x: `bytes3[${string}]`]: undefined;
            [x: `bytes6[${string}]`]: undefined;
            [x: `bytes12[${string}]`]: undefined;
            [x: `bytes4[${string}]`]: undefined;
            [x: `bytes5[${string}]`]: undefined;
            [x: `bytes7[${string}]`]: undefined;
            [x: `bytes8[${string}]`]: undefined;
            [x: `bytes9[${string}]`]: undefined;
            [x: `bytes10[${string}]`]: undefined;
            [x: `bytes11[${string}]`]: undefined;
            [x: `bytes13[${string}]`]: undefined;
            [x: `bytes14[${string}]`]: undefined;
            [x: `bytes15[${string}]`]: undefined;
            [x: `bytes16[${string}]`]: undefined;
            [x: `bytes17[${string}]`]: undefined;
            [x: `bytes18[${string}]`]: undefined;
            [x: `bytes19[${string}]`]: undefined;
            [x: `bytes20[${string}]`]: undefined;
            [x: `bytes21[${string}]`]: undefined;
            [x: `bytes22[${string}]`]: undefined;
            [x: `bytes23[${string}]`]: undefined;
            [x: `bytes24[${string}]`]: undefined;
            [x: `bytes25[${string}]`]: undefined;
            [x: `bytes26[${string}]`]: undefined;
            [x: `bytes27[${string}]`]: undefined;
            [x: `bytes28[${string}]`]: undefined;
            [x: `bytes29[${string}]`]: undefined;
            [x: `bytes30[${string}]`]: undefined;
            [x: `bytes31[${string}]`]: undefined;
            [x: `bytes32[${string}]`]: undefined;
            [x: `int[${string}]`]: undefined;
            [x: `int200[${string}]`]: undefined;
            [x: `int8[${string}]`]: undefined;
            [x: `int16[${string}]`]: undefined;
            [x: `int24[${string}]`]: undefined;
            [x: `int32[${string}]`]: undefined;
            [x: `int40[${string}]`]: undefined;
            [x: `int48[${string}]`]: undefined;
            [x: `int56[${string}]`]: undefined;
            [x: `int64[${string}]`]: undefined;
            [x: `int72[${string}]`]: undefined;
            [x: `int80[${string}]`]: undefined;
            [x: `int88[${string}]`]: undefined;
            [x: `int96[${string}]`]: undefined;
            [x: `int104[${string}]`]: undefined;
            [x: `int112[${string}]`]: undefined;
            [x: `int120[${string}]`]: undefined;
            [x: `int128[${string}]`]: undefined;
            [x: `int136[${string}]`]: undefined;
            [x: `int144[${string}]`]: undefined;
            [x: `int152[${string}]`]: undefined;
            [x: `int160[${string}]`]: undefined;
            [x: `int168[${string}]`]: undefined;
            [x: `int176[${string}]`]: undefined;
            [x: `int184[${string}]`]: undefined;
            [x: `int192[${string}]`]: undefined;
            [x: `int208[${string}]`]: undefined;
            [x: `int216[${string}]`]: undefined;
            [x: `int224[${string}]`]: undefined;
            [x: `int232[${string}]`]: undefined;
            [x: `int240[${string}]`]: undefined;
            [x: `int248[${string}]`]: undefined;
            [x: `int256[${string}]`]: undefined;
            [x: `uint[${string}]`]: undefined;
            [x: `uint200[${string}]`]: undefined;
            [x: `uint8[${string}]`]: undefined;
            [x: `uint16[${string}]`]: undefined;
            [x: `uint24[${string}]`]: undefined;
            [x: `uint32[${string}]`]: undefined;
            [x: `uint40[${string}]`]: undefined;
            [x: `uint48[${string}]`]: undefined;
            [x: `uint56[${string}]`]: undefined;
            [x: `uint64[${string}]`]: undefined;
            [x: `uint72[${string}]`]: undefined;
            [x: `uint80[${string}]`]: undefined;
            [x: `uint88[${string}]`]: undefined;
            [x: `uint96[${string}]`]: undefined;
            [x: `uint104[${string}]`]: undefined;
            [x: `uint112[${string}]`]: undefined;
            [x: `uint120[${string}]`]: undefined;
            [x: `uint128[${string}]`]: undefined;
            [x: `uint136[${string}]`]: undefined;
            [x: `uint144[${string}]`]: undefined;
            [x: `uint152[${string}]`]: undefined;
            [x: `uint160[${string}]`]: undefined;
            [x: `uint168[${string}]`]: undefined;
            [x: `uint176[${string}]`]: undefined;
            [x: `uint184[${string}]`]: undefined;
            [x: `uint192[${string}]`]: undefined;
            [x: `uint208[${string}]`]: undefined;
            [x: `uint216[${string}]`]: undefined;
            [x: `uint224[${string}]`]: undefined;
            [x: `uint232[${string}]`]: undefined;
            [x: `uint240[${string}]`]: undefined;
            [x: `uint248[${string}]`]: undefined;
            [x: `uint256[${string}]`]: undefined;
            string?: undefined;
            address?: undefined;
            bool?: undefined;
            bytes?: undefined;
            bytes2?: undefined;
            bytes1?: undefined;
            bytes3?: undefined;
            bytes6?: undefined;
            bytes12?: undefined;
            bytes4?: undefined;
            bytes5?: undefined;
            bytes7?: undefined;
            bytes8?: undefined;
            bytes9?: undefined;
            bytes10?: undefined;
            bytes11?: undefined;
            bytes13?: undefined;
            bytes14?: undefined;
            bytes15?: undefined;
            bytes16?: undefined;
            bytes17?: undefined;
            bytes18?: undefined;
            bytes19?: undefined;
            bytes20?: undefined;
            bytes21?: undefined;
            bytes22?: undefined;
            bytes23?: undefined;
            bytes24?: undefined;
            bytes25?: undefined;
            bytes26?: undefined;
            bytes27?: undefined;
            bytes28?: undefined;
            bytes29?: undefined;
            bytes30?: undefined;
            bytes31?: undefined;
            bytes32?: undefined;
            int200?: undefined;
            int8?: undefined;
            int16?: undefined;
            int24?: undefined;
            int32?: undefined;
            int40?: undefined;
            int48?: undefined;
            int56?: undefined;
            int64?: undefined;
            int72?: undefined;
            int80?: undefined;
            int88?: undefined;
            int96?: undefined;
            int104?: undefined;
            int112?: undefined;
            int120?: undefined;
            int128?: undefined;
            int136?: undefined;
            int144?: undefined;
            int152?: undefined;
            int160?: undefined;
            int168?: undefined;
            int176?: undefined;
            int184?: undefined;
            int192?: undefined;
            int208?: undefined;
            int216?: undefined;
            int224?: undefined;
            int232?: undefined;
            int240?: undefined;
            int248?: undefined;
            int256?: undefined;
            uint200?: undefined;
            uint8?: undefined;
            uint16?: undefined;
            uint24?: undefined;
            uint32?: undefined;
            uint40?: undefined;
            uint48?: undefined;
            uint56?: undefined;
            uint64?: undefined;
            uint72?: undefined;
            uint80?: undefined;
            uint88?: undefined;
            uint96?: undefined;
            uint104?: undefined;
            uint112?: undefined;
            uint120?: undefined;
            uint128?: undefined;
            uint136?: undefined;
            uint144?: undefined;
            uint152?: undefined;
            uint160?: undefined;
            uint168?: undefined;
            uint176?: undefined;
            uint184?: undefined;
            uint192?: undefined;
            uint208?: undefined;
            uint216?: undefined;
            uint224?: undefined;
            uint232?: undefined;
            uint240?: undefined;
            uint248?: undefined;
            uint256?: undefined;
        };
        primaryType: string;
    }>;
}, "strip", z.ZodTypeAny, {
    method: "eth_signTypedData_v4";
    params: {
        message: Record<string, unknown>;
        domain: TypedDataDomain;
        types: {
            [x: string]: readonly viem.TypedDataParameter[];
            [x: `string[${string}]`]: undefined;
            [x: `function[${string}]`]: undefined;
            [x: `address[${string}]`]: undefined;
            [x: `bool[${string}]`]: undefined;
            [x: `bytes[${string}]`]: undefined;
            [x: `bytes2[${string}]`]: undefined;
            [x: `bytes1[${string}]`]: undefined;
            [x: `bytes3[${string}]`]: undefined;
            [x: `bytes6[${string}]`]: undefined;
            [x: `bytes12[${string}]`]: undefined;
            [x: `bytes4[${string}]`]: undefined;
            [x: `bytes5[${string}]`]: undefined;
            [x: `bytes7[${string}]`]: undefined;
            [x: `bytes8[${string}]`]: undefined;
            [x: `bytes9[${string}]`]: undefined;
            [x: `bytes10[${string}]`]: undefined;
            [x: `bytes11[${string}]`]: undefined;
            [x: `bytes13[${string}]`]: undefined;
            [x: `bytes14[${string}]`]: undefined;
            [x: `bytes15[${string}]`]: undefined;
            [x: `bytes16[${string}]`]: undefined;
            [x: `bytes17[${string}]`]: undefined;
            [x: `bytes18[${string}]`]: undefined;
            [x: `bytes19[${string}]`]: undefined;
            [x: `bytes20[${string}]`]: undefined;
            [x: `bytes21[${string}]`]: undefined;
            [x: `bytes22[${string}]`]: undefined;
            [x: `bytes23[${string}]`]: undefined;
            [x: `bytes24[${string}]`]: undefined;
            [x: `bytes25[${string}]`]: undefined;
            [x: `bytes26[${string}]`]: undefined;
            [x: `bytes27[${string}]`]: undefined;
            [x: `bytes28[${string}]`]: undefined;
            [x: `bytes29[${string}]`]: undefined;
            [x: `bytes30[${string}]`]: undefined;
            [x: `bytes31[${string}]`]: undefined;
            [x: `bytes32[${string}]`]: undefined;
            [x: `int[${string}]`]: undefined;
            [x: `int200[${string}]`]: undefined;
            [x: `int8[${string}]`]: undefined;
            [x: `int16[${string}]`]: undefined;
            [x: `int24[${string}]`]: undefined;
            [x: `int32[${string}]`]: undefined;
            [x: `int40[${string}]`]: undefined;
            [x: `int48[${string}]`]: undefined;
            [x: `int56[${string}]`]: undefined;
            [x: `int64[${string}]`]: undefined;
            [x: `int72[${string}]`]: undefined;
            [x: `int80[${string}]`]: undefined;
            [x: `int88[${string}]`]: undefined;
            [x: `int96[${string}]`]: undefined;
            [x: `int104[${string}]`]: undefined;
            [x: `int112[${string}]`]: undefined;
            [x: `int120[${string}]`]: undefined;
            [x: `int128[${string}]`]: undefined;
            [x: `int136[${string}]`]: undefined;
            [x: `int144[${string}]`]: undefined;
            [x: `int152[${string}]`]: undefined;
            [x: `int160[${string}]`]: undefined;
            [x: `int168[${string}]`]: undefined;
            [x: `int176[${string}]`]: undefined;
            [x: `int184[${string}]`]: undefined;
            [x: `int192[${string}]`]: undefined;
            [x: `int208[${string}]`]: undefined;
            [x: `int216[${string}]`]: undefined;
            [x: `int224[${string}]`]: undefined;
            [x: `int232[${string}]`]: undefined;
            [x: `int240[${string}]`]: undefined;
            [x: `int248[${string}]`]: undefined;
            [x: `int256[${string}]`]: undefined;
            [x: `uint[${string}]`]: undefined;
            [x: `uint200[${string}]`]: undefined;
            [x: `uint8[${string}]`]: undefined;
            [x: `uint16[${string}]`]: undefined;
            [x: `uint24[${string}]`]: undefined;
            [x: `uint32[${string}]`]: undefined;
            [x: `uint40[${string}]`]: undefined;
            [x: `uint48[${string}]`]: undefined;
            [x: `uint56[${string}]`]: undefined;
            [x: `uint64[${string}]`]: undefined;
            [x: `uint72[${string}]`]: undefined;
            [x: `uint80[${string}]`]: undefined;
            [x: `uint88[${string}]`]: undefined;
            [x: `uint96[${string}]`]: undefined;
            [x: `uint104[${string}]`]: undefined;
            [x: `uint112[${string}]`]: undefined;
            [x: `uint120[${string}]`]: undefined;
            [x: `uint128[${string}]`]: undefined;
            [x: `uint136[${string}]`]: undefined;
            [x: `uint144[${string}]`]: undefined;
            [x: `uint152[${string}]`]: undefined;
            [x: `uint160[${string}]`]: undefined;
            [x: `uint168[${string}]`]: undefined;
            [x: `uint176[${string}]`]: undefined;
            [x: `uint184[${string}]`]: undefined;
            [x: `uint192[${string}]`]: undefined;
            [x: `uint208[${string}]`]: undefined;
            [x: `uint216[${string}]`]: undefined;
            [x: `uint224[${string}]`]: undefined;
            [x: `uint232[${string}]`]: undefined;
            [x: `uint240[${string}]`]: undefined;
            [x: `uint248[${string}]`]: undefined;
            [x: `uint256[${string}]`]: undefined;
            string?: undefined;
            address?: undefined;
            bool?: undefined;
            bytes?: undefined;
            bytes2?: undefined;
            bytes1?: undefined;
            bytes3?: undefined;
            bytes6?: undefined;
            bytes12?: undefined;
            bytes4?: undefined;
            bytes5?: undefined;
            bytes7?: undefined;
            bytes8?: undefined;
            bytes9?: undefined;
            bytes10?: undefined;
            bytes11?: undefined;
            bytes13?: undefined;
            bytes14?: undefined;
            bytes15?: undefined;
            bytes16?: undefined;
            bytes17?: undefined;
            bytes18?: undefined;
            bytes19?: undefined;
            bytes20?: undefined;
            bytes21?: undefined;
            bytes22?: undefined;
            bytes23?: undefined;
            bytes24?: undefined;
            bytes25?: undefined;
            bytes26?: undefined;
            bytes27?: undefined;
            bytes28?: undefined;
            bytes29?: undefined;
            bytes30?: undefined;
            bytes31?: undefined;
            bytes32?: undefined;
            int200?: undefined;
            int8?: undefined;
            int16?: undefined;
            int24?: undefined;
            int32?: undefined;
            int40?: undefined;
            int48?: undefined;
            int56?: undefined;
            int64?: undefined;
            int72?: undefined;
            int80?: undefined;
            int88?: undefined;
            int96?: undefined;
            int104?: undefined;
            int112?: undefined;
            int120?: undefined;
            int128?: undefined;
            int136?: undefined;
            int144?: undefined;
            int152?: undefined;
            int160?: undefined;
            int168?: undefined;
            int176?: undefined;
            int184?: undefined;
            int192?: undefined;
            int208?: undefined;
            int216?: undefined;
            int224?: undefined;
            int232?: undefined;
            int240?: undefined;
            int248?: undefined;
            int256?: undefined;
            uint200?: undefined;
            uint8?: undefined;
            uint16?: undefined;
            uint24?: undefined;
            uint32?: undefined;
            uint40?: undefined;
            uint48?: undefined;
            uint56?: undefined;
            uint64?: undefined;
            uint72?: undefined;
            uint80?: undefined;
            uint88?: undefined;
            uint96?: undefined;
            uint104?: undefined;
            uint112?: undefined;
            uint120?: undefined;
            uint128?: undefined;
            uint136?: undefined;
            uint144?: undefined;
            uint152?: undefined;
            uint160?: undefined;
            uint168?: undefined;
            uint176?: undefined;
            uint184?: undefined;
            uint192?: undefined;
            uint208?: undefined;
            uint216?: undefined;
            uint224?: undefined;
            uint232?: undefined;
            uint240?: undefined;
            uint248?: undefined;
            uint256?: undefined;
        };
        primaryType: string;
    };
    chainId: string;
}, {
    method: "eth_signTypedData_v4";
    params: {
        message: Record<string, unknown>;
        domain: TypedDataDomain;
        types: {
            [x: string]: readonly viem.TypedDataParameter[];
            [x: `string[${string}]`]: undefined;
            [x: `function[${string}]`]: undefined;
            [x: `address[${string}]`]: undefined;
            [x: `bool[${string}]`]: undefined;
            [x: `bytes[${string}]`]: undefined;
            [x: `bytes2[${string}]`]: undefined;
            [x: `bytes1[${string}]`]: undefined;
            [x: `bytes3[${string}]`]: undefined;
            [x: `bytes6[${string}]`]: undefined;
            [x: `bytes12[${string}]`]: undefined;
            [x: `bytes4[${string}]`]: undefined;
            [x: `bytes5[${string}]`]: undefined;
            [x: `bytes7[${string}]`]: undefined;
            [x: `bytes8[${string}]`]: undefined;
            [x: `bytes9[${string}]`]: undefined;
            [x: `bytes10[${string}]`]: undefined;
            [x: `bytes11[${string}]`]: undefined;
            [x: `bytes13[${string}]`]: undefined;
            [x: `bytes14[${string}]`]: undefined;
            [x: `bytes15[${string}]`]: undefined;
            [x: `bytes16[${string}]`]: undefined;
            [x: `bytes17[${string}]`]: undefined;
            [x: `bytes18[${string}]`]: undefined;
            [x: `bytes19[${string}]`]: undefined;
            [x: `bytes20[${string}]`]: undefined;
            [x: `bytes21[${string}]`]: undefined;
            [x: `bytes22[${string}]`]: undefined;
            [x: `bytes23[${string}]`]: undefined;
            [x: `bytes24[${string}]`]: undefined;
            [x: `bytes25[${string}]`]: undefined;
            [x: `bytes26[${string}]`]: undefined;
            [x: `bytes27[${string}]`]: undefined;
            [x: `bytes28[${string}]`]: undefined;
            [x: `bytes29[${string}]`]: undefined;
            [x: `bytes30[${string}]`]: undefined;
            [x: `bytes31[${string}]`]: undefined;
            [x: `bytes32[${string}]`]: undefined;
            [x: `int[${string}]`]: undefined;
            [x: `int200[${string}]`]: undefined;
            [x: `int8[${string}]`]: undefined;
            [x: `int16[${string}]`]: undefined;
            [x: `int24[${string}]`]: undefined;
            [x: `int32[${string}]`]: undefined;
            [x: `int40[${string}]`]: undefined;
            [x: `int48[${string}]`]: undefined;
            [x: `int56[${string}]`]: undefined;
            [x: `int64[${string}]`]: undefined;
            [x: `int72[${string}]`]: undefined;
            [x: `int80[${string}]`]: undefined;
            [x: `int88[${string}]`]: undefined;
            [x: `int96[${string}]`]: undefined;
            [x: `int104[${string}]`]: undefined;
            [x: `int112[${string}]`]: undefined;
            [x: `int120[${string}]`]: undefined;
            [x: `int128[${string}]`]: undefined;
            [x: `int136[${string}]`]: undefined;
            [x: `int144[${string}]`]: undefined;
            [x: `int152[${string}]`]: undefined;
            [x: `int160[${string}]`]: undefined;
            [x: `int168[${string}]`]: undefined;
            [x: `int176[${string}]`]: undefined;
            [x: `int184[${string}]`]: undefined;
            [x: `int192[${string}]`]: undefined;
            [x: `int208[${string}]`]: undefined;
            [x: `int216[${string}]`]: undefined;
            [x: `int224[${string}]`]: undefined;
            [x: `int232[${string}]`]: undefined;
            [x: `int240[${string}]`]: undefined;
            [x: `int248[${string}]`]: undefined;
            [x: `int256[${string}]`]: undefined;
            [x: `uint[${string}]`]: undefined;
            [x: `uint200[${string}]`]: undefined;
            [x: `uint8[${string}]`]: undefined;
            [x: `uint16[${string}]`]: undefined;
            [x: `uint24[${string}]`]: undefined;
            [x: `uint32[${string}]`]: undefined;
            [x: `uint40[${string}]`]: undefined;
            [x: `uint48[${string}]`]: undefined;
            [x: `uint56[${string}]`]: undefined;
            [x: `uint64[${string}]`]: undefined;
            [x: `uint72[${string}]`]: undefined;
            [x: `uint80[${string}]`]: undefined;
            [x: `uint88[${string}]`]: undefined;
            [x: `uint96[${string}]`]: undefined;
            [x: `uint104[${string}]`]: undefined;
            [x: `uint112[${string}]`]: undefined;
            [x: `uint120[${string}]`]: undefined;
            [x: `uint128[${string}]`]: undefined;
            [x: `uint136[${string}]`]: undefined;
            [x: `uint144[${string}]`]: undefined;
            [x: `uint152[${string}]`]: undefined;
            [x: `uint160[${string}]`]: undefined;
            [x: `uint168[${string}]`]: undefined;
            [x: `uint176[${string}]`]: undefined;
            [x: `uint184[${string}]`]: undefined;
            [x: `uint192[${string}]`]: undefined;
            [x: `uint208[${string}]`]: undefined;
            [x: `uint216[${string}]`]: undefined;
            [x: `uint224[${string}]`]: undefined;
            [x: `uint232[${string}]`]: undefined;
            [x: `uint240[${string}]`]: undefined;
            [x: `uint248[${string}]`]: undefined;
            [x: `uint256[${string}]`]: undefined;
            string?: undefined;
            address?: undefined;
            bool?: undefined;
            bytes?: undefined;
            bytes2?: undefined;
            bytes1?: undefined;
            bytes3?: undefined;
            bytes6?: undefined;
            bytes12?: undefined;
            bytes4?: undefined;
            bytes5?: undefined;
            bytes7?: undefined;
            bytes8?: undefined;
            bytes9?: undefined;
            bytes10?: undefined;
            bytes11?: undefined;
            bytes13?: undefined;
            bytes14?: undefined;
            bytes15?: undefined;
            bytes16?: undefined;
            bytes17?: undefined;
            bytes18?: undefined;
            bytes19?: undefined;
            bytes20?: undefined;
            bytes21?: undefined;
            bytes22?: undefined;
            bytes23?: undefined;
            bytes24?: undefined;
            bytes25?: undefined;
            bytes26?: undefined;
            bytes27?: undefined;
            bytes28?: undefined;
            bytes29?: undefined;
            bytes30?: undefined;
            bytes31?: undefined;
            bytes32?: undefined;
            int200?: undefined;
            int8?: undefined;
            int16?: undefined;
            int24?: undefined;
            int32?: undefined;
            int40?: undefined;
            int48?: undefined;
            int56?: undefined;
            int64?: undefined;
            int72?: undefined;
            int80?: undefined;
            int88?: undefined;
            int96?: undefined;
            int104?: undefined;
            int112?: undefined;
            int120?: undefined;
            int128?: undefined;
            int136?: undefined;
            int144?: undefined;
            int152?: undefined;
            int160?: undefined;
            int168?: undefined;
            int176?: undefined;
            int184?: undefined;
            int192?: undefined;
            int208?: undefined;
            int216?: undefined;
            int224?: undefined;
            int232?: undefined;
            int240?: undefined;
            int248?: undefined;
            int256?: undefined;
            uint200?: undefined;
            uint8?: undefined;
            uint16?: undefined;
            uint24?: undefined;
            uint32?: undefined;
            uint40?: undefined;
            uint48?: undefined;
            uint56?: undefined;
            uint64?: undefined;
            uint72?: undefined;
            uint80?: undefined;
            uint88?: undefined;
            uint96?: undefined;
            uint104?: undefined;
            uint112?: undefined;
            uint120?: undefined;
            uint128?: undefined;
            uint136?: undefined;
            uint144?: undefined;
            uint152?: undefined;
            uint160?: undefined;
            uint168?: undefined;
            uint176?: undefined;
            uint184?: undefined;
            uint192?: undefined;
            uint208?: undefined;
            uint216?: undefined;
            uint224?: undefined;
            uint232?: undefined;
            uint240?: undefined;
            uint248?: undefined;
            uint256?: undefined;
        };
        primaryType: string;
    };
    chainId: string;
}>;
type EthSignTypedDataV4Action = z.infer<typeof ethSignTypedDataV4ActionSchema>;
declare const walletActionRequestSchema: z.ZodObject<{
    jsonrpc: z.ZodLiteral<"2.0">;
    id: z.ZodString;
    method: z.ZodLiteral<"fc_requestWalletAction">;
    params: z.ZodObject<{
        action: z.ZodUnion<[z.ZodObject<{
            chainId: z.ZodString;
            method: z.ZodLiteral<"eth_sendTransaction">;
            attribution: z.ZodOptional<z.ZodBoolean>;
            params: z.ZodObject<{
                abi: z.ZodType<Abi, z.ZodTypeDef, Abi>;
                to: z.ZodType<`0x${string}`, z.ZodTypeDef, `0x${string}`>;
                value: z.ZodOptional<z.ZodString>;
                data: z.ZodOptional<z.ZodType<`0x${string}`, z.ZodTypeDef, `0x${string}`>>;
            }, "strip", z.ZodTypeAny, {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            }, {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            }>;
        }, "strip", z.ZodTypeAny, {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        }, {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        }>, z.ZodObject<{
            chainId: z.ZodString;
            method: z.ZodLiteral<"eth_signTypedData_v4">;
            params: z.ZodObject<{
                domain: z.ZodType<TypedDataDomain, z.ZodTypeDef, TypedDataDomain>;
                types: z.ZodType<{
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                }, z.ZodTypeDef, {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                }>;
                primaryType: z.ZodString;
                message: z.ZodRecord<z.ZodString, z.ZodUnknown>;
            }, "strip", z.ZodTypeAny, {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            }, {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            }>;
        }, "strip", z.ZodTypeAny, {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        }, {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        }>]>;
    }, "strip", z.ZodTypeAny, {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    }, {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    }>;
}, "strip", z.ZodTypeAny, {
    id: string;
    method: "fc_requestWalletAction";
    params: {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    };
    jsonrpc: "2.0";
}, {
    id: string;
    method: "fc_requestWalletAction";
    params: {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    };
    jsonrpc: "2.0";
}>;
type RequestWalletActionMessage = z.infer<typeof walletActionRequestSchema>;
declare const miniAppMessageSchema: z.ZodUnion<[z.ZodObject<{
    type: z.ZodLiteral<"createCast">;
    data: z.ZodObject<{
        cast: z.ZodObject<{
            parent: z.ZodOptional<z.ZodString>;
            text: z.ZodString;
            embeds: z.ZodArray<z.ZodString, "many">;
        }, "strip", z.ZodTypeAny, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }>;
    }, "strip", z.ZodTypeAny, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }>;
}, "strip", z.ZodTypeAny, {
    type: "createCast";
    data: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
}, {
    type: "createCast";
    data: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
}>, z.ZodObject<{
    jsonrpc: z.ZodLiteral<"2.0">;
    id: z.ZodString;
    method: z.ZodLiteral<"fc_requestWalletAction">;
    params: z.ZodObject<{
        action: z.ZodUnion<[z.ZodObject<{
            chainId: z.ZodString;
            method: z.ZodLiteral<"eth_sendTransaction">;
            attribution: z.ZodOptional<z.ZodBoolean>;
            params: z.ZodObject<{
                abi: z.ZodType<Abi, z.ZodTypeDef, Abi>;
                to: z.ZodType<`0x${string}`, z.ZodTypeDef, `0x${string}`>;
                value: z.ZodOptional<z.ZodString>;
                data: z.ZodOptional<z.ZodType<`0x${string}`, z.ZodTypeDef, `0x${string}`>>;
            }, "strip", z.ZodTypeAny, {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            }, {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            }>;
        }, "strip", z.ZodTypeAny, {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        }, {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        }>, z.ZodObject<{
            chainId: z.ZodString;
            method: z.ZodLiteral<"eth_signTypedData_v4">;
            params: z.ZodObject<{
                domain: z.ZodType<TypedDataDomain, z.ZodTypeDef, TypedDataDomain>;
                types: z.ZodType<{
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                }, z.ZodTypeDef, {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                }>;
                primaryType: z.ZodString;
                message: z.ZodRecord<z.ZodString, z.ZodUnknown>;
            }, "strip", z.ZodTypeAny, {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            }, {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            }>;
        }, "strip", z.ZodTypeAny, {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        }, {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        }>]>;
    }, "strip", z.ZodTypeAny, {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    }, {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    }>;
}, "strip", z.ZodTypeAny, {
    id: string;
    method: "fc_requestWalletAction";
    params: {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    };
    jsonrpc: "2.0";
}, {
    id: string;
    method: "fc_requestWalletAction";
    params: {
        action: {
            method: "eth_sendTransaction";
            params: {
                abi: Abi;
                to: `0x${string}`;
                data?: `0x${string}` | undefined;
                value?: string | undefined;
            };
            chainId: string;
            attribution?: boolean | undefined;
        } | {
            method: "eth_signTypedData_v4";
            params: {
                message: Record<string, unknown>;
                domain: TypedDataDomain;
                types: {
                    [x: string]: readonly viem.TypedDataParameter[];
                    [x: `string[${string}]`]: undefined;
                    [x: `function[${string}]`]: undefined;
                    [x: `address[${string}]`]: undefined;
                    [x: `bool[${string}]`]: undefined;
                    [x: `bytes[${string}]`]: undefined;
                    [x: `bytes2[${string}]`]: undefined;
                    [x: `bytes1[${string}]`]: undefined;
                    [x: `bytes3[${string}]`]: undefined;
                    [x: `bytes6[${string}]`]: undefined;
                    [x: `bytes12[${string}]`]: undefined;
                    [x: `bytes4[${string}]`]: undefined;
                    [x: `bytes5[${string}]`]: undefined;
                    [x: `bytes7[${string}]`]: undefined;
                    [x: `bytes8[${string}]`]: undefined;
                    [x: `bytes9[${string}]`]: undefined;
                    [x: `bytes10[${string}]`]: undefined;
                    [x: `bytes11[${string}]`]: undefined;
                    [x: `bytes13[${string}]`]: undefined;
                    [x: `bytes14[${string}]`]: undefined;
                    [x: `bytes15[${string}]`]: undefined;
                    [x: `bytes16[${string}]`]: undefined;
                    [x: `bytes17[${string}]`]: undefined;
                    [x: `bytes18[${string}]`]: undefined;
                    [x: `bytes19[${string}]`]: undefined;
                    [x: `bytes20[${string}]`]: undefined;
                    [x: `bytes21[${string}]`]: undefined;
                    [x: `bytes22[${string}]`]: undefined;
                    [x: `bytes23[${string}]`]: undefined;
                    [x: `bytes24[${string}]`]: undefined;
                    [x: `bytes25[${string}]`]: undefined;
                    [x: `bytes26[${string}]`]: undefined;
                    [x: `bytes27[${string}]`]: undefined;
                    [x: `bytes28[${string}]`]: undefined;
                    [x: `bytes29[${string}]`]: undefined;
                    [x: `bytes30[${string}]`]: undefined;
                    [x: `bytes31[${string}]`]: undefined;
                    [x: `bytes32[${string}]`]: undefined;
                    [x: `int[${string}]`]: undefined;
                    [x: `int200[${string}]`]: undefined;
                    [x: `int8[${string}]`]: undefined;
                    [x: `int16[${string}]`]: undefined;
                    [x: `int24[${string}]`]: undefined;
                    [x: `int32[${string}]`]: undefined;
                    [x: `int40[${string}]`]: undefined;
                    [x: `int48[${string}]`]: undefined;
                    [x: `int56[${string}]`]: undefined;
                    [x: `int64[${string}]`]: undefined;
                    [x: `int72[${string}]`]: undefined;
                    [x: `int80[${string}]`]: undefined;
                    [x: `int88[${string}]`]: undefined;
                    [x: `int96[${string}]`]: undefined;
                    [x: `int104[${string}]`]: undefined;
                    [x: `int112[${string}]`]: undefined;
                    [x: `int120[${string}]`]: undefined;
                    [x: `int128[${string}]`]: undefined;
                    [x: `int136[${string}]`]: undefined;
                    [x: `int144[${string}]`]: undefined;
                    [x: `int152[${string}]`]: undefined;
                    [x: `int160[${string}]`]: undefined;
                    [x: `int168[${string}]`]: undefined;
                    [x: `int176[${string}]`]: undefined;
                    [x: `int184[${string}]`]: undefined;
                    [x: `int192[${string}]`]: undefined;
                    [x: `int208[${string}]`]: undefined;
                    [x: `int216[${string}]`]: undefined;
                    [x: `int224[${string}]`]: undefined;
                    [x: `int232[${string}]`]: undefined;
                    [x: `int240[${string}]`]: undefined;
                    [x: `int248[${string}]`]: undefined;
                    [x: `int256[${string}]`]: undefined;
                    [x: `uint[${string}]`]: undefined;
                    [x: `uint200[${string}]`]: undefined;
                    [x: `uint8[${string}]`]: undefined;
                    [x: `uint16[${string}]`]: undefined;
                    [x: `uint24[${string}]`]: undefined;
                    [x: `uint32[${string}]`]: undefined;
                    [x: `uint40[${string}]`]: undefined;
                    [x: `uint48[${string}]`]: undefined;
                    [x: `uint56[${string}]`]: undefined;
                    [x: `uint64[${string}]`]: undefined;
                    [x: `uint72[${string}]`]: undefined;
                    [x: `uint80[${string}]`]: undefined;
                    [x: `uint88[${string}]`]: undefined;
                    [x: `uint96[${string}]`]: undefined;
                    [x: `uint104[${string}]`]: undefined;
                    [x: `uint112[${string}]`]: undefined;
                    [x: `uint120[${string}]`]: undefined;
                    [x: `uint128[${string}]`]: undefined;
                    [x: `uint136[${string}]`]: undefined;
                    [x: `uint144[${string}]`]: undefined;
                    [x: `uint152[${string}]`]: undefined;
                    [x: `uint160[${string}]`]: undefined;
                    [x: `uint168[${string}]`]: undefined;
                    [x: `uint176[${string}]`]: undefined;
                    [x: `uint184[${string}]`]: undefined;
                    [x: `uint192[${string}]`]: undefined;
                    [x: `uint208[${string}]`]: undefined;
                    [x: `uint216[${string}]`]: undefined;
                    [x: `uint224[${string}]`]: undefined;
                    [x: `uint232[${string}]`]: undefined;
                    [x: `uint240[${string}]`]: undefined;
                    [x: `uint248[${string}]`]: undefined;
                    [x: `uint256[${string}]`]: undefined;
                    string?: undefined;
                    address?: undefined;
                    bool?: undefined;
                    bytes?: undefined;
                    bytes2?: undefined;
                    bytes1?: undefined;
                    bytes3?: undefined;
                    bytes6?: undefined;
                    bytes12?: undefined;
                    bytes4?: undefined;
                    bytes5?: undefined;
                    bytes7?: undefined;
                    bytes8?: undefined;
                    bytes9?: undefined;
                    bytes10?: undefined;
                    bytes11?: undefined;
                    bytes13?: undefined;
                    bytes14?: undefined;
                    bytes15?: undefined;
                    bytes16?: undefined;
                    bytes17?: undefined;
                    bytes18?: undefined;
                    bytes19?: undefined;
                    bytes20?: undefined;
                    bytes21?: undefined;
                    bytes22?: undefined;
                    bytes23?: undefined;
                    bytes24?: undefined;
                    bytes25?: undefined;
                    bytes26?: undefined;
                    bytes27?: undefined;
                    bytes28?: undefined;
                    bytes29?: undefined;
                    bytes30?: undefined;
                    bytes31?: undefined;
                    bytes32?: undefined;
                    int200?: undefined;
                    int8?: undefined;
                    int16?: undefined;
                    int24?: undefined;
                    int32?: undefined;
                    int40?: undefined;
                    int48?: undefined;
                    int56?: undefined;
                    int64?: undefined;
                    int72?: undefined;
                    int80?: undefined;
                    int88?: undefined;
                    int96?: undefined;
                    int104?: undefined;
                    int112?: undefined;
                    int120?: undefined;
                    int128?: undefined;
                    int136?: undefined;
                    int144?: undefined;
                    int152?: undefined;
                    int160?: undefined;
                    int168?: undefined;
                    int176?: undefined;
                    int184?: undefined;
                    int192?: undefined;
                    int208?: undefined;
                    int216?: undefined;
                    int224?: undefined;
                    int232?: undefined;
                    int240?: undefined;
                    int248?: undefined;
                    int256?: undefined;
                    uint200?: undefined;
                    uint8?: undefined;
                    uint16?: undefined;
                    uint24?: undefined;
                    uint32?: undefined;
                    uint40?: undefined;
                    uint48?: undefined;
                    uint56?: undefined;
                    uint64?: undefined;
                    uint72?: undefined;
                    uint80?: undefined;
                    uint88?: undefined;
                    uint96?: undefined;
                    uint104?: undefined;
                    uint112?: undefined;
                    uint120?: undefined;
                    uint128?: undefined;
                    uint136?: undefined;
                    uint144?: undefined;
                    uint152?: undefined;
                    uint160?: undefined;
                    uint168?: undefined;
                    uint176?: undefined;
                    uint184?: undefined;
                    uint192?: undefined;
                    uint208?: undefined;
                    uint216?: undefined;
                    uint224?: undefined;
                    uint232?: undefined;
                    uint240?: undefined;
                    uint248?: undefined;
                    uint256?: undefined;
                };
                primaryType: string;
            };
            chainId: string;
        };
    };
    jsonrpc: "2.0";
}>, z.ZodObject<{
    jsonrpc: z.ZodLiteral<"2.0">;
    id: z.ZodUnion<[z.ZodString, z.ZodNumber, z.ZodNull]>;
    method: z.ZodLiteral<"fc_createCast">;
    params: z.ZodObject<{
        cast: z.ZodObject<{
            parent: z.ZodOptional<z.ZodString>;
            text: z.ZodString;
            embeds: z.ZodArray<z.ZodString, "many">;
        }, "strip", z.ZodTypeAny, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }, {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        }>;
    }, "strip", z.ZodTypeAny, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }, {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    }>;
}, "strip", z.ZodTypeAny, {
    id: string | number | null;
    method: "fc_createCast";
    params: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
    jsonrpc: "2.0";
}, {
    id: string | number | null;
    method: "fc_createCast";
    params: {
        cast: {
            text: string;
            embeds: string[];
            parent?: string | undefined;
        };
    };
    jsonrpc: "2.0";
}>]>;
type MiniAppMessage = z.infer<typeof miniAppMessageSchema>;

export { type CreateCastLegacyMessage, type CreateCastMessage, type CreateCastResponse, type EthSendTransactionAction, type EthSendTransactionSuccessBody, type EthSignTypedDataV4Action, type EthSignTypedDataV4SuccessBody, type MiniAppMessage, type MiniAppResponse, type RequestWalletActionMessage, type TransactionResponse, type TransactionResponseFailure, type TransactionResponseSuccess, type TransactionSuccessBody, miniAppMessageSchema };
